#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_browsetable import browsetable
from task_browsetable import casalog

class browsetable_pg_:
    __name__ = "browsetable"

    def __init__(self) :
       self.__bases__ = (browsetable_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, tablename=None,mightedit=None,sortlist=None,taql=None,skipcols=None,):

        """Browse a table (MS, calibration table, image)
    This task brings up a browser that can open and display any CASA table.
    The tablename can be specified at startup, or any table can be loaded after
    the browser comes up.
            
    Parameters:
    tablename -- Name of table file on disk (vis, calibration table, image)
                 default: none; example: tablename='ngc5921.ms'
    mightedit -- If True disable the filtering options (below) and allow
                 editing the table.  Warning: the GUI appears to ignore
                 whether the table tool is opened read-only - just be
                 aware that you should not edit filtered tables unless
                 you know what you are doing.
    sortlist  -- List of columns to sort by.
                 default: [] (none)
    taql      -- TaQL query string for prefiltering the table.
                 default: "" (none); example: taql="ANTENNA2 < 6"
    skipcols  -- Columns to NOT display.
                 default: [] (none); example: skipcols='feed1, feed2'    
            
    
    Some comments on using browsetable (see cookbook also):
    
     Most often you will browse a measurement set.  Either specify the vis name
     as the tablename, or when the browser comes up,
    
            click on <file> (upper left), then click on <open table>
    
     If you want to look at sub-tables, use the tab table keywords along the
     left side to bring up a panel with the sub-tables listed (Fig 3.8), then
     choose (left-click) a table and View.
    
     Note that one useful feature is that you can Edit any table and its
     contents. Use the Edit tab (to the right of the file tab).  Be careful
     with this, and make a backup copy of the table before editing!
    
     Use the Close Tables and Exit option from the Files menu to quit the
     casabrowser.
    
     To get a plot of two table values, click on tools, then click on plot 2D.
     For example, to get a u-v plot, in the Plotter Option Gui,
          set Rows:  0   to  <Large Number>
          X Axis:  UVW      Slice  (set 0)
          Y Axis:  UVW      Slice  (set 1)
          click 'Clear and Plot' on right.
    
     For visibility plots
          X Axis:  TIME
          Y Axis:  DATA     Slice Amplitude
          click 'Clear and Plot' on right.
       
       

        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'browsetable'
        myf['taskname'] = 'browsetable'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['tablename'] = tablename = myf['tablename']
            myparams['mightedit'] = mightedit = myf['mightedit']
            myparams['sortlist'] = sortlist = myf['sortlist']
            myparams['taql'] = taql = myf['taql']
            myparams['skipcols'] = skipcols = myf['skipcols']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['tablename'] = tablename
        mytmp['mightedit'] = mightedit
        mytmp['sortlist'] = sortlist
        mytmp['taql'] = taql
        mytmp['skipcols'] = skipcols
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'browsetable.xml')

        casalog.origin('browsetable')
        if not trec.has_key('browsetable') or not casac.utils().verify(mytmp, trec['browsetable']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: browsetable           #####')
          casalog.post('')
          result = browsetable(tablename, mightedit, sortlist, taql, skipcols)
          casalog.post('')
          casalog.post('##### End Task: browsetable           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('browsetable', 'browsetable.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'browsetable'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('browsetable', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['tablename']  = ''

        a['tablename'] = {
                    0:odict([{'notvalue':''}, {'mightedit':False}, {'sortlist':''}, {'taql':''}, {'skipcols':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='browsetable', subkey=None):
        desc={'browsetable': 'Browse a table (MS, calibration table, image)',
               'tablename': 'Name of input table',
               'mightedit': 'Warning: the GUI seems to ignore whether the table tool is opened read-only - just be careful, esp. if filtering.',
               'sortlist': 'Columns to sort by (ascending)',
               'taql': 'TaQL query string for prefiltering the table.',
               'skipcols': 'Columns to omit',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['tablename']  = ''
        a['mightedit']  = False
        a['sortlist']  = ''
        a['taql']  = ''
        a['skipcols']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
browsetable_pg = browsetable_pg_()
