#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_feather import feather
from task_feather import casalog

class feather_pg_:
    __name__ = "feather"

    def __init__(self) :
       self.__bases__ = (feather_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None,highres=None,lowres=None,sdfactor=None,effdishdiam=None,lowpassfiltersd=None,):

        """Combine two images using their Fourier transforms

	Feathering is a simple method for combining two images with different 
        spatial resolution.  The processing steps are:

	       1. Regrid the low-resolution image to a temporary copy matching the
	           resolution of the high-resolution image,
               2. Transform each image to the spatial-frequency plane (gridded).
               3. Scale the low-resolution image (uv-grid) by the ratio of the
                   volumes of the two 'clean beams' (high-res/low-res). 
               4. Add to this, the uv-grid of the high-resolution image, scaled by
                   (1-wt) where 'wt' is the Fourier transform of the 'clean beam'
                   defined in the low-resolution image.
               5. Transform back to the image plane.

	Both input images must have a well-defined beam shape for this task to work. 
	This could be a 'clean beam' for interferometric images, and a 'primary-beam' 
        for a single-dish image. 

        The two images must also have the same flux density normalization scale.

	Keyword arguments:
        imagename -- Name of output feathered image
                default: none; example: imagename='orion_combined.im'
        highres -- Name of high resolution (interferometer) image
                default: none; example: highres='orion_vla.im'
	     This image is often a clean image obtained from synthesis
		observations.
        lowres -- Name of low resolution (single dish) image
                default: none; example: lowres='orion_gbt.im'
	     This image is often a image from a single-dish observations
	        or a clean image obtained from lower resolution synthesis
		observations.
         sdfactor -- value by which to scale the Single Dish image. Default is 1.0
                         Basically modifying the flux scale of the SD image
	 effdishdiam -- New effective SingleDish diameter to use in m. Obviously one can only reduce the dish effective dish diameter in feathering. Default -1.0 means leave as is.
	 lowpassfiltersd -- If True the high spatial frequency in the SD image is rejected. 
                                   Any data outside the maximum uv distance that the SD 
                                   has illuminated  is filtered out.


        Comments:

        This task can be used as one method of combining single-dish and 
        interferometric images after they have been separately made.
        
        The clean task allows another method of combining single-dish and
        interferometric data. The single-dish image can be used as a starting 
        model for the interferometric image-reconstruction. If there is
        some overlap between the spatial-frequencies contained in the 
        single-dish image and the interferometer sampling function, then,
        such a starting model will help constrain the solutions on the 
        short-baselines of the interferometric data.
        


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'feather'
        myf['taskname'] = 'feather'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['highres'] = highres = myf['highres']
            myparams['lowres'] = lowres = myf['lowres']
            myparams['sdfactor'] = sdfactor = myf['sdfactor']
            myparams['effdishdiam'] = effdishdiam = myf['effdishdiam']
            myparams['lowpassfiltersd'] = lowpassfiltersd = myf['lowpassfiltersd']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['highres'] = highres
        mytmp['lowres'] = lowres
        mytmp['sdfactor'] = sdfactor
        mytmp['effdishdiam'] = effdishdiam
        mytmp['lowpassfiltersd'] = lowpassfiltersd
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'feather.xml')

        casalog.origin('feather')
        if not trec.has_key('feather') or not casac.utils().verify(mytmp, trec['feather']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: feather           #####')
          casalog.post('')
          result = feather(imagename, highres, lowres, sdfactor, effdishdiam, lowpassfiltersd)
          casalog.post('')
          casalog.post('##### End Task: feather           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('feather', 'feather.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'feather'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('feather', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['highres']  = ''
        a['lowres']  = ''
        a['sdfactor']  = 1.0
        a['effdishdiam']  = -1.0
        a['lowpassfiltersd']  = False


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='feather', subkey=None):
        desc={'feather': 'Combine two images using their Fourier transforms',
               'imagename': 'Name of output feathered image',
               'highres': 'Name of high resolution (interferometer) image',
               'lowres': 'Name of low resolution (single dish) image',
               'sdfactor': 'Scale factor to apply to Single Dish image',
               'effdishdiam': 'New effective SingleDish diameter to use in m ',
               'lowpassfiltersd': 'Filter out the high spatial frequencies of the SD image',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['highres']  = ''
        a['lowres']  = ''
        a['sdfactor']  = 1.0
        a['effdishdiam']  = -1.0
        a['lowpassfiltersd']  = False

        if a.has_key(paramname) :
	      return a[paramname]
feather_pg = feather_pg_()
