#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_importvla import importvla
from task_importvla import casalog

class importvla_pg_:
    __name__ = "importvla"

    def __init__(self) :
       self.__bases__ = (importvla_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, archivefiles=None,vis=None,bandname=None,frequencytol=None,project=None,starttime=None,stoptime=None,applytsys=None,autocorr=None,antnamescheme=None,keepblanks=None,evlabands=None,):

        """Import VLA archive file(s) to a measurement set

      Imports an arbitrary number of VLA archive-format data sets into
      a casa measurement set.  If more than one band is present, they
      will be put in the same measurement set but in a separate spectral
      window.  The task will handle old style and new style VLA (after
      July 2007) archive data and apply the tsys to the data and to
      the weights.

      Keyword arguments:
      archivefiles -- Name of input VLA archive file(s)
              default: none.  Must be supplied
              example: archivefiles = 'AP314_A959519.xp1'
              example: archivefiles=['AP314_A950519.xp1','AP314_A950519.xp2']
      vis -- Name of output visibility file
              default: none.  Must be supplied.
              example: vis='NGC7538.ms'
              Will not over-write existing ms of same name.
              A backup flag-file version 'Original' will be made in
                vis.flagversions.  See help flagmanager
      bandname -- VLA Frequency band
              default: => '' = all bands
              example: bandname='K'
              Options: '4'=48-96 MHz,'P'=298-345 MHz,'L'=1.15-1.75 GHz,
              'C'=4.2-5.1 GHz,'X'=6.8-9.6 GHz,'U'=13.5-16.3 GHz,
              'K'=20.8-25.8 GHz,'Q'=38-51 GHz
      frequencytol -- Tolerance in frequency shift in making spectral windows
              default: => 150000 (Hz).  For Doppler shifted data, <10000 Hz may
              may produce too many unnecessary spectral windows.
              example: frequencytol = 1500000.0 (units = Hz)
      project -- Project name to import from archive files:
              default: '' => all projects in file
              example: project='AL519'
              project = 'al519' or AL519 will work.  Do not include
              leading zeros; project = 'AL0519' will not work.
      starttime -- Time after which data will be considered for importing
              default: '' => all:  Date must be included.
              syntax: starttime = '2003/1/31/05:05:23'
      stoptime --  Time before which data will be considered for importing
              default: '' => all:  Date must be included.
              syntax: stoptime = '2003/1/31/08:05:23'
      applytsys -- Apply data scaling and weight scaling by nominal
              sensitivity (~Tsys)
              default: True.  Strongly recommended
      autocorr --  import autocorrelations to ms
              default:  => False (no autocorrelations)
      antnamescheme -- 'old' or 'new' antenna names.
              default => 'new' gives antnenna names
                'VA04' or 'EA13 for VLA telescopse 04 and 13 (EVLA)
                'old' gives names '04' or '13'
      keepblanks -- Should sources with blank names be filled into the data base
              default => false.  Do not fill
              These scans are tipping scans (as of June 1, 2009) and should not
              be filled in the visibility data set.
      evlabands -- Use the EVLA's center frequency and bandwidths for frequencies
              specified via wavelength or band. 
	      default => True. 
      async --  Run asynchronously
              default = False; do not run asychronously
      
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'importvla'
        myf['taskname'] = 'importvla'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['archivefiles'] = archivefiles = myf['archivefiles']
            myparams['vis'] = vis = myf['vis']
            myparams['bandname'] = bandname = myf['bandname']
            myparams['frequencytol'] = frequencytol = myf['frequencytol']
            myparams['project'] = project = myf['project']
            myparams['starttime'] = starttime = myf['starttime']
            myparams['stoptime'] = stoptime = myf['stoptime']
            myparams['applytsys'] = applytsys = myf['applytsys']
            myparams['autocorr'] = autocorr = myf['autocorr']
            myparams['antnamescheme'] = antnamescheme = myf['antnamescheme']
            myparams['keepblanks'] = keepblanks = myf['keepblanks']
            myparams['evlabands'] = evlabands = myf['evlabands']

        if type(archivefiles)==str: archivefiles=[archivefiles]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['archivefiles'] = archivefiles
        mytmp['vis'] = vis
        mytmp['bandname'] = bandname
        if type(frequencytol) == str :
           mytmp['frequencytol'] = casac.quanta().quantity(frequencytol)
        else :
           mytmp['frequencytol'] = frequencytol
        mytmp['project'] = project
        mytmp['starttime'] = starttime
        mytmp['stoptime'] = stoptime
        mytmp['applytsys'] = applytsys
        mytmp['autocorr'] = autocorr
        mytmp['antnamescheme'] = antnamescheme
        mytmp['keepblanks'] = keepblanks
        mytmp['evlabands'] = evlabands
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'importvla.xml')

        casalog.origin('importvla')
        if not trec.has_key('importvla') or not casac.utils().verify(mytmp, trec['importvla']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: importvla           #####')
          casalog.post('')
          result = importvla(archivefiles, vis, bandname, frequencytol, project, starttime, stoptime, applytsys, autocorr, antnamescheme, keepblanks, evlabands)
          casalog.post('')
          casalog.post('##### End Task: importvla           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('importvla', 'importvla.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'importvla'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('importvla', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['archivefiles']  = ['']
        a['vis']  = ''
        a['bandname']  = ''
        a['frequencytol']  = '150000.0Hz'
        a['project']  = ''
        a['starttime']  = ''
        a['stoptime']  = ''
        a['applytsys']  = True
        a['autocorr']  = False
        a['antnamescheme']  = 'new'
        a['keepblanks']  = False
        a['evlabands']  = False


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='importvla', subkey=None):
        desc={'importvla': 'Import VLA archive file(s) to a measurement set',
               'archivefiles': 'Name of input VLA archive file(s)',
               'vis': 'Name of output visibility file',
               'bandname': 'VLA frequency band name:\'\'=>obtain all bands in the archive file',
               'frequencytol': 'Frequency shift to define a unique spectra window (Hz)',
               'project': 'Project name: \'\' => all projects in files',
               'starttime': 'start time to search for data',
               'stoptime': 'end time to search for data',
               'applytsys': 'apply nominal sensistivity scaling to data and weights',
               'autocorr': 'import autocorrelations to ms, if set to True',
               'antnamescheme': '\'old\' or \'new\'; \'VA04\' or \'04\' for VLA ant 4',
               'keepblanks': 'Fill scans with blank (empty) source names (e.g. tipping scans)',
               'evlabands': 'Use updated eVLA frequencies and bandwidths for bands and wavelengths',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['archivefiles']  = ['']
        a['vis']  = ''
        a['bandname']  = ''
        a['frequencytol']  = '150000.0Hz'
        a['project']  = ''
        a['starttime']  = ''
        a['stoptime']  = ''
        a['applytsys']  = True
        a['autocorr']  = False
        a['antnamescheme']  = 'new'
        a['keepblanks']  = False
        a['evlabands']  = False

        if a.has_key(paramname) :
	      return a[paramname]
importvla_pg = importvla_pg_()
