#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_imstat import imstat
from task_imstat import casalog

class imstat_pg_:
    __name__ = "imstat"

    def __init__(self) :
       self.__bases__ = (imstat_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, imagename=None,axes=None,region=None,box=None,chans=None,stokes=None,listit=None,verbose=None,mask=None,stretch=None,logfile=None,append=None,):

        """Displays statistical information from an image or image region

     Many parameters are determined from the specified region of an image.
     For this version, the region can be specified by a set of rectangular
     pixel coordinates, the channel ranges and the Stokes.

     For directed output, run as 
                    myoutput = imstat()
   

Keyword arguments:
imagename    Name of input image
		     Default: none; Example: imagename='ngc5921_task.im'
axes         axes to compute statistics over. -1 => all axes.
region       Region of interest. See help par.region.
box          A box region specified in pixels on the directional plane
		     Default: none (whole 2-D plane); 
             Example: box='10,10,50,50'
             box = '10,10,30,30,35,35,50,50' (two boxes)
chans        Zero based channel numbers
	         Range of channel numbers to include in statistics
             See "help par.chans" for examples. 
		     Default:''= all;  Example: chans='3~20'    
stokes       Stokes parameters to analyze.
             Default: all; Example: stokes='IQUV';  
             Example:stokes='I,Q'
             Options: 'I','Q','U','V','RR','RL','LR','LL','XX','YX','XY','YY', ...
listit       Print stats and bounding box to logger? 
verbose      Print additional messages to logger?
mask         Mask to use. See help par.mask. Default is none.
stretch      Stretch the mask if necessary and possible? See help par.stretch 
logfile      Name of file to write fit results.
append       If logfile exists, append to it (True) or overwrite it (False).


      General procedure:

         1.  Specify inputs, then

         2.  myoutput = imstat()
               or specify inputs directly in calling sequence to task
             myoutput = imstat(imagename='image.im', etc)

         3.  myoutput['KEYS'] will contain the result associated with any
               of the keys given below
        
        KEYS CURRENTLY AVAILABLE
    	blc          - absolute PIXEL coordinate of the bottom left corner of 
	               the bounding box surrounding the selected region
    	blcf         - Same as blc, but uses WORLD coordinates instead of pixels
    	trc          - the absolute PIXEL coordinate of the top right corner 
                       of the bounding box surrounding the selected region
    	trcf         - Same as trc, but uses WORLD coordinates instead of pixels
        flux         - the integrated flux density if the beam is defined and 
                       the if brightness units are $Jy/beam$
	npts         - the number of unmasked points used
	max          - the maximum pixel value
    	min          - minimum pixel value
	maxpos       - absolute PIXEL coordinate of maximum pixel value
	maxposf      - Same as maxpos, but uses WORLD coordinates instead of pixels
    	minpos       - absolute pixel coordinate of minimum pixel value
	minposf      - Same as minpos, but uses WORLD coordinates instead of pixels
	sum          - the sum of the pixel values: $\sum I_i$
        sumsq        - the sum of the squares of the pixel values: $\sum I_i^2$
	mean         - the mean of pixel values: 
                       $\bar{I} = \sum I_i / n$
	sigma        - the standard deviation about the mean: 
                       $\sigma^2 = (\sum I_i - \bar{I})^2 / (n-1)$
        rms          - the root mean square: 
                       $\sqrt {\sum I_i^2 / n}$
	median       - the median pixel value
	medabsdevmed - the median of the absolute deviations from the 
                       median
	quartile     - the inter-quartile range. Find the points 
                       which are 25% largest and 75% largest (the median is 
                       50% largest), find their difference and divide that 
                       difference by 2.

	Additional Examples
	# Selected two box region
	# box 1, bottom-left coord is 2,3 and top-right coord is 14,15
	# box 2, bottom-left coord is 30,31 and top-right coord is 42,43
	imstat( 'myImage', box='2,3,14,15;30,31,42,43' )

	# Select the same two box regions but only channels 4 and 5
	imstat( 'myImage', box='2,3,14,15;30,31,42,43', chan='4~5' )

	# Select all channels greater the 20 as well as channel 0.
 	# Then the mean and standard deviation are printed
	results = imstat( 'myImage', chans='>20;0' )
        print "Mean is: ", results['mean'], "  s.d. ", results['sigma']

        # Find statistical information for the Q stokes value only
        # then the I stokes values only, and printing out the statistical
        # values that we are interested in.
	s1 = imstat( 'myimage', stokes='Q' )
	s2 = imstat( 'myimage', stokes='I' )
        print "       |  MIN  |   MAX  | MEAN"
        print "  Q    | ",s1['min'][0],"  |  ",s1['max'][0],"  |  ",,"  |  ",s1['mean'][0]
        print "  I    | ",s2['min'][0],"  |  ",s2['max'][0],"  |  ",,"  |  ",s2['mean'][0]

# evaluate statistics for each spectral plane in an ra x dec x frequency image
ia.fromshape("", [20,30,40])
# give pixels non-zero values
ia.addnoise()
# These are the display axes, the calculation of statistics occurs
# for each (hyper)plane along axes not listed in the axes parameter,
# in this case axis 2 (the frequency axis)
# display the rms for each frequency plane (your mileage will vary with
# the values).
stats = ia.statistics(axes=[0,1])
 stats["rms"]
  Out[10]: 
array([ 0.99576014,  1.03813124,  0.97749186,  0.97587883,  1.04189885,
        1.03784776,  1.03371549,  1.03153074,  1.00841606,  0.947155  ,
        0.97335404,  0.94389403,  1.0010221 ,  0.97151822,  1.03942156,
        1.01158476,  0.96957082,  1.04212773,  1.00589049,  0.98696715,
        1.00451481,  1.02307892,  1.03102005,  0.97334671,  0.95209879,
        1.02088714,  0.96999902,  0.98661619,  1.01039267,  0.96842754,
        0.99464947,  1.01536798,  1.02466023,  0.96956468,  0.98090756,
        0.9835844 ,  0.95698935,  1.05487967,  0.99846411,  0.99634868])



        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'imstat'
        myf['taskname'] = 'imstat'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['imagename'] = imagename = myf['imagename']
            myparams['axes'] = axes = myf['axes']
            myparams['region'] = region = myf['region']
            myparams['box'] = box = myf['box']
            myparams['chans'] = chans = myf['chans']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['listit'] = listit = myf['listit']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['mask'] = mask = myf['mask']
            myparams['stretch'] = stretch = myf['stretch']
            myparams['logfile'] = logfile = myf['logfile']
            myparams['append'] = append = myf['append']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['imagename'] = imagename
        mytmp['axes'] = axes
        mytmp['region'] = region
        mytmp['box'] = box
        mytmp['chans'] = chans
        mytmp['stokes'] = stokes
        mytmp['listit'] = listit
        mytmp['verbose'] = verbose
        mytmp['mask'] = mask
        mytmp['stretch'] = stretch
        mytmp['logfile'] = logfile
        mytmp['append'] = append
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'imstat.xml')

        casalog.origin('imstat')
        if not trec.has_key('imstat') or not casac.utils().verify(mytmp, trec['imstat']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: imstat           #####')
          casalog.post('')
          result = imstat(imagename, axes, region, box, chans, stokes, listit, verbose, mask, stretch, logfile, append)
          casalog.post('')
          casalog.post('##### End Task: imstat           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('imstat', 'imstat.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'imstat'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('imstat', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['imagename']  = ''
        a['axes']  = -1
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['listit']  = True
        a['verbose']  = True
        a['mask']  = ''
        a['logfile']  = ''

        a['mask'] = {
                    0:odict([{'notvalue':''}, {'stretch':False}])}
        a['logfile'] = {
                    0:odict([{'notvalue':''}, {'append':True}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='imstat', subkey=None):
        desc={'imstat': 'Displays statistical information from an image or image region',
               'imagename': 'Name of the input image',
               'axes': 'List of axes to evaluate statistics over. Default is all axes.',
               'region': 'Image Region or name. Use Viewer',
               'box': 'Select one or more box regions',
               'chans': 'Select the channel(spectral) range. See "help par.chans" for examples.',
               'stokes': 'Stokes params to image (I,IV,IQU,IQUV). Default "" => include all',
               'listit': 'Print stats and bounding box to logger?',
               'verbose': 'Print additional messages to logger?',
               'mask': 'Mask to use. See help par.mask. Default is none.',
               'stretch': 'Stretch the mask if necessary and possible? See help par.stretch ',
               'logfile': 'Name of file to write fit results.',
               'append': 'If logfile exists, append to it if True or overwrite it if False',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['imagename']  = ''
        a['axes']  = -1
        a['region']  = ''
        a['box']  = ''
        a['chans']  = ''
        a['stokes']  = ''
        a['listit']  = True
        a['verbose']  = True
        a['mask']  = ''
        a['stretch']  = False
        a['logfile']  = ''
        a['append']  = True

        if a.has_key(paramname) :
	      return a[paramname]
imstat_pg = imstat_pg_()
