#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_listpartition import listpartition
from task_listpartition import casalog

class listpartition_pg_:
    __name__ = "listpartition"

    def __init__(self) :
       self.__bases__ = (listpartition_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,createdict=None,listfile=None,):

        """List the summary of a multi-MS data set in the logger or in a file

       A multi-measurement set (MMS) is an MS that has been split into sub-MSs.
       An MMS contains a reference MS in the top directory and the sub-MSs are 
       located in a directory called SUBMSS inside the MMS directory.
       Example of a MS that was partitioned in the 'scan' axis using the task partition: 

       > ls ngc5921.mms
         ANTENNA           FLAG_CMD     POLARIZATION  SPECTRAL_WINDOW  table.dat
         DATA_DESCRIPTION  HISTORY      PROCESSOR     STATE            table.info
         FEED              OBSERVATION  SORTED_TABLE  SUBMSS           WEATHER
         FIELD             POINTING     SOURCE        SYSCAL

       > ls ngc5921.mms/SUBMSS/
         ngc5921.0000.ms/  ngc5921.0002.ms/  ngc5921.0004.ms/  ngc5921.0006.ms/
         ngc5921.0001.ms/  ngc5921.0003.ms/  ngc5921.0005.ms/
              
       The task lists the following properties of a multi-MS or MS:
       sub-MS name, scan, spw list, list of number of channels per spw, 
       number of rows for each scan and the size in disk. Example of logger output:
       
        Sub-MS          Scan  Spw      Nchan    Nrows   Size  
        ngc5921.0000.ms  1    [0]      [63]     4509    11M
        ngc5921.0001.ms  2    [0]      [63]     1890    6.4M
        ngc5921.0002.ms  3    [0]      [63]     6048    13M
        ngc5921.0003.ms  4    [0]      [63]     756     4.9M
        ngc5921.0004.ms  5    [0]      [63]     1134    6.4M
        ngc5921.0005.ms  6    [0]      [63]     6804    15M
        ngc5921.0006.ms  7    [0]      [63]     1512    6.4M


------- Detailed description of keyword arguments -------
       vis -- Name of multi-MS or normal MS.
              default: ''. 
              example: vis='pScan.mms'

       createdict -- Create and return a dictionary containing scan summaries of each
                     sub-MS. 
              default: False
              
              If set to True, the returned dictionary will contain information from
              ms.getscansummary() and ms.getspectralwindowinfo(), with the addition of an 
              index as the top key and the sub-MS name.
              Example:
              
            {0: {'MS': 'ngc5921.0000.ms',
                 'scanId': {1: {'nchans': array([63], dtype=int32),
                                'nrows': 4509,
                                'spwIds': array([0], dtype=int32)}},
                 'size': '11M'},
             1: {'MS': 'ngc5921.0001.ms',
                 'scanId': {2: {'nchans': array([63], dtype=int32),
                                'nrows': 1890,
                                'spwIds': array([0], dtype=int32)}},
                 'size': '6.4M'}}
                    
       listfile -- Name of ASCII file to save output to. If empty, it will 
                   list on the logger/terminal.
              default: ''
              example: listfile='pscan.txt'

 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'listpartition'
        myf['taskname'] = 'listpartition'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['createdict'] = createdict = myf['createdict']
            myparams['listfile'] = listfile = myf['listfile']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['createdict'] = createdict
        mytmp['listfile'] = listfile
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'listpartition.xml')

        casalog.origin('listpartition')
        if not trec.has_key('listpartition') or not casac.utils().verify(mytmp, trec['listpartition']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: listpartition           #####')
          casalog.post('')
          result = listpartition(vis, createdict, listfile)
          casalog.post('')
          casalog.post('##### End Task: listpartition           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('listpartition', 'listpartition.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'listpartition'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('listpartition', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['createdict']  = False
        a['listfile']  = ''


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='listpartition', subkey=None):
        desc={'listpartition': 'List the summary of a multi-MS data set in the logger or in a file',
               'vis': 'Name of multi-MS or normal MS.',
               'createdict': 'Create and return a dictionary with sub-MS information',
               'listfile': 'Name of ASCII file to save output: \'\'==>to terminal',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['createdict']  = False
        a['listfile']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
listpartition_pg = listpartition_pg_()
