#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_msmoments import msmoments
class msmoments_cli_:
    __name__ = "msmoments"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (msmoments_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'moments':None, 'antenna':None, 'field':None, 'spw':None, 'includemask':None, 'excludemask':None, 'outfile':None, 'overwrite':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, moments=None, antenna=None, field=None, spw=None, includemask=None, excludemask=None, outfile=None, overwrite=None, ):

        """Compute moments from an MS

	Detailed Description: 


	Arguments :
		infile:	Name of the input MS data
		   Default Value: 

		moments:	List of moments you want to compute
		   Default Value: 0

		antenna:	antenna name or id
		   Default Value: 

		field:	field name or id
		   Default Value: 

		spw:	spectral window id
		   Default Value: 

		includemask:	Range of rows to include
		   Default Value: -1

		excludemask:	Range of rows to exclude
		   Default Value: -1

		outfile:	Output file name (or root for multiple moments) 
		   Default Value: 

		overwrite:	Overwrite existing output files
		   Default Value: False

	Returns: void

	Example :

        The spectral moment distributions at each row in input MS are
        determined. Input MS must have FLOAT_DATA column, i.e. 
        autocorrelation data.  
        See the cookbook and User Reference Manual for
        mathematical details.

        The main control of the calculation is given by parameter
        moments:
	
        moments=-1  - mean value of the spectrum
        moments=0   - integrated value of the spectrum
        moments=1   - intensity weighted coordinate;traditionally used to get 
                      'velocity fields'
        moments=2   - intensity weighted dispersion of the coordinate; traditionally
                      used to get "velocity dispersion"
        moments=3   - median of I
        moments=4   - median coordinate
        moments=5   - standard deviation about the mean of the spectrum
        moments=6   - root mean square of the spectrum
        moments=7   - absolute mean deviation of the spectrum
        moments=8   - maximum value of the spectrum
        moments=9   - coordinate of the maximum value of the spectrum
        moments=10  - minimum value of the spectrum
        moments=11  - coordinate of the minimum value of the spectrum

        Note that includemask and excludemask cannot set simultaneously. 

        Keyword arguments:
        infile -- Name of input MS data
                default: none; example: infile="OrionS_rawACSmod"
        moments -- List of moments you would like to compute
                default: 0 (integrated spectrum);example: moments=[0,1]
                see list above
        antenna -- antenna name or id that the user wants to compute moments
                default: '' (all antennae)
        field -- field name or id that the user wants to compute moments
                default: '' (all fields)
        spw -- spectral window id that the user wants to compute moments
                default: '' (all spectral windows)

        includemask -- List of masks to include
                default: [-1] (include all channels); example=[2,100]
        excludemask -- List of masks to exclude
                default: [-1] (don't exclude channels); example=[100,200]
        outfile -- Output MS file name (or root for multiple moments)
                default: '' (input+auto-determined suffix);example: outfile='source_moment'
        overwrite -- Overwrite existing output files
                default: false

        Example for finding the 1-momment, intensity-weighted
        coordinate, often used for finding velocity fields.
        msmoments( infile='mydata', moment=1, outfile='velocityfields' )


        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'msmoments'
        self.__globals__['taskname'] = 'msmoments'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['moments'] = moments = self.parameters['moments']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['includemask'] = includemask = self.parameters['includemask']
            myparams['excludemask'] = excludemask = self.parameters['excludemask']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']

        if type(moments)==int: moments=[moments]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['moments'] = moments
        mytmp['antenna'] = antenna
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['includemask'] = includemask
        mytmp['excludemask'] = excludemask
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'msmoments.xml')

        casalog.origin('msmoments')
	try :
          #if not trec.has_key('msmoments') or not casac.casac.utils().verify(mytmp, trec['msmoments']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['msmoments'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('msmoments', 'msmoments.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'msmoments'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = msmoments(infile, moments, antenna, field, spw, includemask, excludemask, outfile, overwrite)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'msmoments'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('msmoments', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['moments']  = [0]
        a['antenna']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['includemask']  = -1
        a['excludemask']  = -1
        a['outfile']  = ''
        a['overwrite']  = False


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='msmoments', subkey=None):
        desc={'msmoments': 'Compute moments from an MS',
               'infile': 'Name of the input MS data',
               'moments': 'List of moments you want to compute',
               'antenna': 'antenna name or id',
               'field': 'field name or id',
               'spw': 'spectral window id',
               'includemask': 'Range of rows to include',
               'excludemask': 'Range of rows to exclude',
               'outfile': 'Output file name (or root for multiple moments) ',
               'overwrite': 'Overwrite existing output files',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['moments']  = [0]
        a['antenna']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['includemask']  = -1
        a['excludemask']  = -1
        a['outfile']  = ''
        a['overwrite']  = False

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if a.has_key(paramname) :
	      return a[paramname]
msmoments_cli = msmoments_cli_()
