#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_pclean import pclean
from task_pclean import casalog

class pclean_pg_:
    __name__ = "pclean"

    def __init__(self) :
       self.__bases__ = (pclean_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,imagename=None,imsize=None,cell=None,phasecenter=None,stokes=None,mask=None,field=None,spw=None,ftmachine=None,alg=None,scales=None,cyclefactor=None,majorcycles=None,niter=None,gain=None,threshold=None,weighting=None,robust=None,npixels=None,mode=None,nterms=None,start=None,nchan=None,width=None,restfreq=None,interactive=None,npercycle=None,wprojplanes=None,facets=None,overwrite=None,uvtaper=None,outertaper=None,timerange=None,uvrange=None,antenna=None,scan=None,observation=None,pbcor=None,minpb=None,clusterdef=None,):

        """Invert and deconvolve images with parallel engines

	Keyword arguments:
        Invert and deconvolve images with parallel engines
        Form images from visibilities. Handles continuum and spectral line 
        cubes using module pcont and pcube respectively.
 
        vis -- Name of input visibility file
               default: none; example: vis='ngc5921.ms'    
    
	imagename -- Pre-name of output CASA image. (only the prefix)
               default: none; 
               example: imagename='m2', output images are:
                 m2.image; cleaned and restored image
                           With or without primary beam correction
                 m2.psf; point-spread function (dirty beam)
                 m2.model; image of clean components
                 m2.mask; image containing clean regions, when interative=True

        imsize -- Image pixel size (x,y).  DOES NOT HAVE TO BE A POWER OF 2
               default: [256,256]; 
               example: imsize=[350,350]
               imsize=500 is equivalent to imsize=[500, 500]
               Avoid odd-numbered imsize.
    
        cell -- Cell size (x,y)
               default: '1.0arcsec';
               example: cell=['0.5arcsec', '0.5arcsec'] or
                        cell=['1arcmin', '1arcmin']
               cell='1arcsec' is equivalent to cell=['1arcsec', '1arcsec']
               NOTE:cell=2.0 => cell=['2arcsec', '2arcsec']

        phasecenter -- direction measure  or fieldid for the mosaic center
               default: '' => first field selected; 
               example: phasecenter=6
                     or phasecenter='J2000 19h30m00 -40d00m00'
        
        mask -- mask image to be used for CLEANing. As long as the image has
               the same shape (size), mask images from a previous
               interactive session can be used for a new execution.
	       Only an image mask is allowed at this stage. Text formats not allowed yet.

        field -- Select fields in MS.  Use field id(s) or field name(s).
                    ['go listobs' to obtain the list id's or names]
               default: ''= all fields
               If field string is a non-negative integer, it is assumed to
               be a field index otherwise, it is assumed to be a field name
               examples:
                 field='0~2'; field ids 0,1,2
                 field='0,4,5~7'; field ids 0,4,5,6,7
                 field='3C286,3C295'; field named 3C286 and 3C295
                 field = '3,4C*'; field id 3, all names starting with 4C
    
        spw --Select spectral window/channels
               NOTE: This selects the data passed as the INPUT to mode
               default: ''=all spectral windows and channels
               examples:
                 spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
                 spw='0:5~61'; spw 0, channels 5 to 61
                 spw='< 2';   spectral windows less than 2 (i.e. 0,1)
                 spw='0,10,3:3~45'; spw 0,10 all channels, spw 3, 
                                     channels 3 to 45.
                 spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
                 spw='0:0~10;15~60'; spectral window 0 with channels 
                                      0-10,15-60
                 spw='0:0~10,1:20~30,2:1;2;3'; spw 0, channels 0-10,
                       spw 1, channels 20-30, and spw 2, channels, 1,2 and 3

        ftmachine -- Fourier Transform Engine (Gridding method)
               Options: 
                 'ft' (standard interferometric gridding), 
                 'sd' (standard single dish),
                 'mosaic' (grid using PB as convolution function).
                 'wproject' (wprojection gridder to correct for widefield 'w' term errors)
               default: 'ft'
 
        alg -- Deconvolution algorithm
               Options: 'clark', 'hogbom', 'multiscale'
               default: 'multiscale'
                Note : For multi-term wideband imaging (nterms>1), please use alg='multiscale'

	 cyclefactor -- Controls the threshhold at which the
                   deconvolution cycle will pause to degrid and subtract the
                   model from the visibilities (Cotton-Schwab (CS) major cycle).
                   With poor PSFs, reconcile often (cyclefactor=4 or 5) for
                   reliability. 
                   With good PSFs, use cyclefactor = 1.5 to 2.0 for speed. 
                   Note: threshold = cyclefactor * max sidelobe * max residual
 		   default: 1.5; example: cyclefactor=4
		   cyclefactor=0 allows the user to control number of CS major cycle 
         >>>  majorcycles -- integer number of CS major cycles to do 
               default: 1; 
               example: majorcycles=10
    
        niter -- Maximum number iterations,
               if niter=0, then no CLEANing is done ("invert" only)
               default: 500; 
               example: niter=5000

        threshold -- Flux level (residual peak) at which to stop CLEANing
               default: '0.0mJy';
               example: 
                 threshold='2.3mJy'  (always include units)
                 threshold='0.0023Jy'
                 threshold='0.0023Jy/beam' (okay also)
    
        weighting -- Weighting to apply to visibilities:
               Options: 'natural','uniform','briggs', 
                        'superuniform','radial'
               default: 'natural'; 
               example: weighting='uniform';

        scales -- list of scales in pixel for multiscale clean 
               default: [0]
               example: scales=[0, 3, 10]

        mode -- type of image to be generated 
               Options: 'continuum', 'cube'
               default: 'continuum'
               example:
                 mode='cube'; Use with nchan, start, step to specify
                        output image cube.
               NOTE: mode='velocity' or 'channel' or 'frequency'
               are aliased to mode='cube' for backward compatibility 
	       and comfort.

  >>> mode='continuum' expandable parameters
           nterms -- Number of terms in the spectral Taylor polynomial fit.
              default: 1  ( standard multi-frequency-synthesis )
              Note : for nterms>1, please use alg='multiscale'

  >>> mode='cube' expandable parameters
           nchan -- Total number of channels in the output image.
              Example: nchan=100.
              Default: -1; Automatically selects enough channels to cover
              data selected by 'spw' and consistent with 'start' and 'step'
              It is often easiest to leave nchan at the default value.
           start -- First channel, velocity, or frequency.
	      if start is an integer pclean will assume it is the a channel index
	      if start is in units of velocity or frequency it will take it as such
              
	      If the user use the the ms channel as starting pclean will assign 
	      the first channel of the image to the data channel frequency in LSRK 
              of the first 
	      spw selected at the first time seen in the data and the direction of the 
	      source selected. 
	      If the data is not in the LSRK frame the user should be aware that the 
	      data channel indicated may not fall on the first image channel as time goes.

              example:start=5
             start can be in units of frequency or velocity too
	     When velocity units is used it is obvious then that it is referring to the line
	     whose restfrequency is provided by the user or is default one for the source 
	     in the MS/SOURCE table.
	     examples: start='5.0km/s', or start='22.3GHz'.
           width -- Output channel width
              should be in the same units as start 
              default=1; >1 indicates channel averaging
              if start is an integer, width has to be an integer defining the image channel 
              width by the number of channels of first spectral window selected
              example: width=4.
              when start is in frequency or velocity units then the width has to be in the same units.  default=''; 
             
              examples: width='1.0km/s', or width='24.2kHz'.

           
	interactive -- Create a mask interactively or not.
              interactive clean allows the user to build the cleaning
              mask interactively using the viewer.  

              default: False; 
              example: interactive=True
                The viewer will open with the image displayed. Select the
        	region for the mask and double click in the middle of it.

	 >>> npercycle -- Number of iteration in between viewer interactions.
	       default=100

	pbcor -- Output primary beam-corrected image
                If pbcor=False, the final output image is NOT corrected for
                the PB pattern (particularly important for mosaics), and
                therefore is not "flux correct". Correction can also be
                done after the  fact using immath to divide
                <imagename>.image by the <imagename>.flux image. 
 	       default: pbcor=False; output un-corrected image 
 	       example: pbcor=True; output pb-corrected image (masked outside
                        minpb) 

       >>> minpb -- Minimum PB level to use for pb-correction and pb-based masking.
                    default=0.2;
                    example: minpb=0.01 
               When ftmachine is *not* 'mosaic' :
                  minpb is applied to the flux image (sensitivity-weighted pb).
               When ftmachine='mosaic' :
	          minpb is applied to the flux.pbcoverage image 
 
        overwrite -- If False use existing model image of same name to continue clean 
               if True the imagename.model and other associated images are overwitten 
               if they exist
               default: True


         timerange  -- Select data based on time range:
	         default: '' (all); examples,
		 timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
		   Note: if YYYY/MM/DD is missing date defaults to first 
		   day in data set
		   timerange='09:14:0~09:54:0' picks 40 min on first day
		   timerange='25:00:00~27:30:00' picks 1 hr to 3 hr 
		   30min on NEXT day
		   timerange='09:44:00' pick data within one integration 
		   of time
		   timerange='>10:24:00' data after this time
		   For multiple MS input, a list of timerange strings can be
		 used:
          uvrange -- Select data within uvrange (default units meters)
                   default: '' (all); example:
                   uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
                   uvrange='>4klambda';uvranges greater than 4 kilo lambda
          antenna -- Select data based on antenna/baseline
                   default: '' (all)
                   If antenna string is a non-negative integer, it is 
 		   assumed to be an antenna index, otherwise, it is
 		   considered an antenna name.
                   antenna='5&6'; baseline between antenna index 5 and 
 				 index 6.
                   antenna='VA05&VA06'; baseline between VLA antenna 5 
 				       and 6.
                   antenna='5&6;7&8'; baselines 5-6 and 7-8
                   antenna='5'; all baselines with antenna index 5
                   antenna='05'; all baselines with antenna number 05 
 				(VLA old name)
                   antenna='5,6,9'; all baselines with antennas 5,6,9 
 				   index number
               scan -- Scan number range.
                   default: '' (all)
                   example: scan='1~5
               observation -- Observation ID range.
                   default: '' (all)
                   example: observation='1~5'


	clusterdef -- Name of a file that contains the cluster definition.
	              NOTE: there is a chapter in the cookbook on how to 
		      define this file
                       If clusterdef='' (the default) then all the cores, if possible, 
		       of the machine on 
                       which casapy is run will be used.
                  
          Example of a cube imaging run:
	  
	  pclean(vis="ngc5921.ms.contsub",imagename="loulou",imsize=[2500, 2500],
	  cell=['15.0arcsec', '15.0arcsec'],phasecenter="",stokes="I",field="0",spw="*",
	  ftmachine="ft",alg="hogbom",majorcycles=2, niter=6000,gain=0.1,
	  threshold="8mJy",weighting="briggs",robust=0.5,npixels=0,mode="cube",
	  start=5,nchan=46,width=1,interactive=True,overwrite=True,uvtaper=False,
	  outertaper=[''],pbcor=True)              
		       
	  Example of a continuum run:

	  pclean(vis='sim100g_4chan15kRows.ms',
          imagename='hundredG_cont', imsize=[1500, 1500],
          cell=['0.135arcsec', '0.135arcsec'], mode='continuum', phasecenter='0',
          field='0', spw='*', ftmachine='wproject', wprojplanes=128,
          threshold='0.1mJy', 
          majorcycles=4, niter=10000, alg='clark',
          weighting='natural',
          overwrite=True)

 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'pclean'
        myf['taskname'] = 'pclean'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['imagename'] = imagename = myf['imagename']
            myparams['imsize'] = imsize = myf['imsize']
            myparams['cell'] = cell = myf['cell']
            myparams['phasecenter'] = phasecenter = myf['phasecenter']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['mask'] = mask = myf['mask']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['ftmachine'] = ftmachine = myf['ftmachine']
            myparams['alg'] = alg = myf['alg']
            myparams['scales'] = scales = myf['scales']
            myparams['cyclefactor'] = cyclefactor = myf['cyclefactor']
            myparams['majorcycles'] = majorcycles = myf['majorcycles']
            myparams['niter'] = niter = myf['niter']
            myparams['gain'] = gain = myf['gain']
            myparams['threshold'] = threshold = myf['threshold']
            myparams['weighting'] = weighting = myf['weighting']
            myparams['robust'] = robust = myf['robust']
            myparams['npixels'] = npixels = myf['npixels']
            myparams['mode'] = mode = myf['mode']
            myparams['nterms'] = nterms = myf['nterms']
            myparams['start'] = start = myf['start']
            myparams['nchan'] = nchan = myf['nchan']
            myparams['width'] = width = myf['width']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['interactive'] = interactive = myf['interactive']
            myparams['npercycle'] = npercycle = myf['npercycle']
            myparams['wprojplanes'] = wprojplanes = myf['wprojplanes']
            myparams['facets'] = facets = myf['facets']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['uvtaper'] = uvtaper = myf['uvtaper']
            myparams['outertaper'] = outertaper = myf['outertaper']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['scan'] = scan = myf['scan']
            myparams['observation'] = observation = myf['observation']
            myparams['pbcor'] = pbcor = myf['pbcor']
            myparams['minpb'] = minpb = myf['minpb']
            myparams['clusterdef'] = clusterdef = myf['clusterdef']

        if type(imsize)==int: imsize=[imsize]
        if type(cell)==float: cell=[cell]
        if type(scales)==int: scales=[scales]
        if type(outertaper)==str: outertaper=[outertaper]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['imagename'] = imagename
        mytmp['imsize'] = imsize
        if type(cell) == str :
           mytmp['cell'] = casac.quanta().quantity(cell)
        else :
           mytmp['cell'] = cell
        mytmp['phasecenter'] = phasecenter
        mytmp['stokes'] = stokes
        mytmp['mask'] = mask
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['ftmachine'] = ftmachine
        mytmp['alg'] = alg
        mytmp['scales'] = scales
        mytmp['cyclefactor'] = cyclefactor
        mytmp['majorcycles'] = majorcycles
        mytmp['niter'] = niter
        mytmp['gain'] = gain
        mytmp['threshold'] = threshold
        mytmp['weighting'] = weighting
        mytmp['robust'] = robust
        mytmp['npixels'] = npixels
        mytmp['mode'] = mode
        mytmp['nterms'] = nterms
        mytmp['start'] = start
        mytmp['nchan'] = nchan
        mytmp['width'] = width
        mytmp['restfreq'] = restfreq
        mytmp['interactive'] = interactive
        mytmp['npercycle'] = npercycle
        mytmp['wprojplanes'] = wprojplanes
        mytmp['facets'] = facets
        mytmp['overwrite'] = overwrite
        mytmp['uvtaper'] = uvtaper
        mytmp['outertaper'] = outertaper
        mytmp['timerange'] = timerange
        mytmp['uvrange'] = uvrange
        mytmp['antenna'] = antenna
        mytmp['scan'] = scan
        mytmp['observation'] = observation
        mytmp['pbcor'] = pbcor
        mytmp['minpb'] = minpb
        mytmp['clusterdef'] = clusterdef
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'pclean.xml')

        casalog.origin('pclean')
        if not trec.has_key('pclean') or not casac.utils().verify(mytmp, trec['pclean']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: pclean           #####')
          casalog.post('')
          result = pclean(vis, imagename, imsize, cell, phasecenter, stokes, mask, field, spw, ftmachine, alg, scales, cyclefactor, majorcycles, niter, gain, threshold, weighting, robust, npixels, mode, nterms, start, nchan, width, restfreq, interactive, npercycle, wprojplanes, facets, overwrite, uvtaper, outertaper, timerange, uvrange, antenna, scan, observation, pbcor, minpb, clusterdef)
          casalog.post('')
          casalog.post('##### End Task: pclean           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('pclean', 'pclean.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'pclean'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('pclean', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['imagename']  = ''
        a['imsize']  = [256, 256]
        a['cell']  = ['1.0arcsec', '1.0arcsec']
        a['phasecenter']  = ''
        a['stokes']  = 'I'
        a['mask']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['ftmachine']  = 'ft'
        a['alg']  = 'multiscale'
        a['cyclefactor']  = 1.5
        a['niter']  = 500
        a['gain']  = 0.1
        a['threshold']  = '0.0Jy'
        a['weighting']  = 'natural'
        a['mode']  = 'continuum'
        a['interactive']  = False
        a['overwrite']  = True
        a['uvtaper']  = False
        a['timerange']  = ''
        a['uvrange']  = ''
        a['antenna']  = ''
        a['scan']  = ''
        a['observation']  = ''
        a['pbcor']  = False
        a['minpb']  = 0.2
        a['clusterdef']  = ''

        a['mode'] = {
                    0:odict([{'value':'continuum'}, {'nterms':1}]), 
                    1:odict([{'value':'mfs'}, {'nterms':1}]), 
                    2:odict([{'value':'cube'}, {'start':0}, {'nchan':2}, {'width':1}, {'restfreq':''}]), 
                    3:odict([{'value':'channel'}, {'start':0}, {'nchan':2}, {'width':1}, {'restfreq':''}]), 
                    4:odict([{'value':'frequency'}, {'start':'0GHz'}, {'nchan':2}, {'width':'1Hz'}, {'restfreq':''}]), 
                    5:odict([{'value':'velocity'}, {'start':'0km/s'}, {'nchan':2}, {'width':'1m/s'}, {'restfreq':''}])}
        a['cyclefactor'] = {
                    0:{'value':1.5}, 
                    1:odict([{'value':0.0}, {'majorcycles':1}])}
        a['ftmachine'] = {
                    0:{'value':'ft'}, 
                    1:odict([{'value':'wproject'}, {'wprojplanes':128}, {'facets':1}]), 
                    2:{'value':'mosaic'}, 
                    3:{'value':'sd'}}
        a['alg'] = {
                    0:odict([{'value':'multiscale'}, {'scales':[0]}]), 
                    1:{'value':'clark'}, 
                    2:{'value':'hogbom'}}
        a['weighting'] = {
                    0:{'value':'natural'}, 
                    1:{'value':'radial'}, 
                    2:{'value':'uniform'}, 
                    3:odict([{'value':'briggs'}, {'robust':0.0}, {'npixels':0}]), 
                    4:odict([{'value':'briggsabs'}, {'npixels':0}, {'robust':0.0}]), 
                    5:{'value':'radial'}, 
                    6:odict([{'value':'superuniform'}, {'npixels':0}])}
        a['uvtaper'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'outertaper':[]}])}
        a['interactive'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'npercycle':100}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='pclean', subkey=None):
        desc={'pclean': 'Invert and deconvolve images with parallel engines',
               'vis': 'Name of input visibility file',
               'imagename': 'Pre-name of output images',
               'imsize': 'Image size in pixels (nx,ny), symmetric for single value',
               'cell': 'The image cell size in arcseconds. ',
               'phasecenter': 'Image center: direction or field index',
               'stokes': 'Stokes params to image (eg I,IV,IQ,IQUV)',
               'mask': 'mask image',
               'field': 'Field Name or id',
               'spw': 'Spectral windows e.g. \'0~3\', \'\' is all',
               'ftmachine': 'Fourier Transform Engine (\'ft\', \'sd\', \'mosaic\' or \'wproject\')',
               'alg': 'Deconvolution algorithm (\'clark\', \'hogbom\', \'multiscale\')',
               'scales': 'Scales to use in deconvolution',
               'cyclefactor': 'Control number of major cycle, threshold of cycle=residualPeak*psfSidelobe*cyclefactor ',
               'majorcycles': 'Number of major cycles',
               'niter': 'Maximum number of iterations',
               'gain': 'Gain to use in deconvolution',
               'threshold': 'Flux level to stop cleaning, must include units: \'1.0mJy\'',
               'weighting': 'Type of weighting',
               'robust': 'Briggs robustness parameter',
               'npixels': 'number of pixels for superuniform or briggs weighting',
               'mode': 'Clean mode (\'continuum\', \'cube\')',
               'nterms': 'number of terms for multifrequency synthesis',
               'start': 'Begin the output cube at the frequency of this channel in the MS in channel number or frequency or velocity',
               'nchan': 'Number of channels (planes) in output image',
               'width': 'width of output channel ',
               'restfreq': 'rest frequency to assign to image cube for velocity translattion',
               'interactive': 'Interactive clean',
               'npercycle': 'niter per cycle of interactive clean',
               'wprojplanes': 'Number of w-projection planes for convolution',
               'facets': 'Number of facats along each axis',
               'overwrite': 'Overwrite an existing model image',
               'uvtaper': 'Apply additional uv tapering of visibilities',
               'outertaper': 'uv-taper on outer baselines in uv-plane',
               'timerange': 'Range of time to select from data',
               'uvrange': 'Select data within uvrange ',
               'antenna': 'Select data based on antenna/baseline',
               'scan': 'Scan number range',
               'observation': 'Observation ID range',
               'pbcor': 'Correct for the primary beam post deconvolution',
               'minpb': 'Fractional of peak of pb coverage where to stop the  pb correction',
               'clusterdef': 'File that contains cluster definition',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['imagename']  = ''
        a['imsize']  = [256, 256]
        a['cell']  = ['1.0arcsec', '1.0arcsec']
        a['phasecenter']  = ''
        a['stokes']  = 'I'
        a['mask']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['ftmachine']  = 'ft'
        a['alg']  = 'multiscale'
        a['scales']  = [0]
        a['cyclefactor']  = 1.5
        a['majorcycles']  = 1
        a['niter']  = 500
        a['gain']  = 0.1
        a['threshold']  = '0.0Jy'
        a['weighting']  = 'natural'
        a['robust']  = 0.0
        a['npixels']  = 0
        a['mode']  = 'continuum'
        a['nterms']  = 1
        a['start']  = 0
        a['nchan']  = 1
        a['width']  = 1
        a['restfreq']  = ''
        a['interactive']  = False
        a['npercycle']  = 100
        a['wprojplanes']  = 128
        a['facets']  = 1
        a['overwrite']  = True
        a['uvtaper']  = False
        a['outertaper']  = ['']
        a['timerange']  = ''
        a['uvrange']  = ''
        a['antenna']  = ''
        a['scan']  = ''
        a['observation']  = ''
        a['pbcor']  = False
        a['minpb']  = 0.2
        a['clusterdef']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
pclean_pg = pclean_pg_()
