#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_plotcal import plotcal
from task_plotcal import casalog

class plotcal_pg_:
    __name__ = "plotcal"

    def __init__(self) :
       self.__bases__ = (plotcal_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, caltable=None,xaxis=None,yaxis=None,poln=None,field=None,antenna=None,spw=None,timerange=None,subplot=None,overplot=None,clearpanel=None,iteration=None,plotrange=None,showflags=None,plotsymbol=None,plotcolor=None,markersize=None,fontsize=None,showgui=None,figfile=None,):

        """An all-purpose plotter for calibration results 
	
       	The values for all calibration solutions (G, T, GSPLINE, B, BPOLY, D, M) 
        can be displayed for a variety of polarization combinations and calibrations.
        The plot solutions may be iterated through antennas/spw/fields during one execution,
        and many frames can be obtained in each plot.

	The plotter permits zooming, listing and flagging of solutions, although
	the results of flagged solutions are not yet available.


        The plotter permits zooming, listing and flagging of solutions, although
        the implications of flagged solutions are not yet made.  See some hints at the end
        of this description.


	Keyword arguments:
	caltable -- Name of input calibration table 
		default: none; example: caltable='ngc5921.gcal'
	        The type of calibration table is determined automatically.
	xaxis -- Value to plot on the x axis
		Options: 'time','scan','chan','freq','antenna','amp','phase','real','imag','snr'
		Default: cal type dependent, usually 'time'
	yaxis -- Value to plot on the y-axis
		Options: 'amp','phase','real','imag','snr','antenna','tsys','delay','spgain'
		Default: cal type dependent, usually 'amp'
	poln -- Polarization (or combination) to plot
	        default: '' (RL); all polarizations
		Options: '' = ('RL'),'R','L','XY','X','Y',
		               '/' --> form complex poln ratio
			             (amp ratio and phase difference)
	field -- Select field using field id(s) or field name(s).
                  ['go listobs' to obtain the fieldt id's or names]
               default: ''=all fields
               If field string is a non-negative integer, it is assumed a
	       field index, otherwise it is assumed a field name
               field='0~2'; field ids 0,1,2
               field='0,4,5~7'; field ids 0,4,5,6,7
               field='3C286,3C295'; field named 3C286 and 3C295
               field = '3,4C*'; field id 3, all names starting with 4C
	antenna -- Antenna selection (baseline syntax ignored)
               default: '' (all);
               example: antenna='1,3~5' means antenna
                  indices 1,3,4,5.  
	spw -- Select spectral window (channel syntax ignored, except for D)
               default: ''=all spectral windows
               spw='0~2,4'; spectral windows 0,1,2,4
               spw='<2';  spectral windows less than 2
	timerange -- Time selection
                  default: '' (all)
		  example: timerange='1995/04/13/09:15:00~1995/04/13/09:25:00'

	--- Plot Options ---
	subplot -- Panel number on the display screen
               default: 111 (full screen display);
               examples:
               if iteration = 'antenna'; subplot=321 then
                  a plot frame will contain the first 6 antennas, in three
                  rows and two columns.  Follow instructions on screen to
                  cycle through the frames
               if iteration = ''; then one frame can be filled with many
                  plots in a piecemeal fashion; for example
                  antenna='0'; subplot=221; plotcal()
                  antenna='1'; subplot=222; plotcal()
                  antenna='2'; subplot=223; plotcal()
                  antenna='3'; subplot=224; plotcal()
	overplot -- Overplot these values on current plot (if possible)
               default: False;
	          True (overplotting) can be done ONLY IF iteration=''
	clearpanel -- Ignore this parameter.
                  Clear nothing on the plot window, automatically
	          clear plotting area, clear the current plot area, or
	          clear the whole plot panel.
	       options: None, Auto, Current, All (None and Auto not supported)
	       default: Auto
	       example: clearpanel='Current'
	iteration -- Create a sequence of plots, iterating over antenna, time,
                 field, and/or spw  
               default: '' --> create in all in one plot
	       example: iteration='antenna' --> create a sequence of
	                separate plots separated by antenna. Flagging cannot
	                be done in iteration mode.
	plotrange -- Control the x and y ranges of the plot, as a list of
                 values, e.g., [xmin,xmax,ymin,ymax]
		 default=[] --> plot will self-scale
		 Note: time plotting ranges are cumbersome to use.
                       Use the zoom option
	showflags -- If true, only flagged solutions will be plotted
	         default: false --> only show unflagged solutions
	plotsymbol -- pylab plot symbol.  See cookbook for details
                   default: '.': large points
                   ',' = small points (see markersize)
                   '-' = connect points by line
                   colors are cycled automatically for multi-function plots
	plotcolor -- Initial color to use on each plot
                 default: 'blue'
	markersize -- Control the size of plot symbols
                  default: 5.0 --> a nice size for symbols
	fontsize -- Control the font size of title (axes labels will be
	          80% of this size)
                  default: 10.0
	showgui -- Whether or not to display the plotting GUI
	          default: True; example showgui=False
        figfile -- File name to save the plotted figure to.
	          default: ''; example figfile=myPlot.png

         Hints on using plotxy (see section 3.4 in cookbook)

         Useful Buttons at bottom left:
              5th--magnifying glass.  Click on this,
                       left mouse button rectangle drag will zoom
                       right mose button rectangle drag will unzoom a certain amount
              1st--restore original magnification

         Useful regions just above:
              Quit will terminate plotter
              Next will go to next plot as specified by iteration
              To locate, you must click 'Mark Region' first
                 then make appropriate region(s)
                 then click locate to list points on logger
                 DO NOT USE Flag, Unflag at the present time.
            		  
 
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'plotcal'
        myf['taskname'] = 'plotcal'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['caltable'] = caltable = myf['caltable']
            myparams['xaxis'] = xaxis = myf['xaxis']
            myparams['yaxis'] = yaxis = myf['yaxis']
            myparams['poln'] = poln = myf['poln']
            myparams['field'] = field = myf['field']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['spw'] = spw = myf['spw']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['subplot'] = subplot = myf['subplot']
            myparams['overplot'] = overplot = myf['overplot']
            myparams['clearpanel'] = clearpanel = myf['clearpanel']
            myparams['iteration'] = iteration = myf['iteration']
            myparams['plotrange'] = plotrange = myf['plotrange']
            myparams['showflags'] = showflags = myf['showflags']
            myparams['plotsymbol'] = plotsymbol = myf['plotsymbol']
            myparams['plotcolor'] = plotcolor = myf['plotcolor']
            myparams['markersize'] = markersize = myf['markersize']
            myparams['fontsize'] = fontsize = myf['fontsize']
            myparams['showgui'] = showgui = myf['showgui']
            myparams['figfile'] = figfile = myf['figfile']

        if type(plotrange)==float: plotrange=[plotrange]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['caltable'] = caltable
        mytmp['xaxis'] = xaxis
        mytmp['yaxis'] = yaxis
        mytmp['poln'] = poln
        mytmp['field'] = field
        mytmp['antenna'] = antenna
        mytmp['spw'] = spw
        mytmp['timerange'] = timerange
        mytmp['subplot'] = subplot
        mytmp['overplot'] = overplot
        mytmp['clearpanel'] = clearpanel
        mytmp['iteration'] = iteration
        mytmp['plotrange'] = plotrange
        mytmp['showflags'] = showflags
        mytmp['plotsymbol'] = plotsymbol
        mytmp['plotcolor'] = plotcolor
        mytmp['markersize'] = markersize
        mytmp['fontsize'] = fontsize
        mytmp['showgui'] = showgui
        mytmp['figfile'] = figfile
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'plotcal.xml')

        casalog.origin('plotcal')
        if not trec.has_key('plotcal') or not casac.utils().verify(mytmp, trec['plotcal']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: plotcal           #####')
          casalog.post('')
          result = plotcal(caltable, xaxis, yaxis, poln, field, antenna, spw, timerange, subplot, overplot, clearpanel, iteration, plotrange, showflags, plotsymbol, plotcolor, markersize, fontsize, showgui, figfile)
          casalog.post('')
          casalog.post('##### End Task: plotcal           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('plotcal', 'plotcal.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'plotcal'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('plotcal', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['caltable']  = ''
        a['xaxis']  = ''
        a['yaxis']  = ''
        a['poln']  = ''
        a['field']  = ''
        a['antenna']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['subplot']  = 111
        a['overplot']  = False
        a['clearpanel']  = 'Auto'
        a['iteration']  = ''
        a['plotrange']  = [
			]
        a['showflags']  = False
        a['plotsymbol']  = 'o'
        a['plotcolor']  = 'blue'
        a['markersize']  = 5.0
        a['fontsize']  = 10.0
        a['showgui']  = True
        a['figfile']  = ''


### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='plotcal', subkey=None):
        desc={'plotcal': 'An all-purpose plotter for calibration results ',
               'caltable': 'Name of input calibration table',
               'xaxis': 'Value to plot along x axis (time,chan,freq, antenna,antenna1,antenna2,scan, amp,phase,real,imag,snr, tsys,delay,spgain)',
               'yaxis': 'Value to plot along y axis (amp,phase,real,imag,snr, antenna,antenna1,antenna2,scan, tsys,delay,spgain,tec)',
               'poln': 'Antenna polarization to plot (RL,R,L,XY,X,Y,/)',
               'field': 'field names or index of calibrators: \'\'==>all',
               'antenna': 'antenna/baselines: \'\'==>all, antenna = \'3,VA04\'',
               'spw': 'spectral window:channels: \'\'==>all, spw=\'1:5~57\'',
               'timerange': 'time range: \'\'==>all',
               'subplot': 'Panel number on display screen (yxn)',
               'overplot': 'Overplot solutions on existing display',
               'clearpanel': 'Specify if old plots are cleared or not (ignore)',
               'iteration': 'Iterate plots on antenna,time,spw,field',
               'plotrange': 'plot axes ranges: [xmin,xmax,ymin,ymax]',
               'showflags': 'If true, show flagged solutions',
               'plotsymbol': 'pylab plot symbol',
               'plotcolor': 'initial plotting color',
               'markersize': 'Size of plotted marks',
               'fontsize': 'Font size for labels',
               'showgui': 'Show plot on gui',
               'figfile': '\'\'= no plot hardcopy, otherwise supply name',

              }

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['caltable']  = ''
        a['xaxis']  = ''
        a['yaxis']  = ''
        a['poln']  = ''
        a['field']  = ''
        a['antenna']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['subplot']  = 111
        a['overplot']  = False
        a['clearpanel']  = 'Auto'
        a['iteration']  = ''
        a['plotrange']  = [
			]
        a['showflags']  = False
        a['plotsymbol']  = 'o'
        a['plotcolor']  = 'blue'
        a['markersize']  = 5.0
        a['fontsize']  = 10.0
        a['showgui']  = True
        a['figfile']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
plotcal_pg = plotcal_pg_()
