#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_plotms import plotms
from task_plotms import casalog

class plotms_pg_:
    __name__ = "plotms"

    def __init__(self) :
       self.__bases__ = (plotms_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,gridrows=None,gridcols=None,rowindex=None,colindex=None,plotindex=None,xaxis=None,xdatacolumn=None,yaxis=None,ydatacolumn=None,yaxislocation=None,selectdata=None,field=None,spw=None,timerange=None,uvrange=None,antenna=None,scan=None,correlation=None,array=None,observation=None,msselect=None,averagedata=None,avgchannel=None,avgtime=None,avgscan=None,avgfield=None,avgbaseline=None,avgantenna=None,avgspw=None,scalar=None,transform=None,freqframe=None,restfreq=None,veldef=None,shift=None,extendflag=None,extcorr=None,extchannel=None,iteraxis=None,xselfscale=None,yselfscale=None,xsharedaxis=None,ysharedaxis=None,customsymbol=None,symbolshape=None,symbolsize=None,symbolcolor=None,symbolfill=None,symboloutline=None,coloraxis=None,customflaggedsymbol=None,flaggedsymbolshape=None,flaggedsymbolsize=None,flaggedsymbolcolor=None,flaggedsymbolfill=None,flaggedsymboloutline=None,plotrange=None,title=None,xlabel=None,ylabel=None,showmajorgrid=None,majorwidth=None,majorstyle=None,majorcolor=None,showminorgrid=None,minorwidth=None,minorstyle=None,minorcolor=None,showlegend=None,legendposition=None,plotfile=None,expformat=None,exprange=None,highres=None,overwrite=None,showgui=None,clearplots=None,):

        """A plotter/interactive flagger for visibility data.

        Task for plotting and interacting with visibility
        data.  Limited support for caltable plotting is also
        included as of CASA v4.1.

        A variety of axes choices (including data column) along 
        with MS selection and averaging options are provided for data 
        selection.  Flag extension parameters are also available for
        flagging operations in the plotter.
        
        All of the provided parameters can also be set using the GUI once
        the application has been launched.  Additional and more specific
        operations are available through the GUI and/or through the plotms
        tool (pm).

        Most basic functions (plotting, iteration, locate, flagging)
        will work for most CalTables. Parameterized CalTables
        (delays, antpos, gaincurve, opacity), will, at best, currently 
        just plot the simple parameters contained in the
        table, not the effective amplitudes or phases sampled at
        observing times, frequencies etc.  BPOLY and GSPLINE tables
        are not yet supported.   Features currently unsupported for
        CalTables include Averaging, Transformations (velocity 
        conversions, etc.), and some details of selection (channel and 
        polarization selection are not yet enabled) and axes choices 
        (geometry options are not yet enabled).  In the plotms gui,
        many options irrelevant for CalTables are not yet hidden when
        interacting with a CalTable, and such settings will be ignored
        (when benign) or cause an error message.

    Keyword arguments:
    vis -- input MS or CalTable
           default: ''  (will merely launch the gui)
    gridrows -- Number of subplot rows
    				default: 1 
    gridcols -- Number of subplot columns
    				default: 1
    rowindex -- Row location of the subplot (0-based).
                    default: 0
    colindex -- Column location of the subplot (0-based).
                    default: 0
    plotindex -- Index to address a subplot (0-based). 
    				default: 0                         
    xaxis, yaxis -- what to plot on the two axes
                    default: '' (defaults are xaxis='time',
                                 yaxis='amp' on first execution;
                                 thereafter the most recent
                                 settings are used)
              valid options (=indicates valid synonyms): 
               MS Ids and other meta info:
                 'scan'   (number)
                 'field'  (index)
                 'time',  
                 'interval'='timeint'='timeinterval'='time_interval'
                 'spw'    (index)
                 'chan'='channel'    (index)  
                 'freq'='frequency'  (GHz)
                 'vel'='velocity'   (km/s)
                 'corr'='correlation'  (index)
                 'ant1'='antenna1'   (index)
                 'ant2'='antenna2'   (index)
                 'baseline'  (a baseline index) 
                 'row'  (absoute row Id from the MS)
               Visibility values, flags:
                 'amp'='amplitude'
                 'phase'  (deg)
                 'real'  
                 'imag'='imaginary'
                 'wt'='weight'  (unchannelized)
                 'wtsp'='weightspectrum'
                 'flag'
                 'flagrow'
               Observational geometry:
                 'uvdist'  (meters)
                 'uvwave'='uvdistl'='uvdist_l'  (wavelengths, per channel)
                 'u'  (meters)
                 'v'  (meters)
                 'w'  (meters)
                 'uwave'  ('u' in wavelengths, per channel)
                 'vwave'  ('v' in wavelengths, per channel)
                 'wwave'  ('w' in wavelengths, per channel)
                 'azimuth'  (at array reference; degrees)
                 'elevation'  (at array reference; degrees)
                 'hourang'='hourangle'  (at array reference; hours)
                 'parang'='parangle'='parallacticangle'  (at array reference; degrees)
               Antenna-based (only works vs. data Ids):
                 'ant'='antenna'
                 'ant-azimuth' 
                 'ant-elevation'
                 'ant-parang'='ant-parangle'

               Calibration:
                 'gainamp'='gamp'
                 'gainphase'='gphase'
                 'gainreal'='greal'
                 'gainimag'='gimag'
                 'delay'='del'
                 'opacity'='opac'
                 'swpower'='swp'='switchedpower'


      >>> xaxis, yaxis expandable parameters
        xdatacolumn, 
        ydatacolumn  -- which data column to use for Visibility values:
                        default: '' ('data' on first execuation;
                                     thereafter the most recent
                                     setting is used)
                        valid options:  'data'      (observed)
                                        'corrected'='corr'
                                        'model'
                                        'residual'  (aliases 'corrected-model')
                                        'corrected-model'
                                        'data-model'
    
    selectdata -- data selection parameters flag
                  default: True  (reveals data selection parameters
                                  described below)
                  Consult listobs output for data selection values,
                  and see help par.selectdata for more detailed 
                  information on syntax; also, visit
                  http://casa.nrao.edu/other_doc.shtml and click
                  on "Measurement Set selection syntax" for more
                  tips on using data selection parameters in CASA)

      >>> selectdata expandable parameters:

      field -- Select field using field id(s) or field name(s).
              default: ''=all fields
              If field string is a non-negative integer, it is assumed a
                field index,  otherwise, it is assumed a field name
              field='0~2'; field ids 0,1,2
              field='0,4,5~7'; field ids 0,4,5,6,7
              field='3C286,3C295'; field named 3C286 and 3C295
              field = '3,4C*'; field id 3, all names starting with 4C
      spw -- Select spectral window/channels
               type 'help par.selection' for more examples.
             spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
             spw='<2';  spectral windows less than 2 (i.e. 0,1)
             spw='0:5~61'; spw 0, channels 5 to 61, INCLUSIVE
             spw='*:5~61'; all spw with channels 5 to 61
             spw='0,10,3:3~45'; spw 0,10 all channels, spw 3, channels 3 to 45.
             spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
             spw='0:0~10;15~60'; spectral window 0 with channels 0-10,15-60
                       NOTE ';' to separate channel selections

      timerange  -- Select data based on time range:
              default = '' (all); examples,
              timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
              Note: if YYYY/MM/DD is missing date defaults to first day in data set
              timerange='09:14:0~09:54:0' picks 40 min on first day
              timerange= '25:00:00~27:30:00' picks 1 hr to 3 hr 30min on NEXT day
              timerange='09:44:00' pick data within one integration of time
              timerange='>10:24:00' data after this time
      uvrange -- Select data within uvrange (default units meters)
              default: '' (all); example:
              uvrange='0~1000klambda'; uvrange from 0-1000 kilo-lambda
              uvrange='>4klambda';uvranges greater than 4 kilo lambda

      antenna -- Select data based on antenna/baseline
              default: '' (all, including auto-correlations, if present)
              If antenna string is a non-negative integer, it is assumed an
                antenna index, otherwise, it is assumed as an antenna name
              antenna='5&6'; baseline between antenna index 5 and index 6.
	      antenna='!ea02'; exclude EVLA antenna 2.
	      antenna='ea13;!ea22'; EVLA antenna 13, excluding antenna 22.
              antenna='VA05&VA06'; baseline between VLA antenna 5 and 6.
              antenna='5&6;7&8'; baselines with indices 5-6 and 7-8
              antenna='5'; all baselines with antenna index 5
              antenna='05'; all baselines with antenna number 05 (VLA old name)
              antenna='5,6,10'; all baselines with antennas 5,6,10 index numbers
              NB: For explicit selections, use a single ampersand (&) to
              select only cross-correlations among the specified antennas, 
              double ampersands (&&) to select cross- and
              auto-correlations among the specified antennas, and
              triple ampersands (&&&) to select only
              auto-correlations.  E.g.:
              antenna='*&'; selects all cross-correlation baseline
                            (excludes all auto-correlations)
              antenna='*&&&'; selects all auto-correlation baselines
                              (excludes all cross-correlations)
              antenna='1&&1,2,3'; selects baselines 1-1 (auto), 1-2,1-3 (cross)
              antenna='VA05&&&'; selects the VA05 autocorrelation
              See the link noted above for more information.
      scan -- Scan numbers or ranges.
              default: ''  (all scans)
              scan='1,2,6,43'; scans 1, 2, 6, and 43
              scan='3~14'; scans 3 through 14, inclusive
      correlation -- Select by correlation
                     default: ''  (all correlations)
                     options: 'RR','RL','LR','LL','XX','XY','YX','YY',
                               or any comma-separated combination; use
                               basis (R/L or X/Y) appropriate to the MS)
      array -- Select the array id
               default: ''  (all array ids)
      observation Select by observation ID(s).
                  default: ''-->all;
		  example: observation='0' (select obsID 0)
      msselect -- Optional TaQL data selection

    averagedata -- data averaging parameters flag
                   default: True   (reveals expandable parameters
                                    described below)
      >>> averagedata expandable parameters
        avgchannel -- average over channel?  either blank for none, or a value
                      in channels.
                      default: '' (no channel averaging).
        avgtime -- average over time?  either blank for none, or a value in
                   seconds.
                   default: '' (no time averaging).
        avgscan -- average over scans?  only valid if time averaging is turned
                   on.
                   default: False.
        avgfield -- average over fields?  only valid if time averaging is
                    turned on.
                    default: False.
        avgbaseline -- average over selected baselines; mutually 
                       exclusive with avgantenna.
                       default: False.  (no averaging over baseline)
        avgantenna -- form per-antenna averages; mutually exclusive with
                      avgbaseline.
                      default: False.   (no per-antenna averaging)
        avgspw -- average over selected spectral windows?
                  default: False.  (no average of spectral windows)
        scalar -- scalar averaging?
                  default: False  (i.e., do vector averaging)
    
    transform -- apply various transformations on data for plotting
                 default: False.
      >>> transform expandable parameters
        freqframe -- the coordinate frame in which to render frequency and velocity axes
                 default: ''  (unspecified: will use frame in which data were taken)
                 options: TOPO, GEO, BARY, LSRK, LSRD
        restfreq -- the rest frequency to use in velocity conversions (MHz)
                 default: '' (use spw central frequency and show relative velocity)
                 example: '22235.08MHz'
        veldef -- the velocity definition to use
                 default: 'RADIO'
                 options: 'RADIO','OPT','TRUE'
        shift -- adjust phase according to a phase center shift [dx,dy] (arcsec)
                 default: [0,0]  (no shift)

    extendflag -- have flagging extend to other data points?
                  default: False.
      >>> extendflag expandable parameters
        extcorr -- extend flags based on correlation? 
                   default: False.
        extchannel -- extend flags based on channel?

    iteraxis -- axis upon which iterate plots (one plot per page, for now)
                default: '' (no iteration)
                options: 'scan','field','spw','baseline','antenna', 'time',''
      >>> iteraxis expandable parameters
        xselfscale -- If true, iterated plots should share a common x-axis label per column.
        yselfscale -- If true, iterated plots should share a common y-axis label per row.
                      default: false, which will scale all plots globally
        xsharedaxis -- If true, iterated plots should share a common x-axis.
        			  default: false, each plot will have its own x-axis.
        ysharedaxis -- If true, iterated plots should share a common y-axis.
        			  default: false, each plot will have its own y-axis.              

    customsymbol -- If true, use a custom symbol for drawing unflagged points
                    default: False
      >>> customsymbol expandable parameters
        symbolshape -- If true, use a custom shape to draw unflagged symbols
                       default: 'autoscaling' (ignores symbolsize)
                       options: 'autoscaling', 'circle', 'square', 'diamond', 'pixel', 'nosymbol'
        symbolsize -- size of the unflagged symbols in pixels
                      default: 2
        symbolcolor -- color to use for unflagged symbols; can be a RGB hex code or a color name
                       default: '0000ff'
                       example: 'purple'
        symbolfill -- type of fill to use for unflagged symbols
                      default: 'fill'
                      options: 'fill', 'mesh1', 'mesh2', 'mesh3', 'nofill'
        symboloutline -- If true, outline unflagged symbols in black

    coloraxis -- axis upon which to colorize the plotted points
                options (= indicates synonyms):
                    'scan',  'field',  'spw',  'antenna1'='ant1',  'antenna2'='ant2',
                    'baseline',  'channel'='chan',  'corr'='correlation', 'time'
                default: ''  (use a single color for all points)

    customflaggedsymbol -- If true, use a custom symbol for drawing flagged points
                           default: False
      >>> customflaggedsymbol expandable parameters
        symbolshape -- If true, use a custom shape to draw flagged symbols
                       default: 'nosymbol'
                       options: 'autoscaling', 'circle', 'square', 'diamond', 'pixel', 'nosymbol'
        symbolsize -- size of the flagged symbols in pixels
                      default: 2
        symbolcolor -- color to use for flagged symbols; can be a RGB hex code or a color name
                       default: '0000ff'
                       example: 'purple'
        symbolfill -- type of fill to use for flagged symbols
                      default: 'fill'
                      options: 'fill', 'mesh1', 'mesh2', 'mesh3', 'nofill'
        symboloutline -- If true, outline flagged symbols in black

    plotrange -- manual plot axis ranges: [xmin,xmax,ymin,ymax]
                 Does not affect data selection.
                 default: []; both axes will be autoscaled according
                 to the ranges found in the selected data
                 If xmin=xmax (or ymin=ymax) then that axis will
                 be autoscaled, e.g.:
                 [0,0,-2.0,14.0]; autoscale the xaxis, and use 
                                  ymin=-2.0, ymax=14.0

    title  -- title along top of plot (called "canvas" in some places)
    
    xlabel -- text to label horizontal axis, with formatting using '%%' 
    ylabel -- text to label horizontal axis, with formatting using '%%' 
    
    
    showmajorgrid  -- show major grid lines (horiz and vert.)
                  default: False
      >>>  showmajorgrid expandable parameters
        majorwidth  -- line width in pixels of major grid lines
        majorstyle  -- major grid line style: solid dash dot none
        majorcolor  -- color in hex code of major grid lines

    showminorgrid  -- show minor grid lines (horiz and vert.)
                  default: False
      >>>  showminorgrid expandable parameters
        minorwidth  --  line width in pixels of minor grid lines
        minorstyle  --  minor grid line style: solid dash dot none
        minorcolor  --  color in hex code of minor grid lines

    plotfile -- name of plot file to save automatically
                default: ''  (i.e., draw an interactive plot in the gui)
      >>> plotfile expandable parameters
        expformat -- export format type; if 'txt' is used an ASCII dump of the plotted points is generated (also available in the export tab)
                     default:  ''   (plotfile extension will be used)
                     options: 'jpg', 'png', 'ps', 'pdf', 'txt'
        exprange -- pages to export for iteration plots
        			default:   ''
        			options: 'current', 'all'             
        highres -- use high resolution? Always true for jpg and png.
                   default: false
        overwrite -- Overwrite plot file if it already exists?
                     default: false
      



        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'plotms'
        myf['taskname'] = 'plotms'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['gridrows'] = gridrows = myf['gridrows']
            myparams['gridcols'] = gridcols = myf['gridcols']
            myparams['rowindex'] = rowindex = myf['rowindex']
            myparams['colindex'] = colindex = myf['colindex']
            myparams['plotindex'] = plotindex = myf['plotindex']
            myparams['xaxis'] = xaxis = myf['xaxis']
            myparams['xdatacolumn'] = xdatacolumn = myf['xdatacolumn']
            myparams['yaxis'] = yaxis = myf['yaxis']
            myparams['ydatacolumn'] = ydatacolumn = myf['ydatacolumn']
            myparams['yaxislocation'] = yaxislocation = myf['yaxislocation']
            myparams['selectdata'] = selectdata = myf['selectdata']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['uvrange'] = uvrange = myf['uvrange']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['scan'] = scan = myf['scan']
            myparams['correlation'] = correlation = myf['correlation']
            myparams['array'] = array = myf['array']
            myparams['observation'] = observation = myf['observation']
            myparams['msselect'] = msselect = myf['msselect']
            myparams['averagedata'] = averagedata = myf['averagedata']
            myparams['avgchannel'] = avgchannel = myf['avgchannel']
            myparams['avgtime'] = avgtime = myf['avgtime']
            myparams['avgscan'] = avgscan = myf['avgscan']
            myparams['avgfield'] = avgfield = myf['avgfield']
            myparams['avgbaseline'] = avgbaseline = myf['avgbaseline']
            myparams['avgantenna'] = avgantenna = myf['avgantenna']
            myparams['avgspw'] = avgspw = myf['avgspw']
            myparams['scalar'] = scalar = myf['scalar']
            myparams['transform'] = transform = myf['transform']
            myparams['freqframe'] = freqframe = myf['freqframe']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['veldef'] = veldef = myf['veldef']
            myparams['shift'] = shift = myf['shift']
            myparams['extendflag'] = extendflag = myf['extendflag']
            myparams['extcorr'] = extcorr = myf['extcorr']
            myparams['extchannel'] = extchannel = myf['extchannel']
            myparams['iteraxis'] = iteraxis = myf['iteraxis']
            myparams['xselfscale'] = xselfscale = myf['xselfscale']
            myparams['yselfscale'] = yselfscale = myf['yselfscale']
            myparams['xsharedaxis'] = xsharedaxis = myf['xsharedaxis']
            myparams['ysharedaxis'] = ysharedaxis = myf['ysharedaxis']
            myparams['customsymbol'] = customsymbol = myf['customsymbol']
            myparams['symbolshape'] = symbolshape = myf['symbolshape']
            myparams['symbolsize'] = symbolsize = myf['symbolsize']
            myparams['symbolcolor'] = symbolcolor = myf['symbolcolor']
            myparams['symbolfill'] = symbolfill = myf['symbolfill']
            myparams['symboloutline'] = symboloutline = myf['symboloutline']
            myparams['coloraxis'] = coloraxis = myf['coloraxis']
            myparams['customflaggedsymbol'] = customflaggedsymbol = myf['customflaggedsymbol']
            myparams['flaggedsymbolshape'] = flaggedsymbolshape = myf['flaggedsymbolshape']
            myparams['flaggedsymbolsize'] = flaggedsymbolsize = myf['flaggedsymbolsize']
            myparams['flaggedsymbolcolor'] = flaggedsymbolcolor = myf['flaggedsymbolcolor']
            myparams['flaggedsymbolfill'] = flaggedsymbolfill = myf['flaggedsymbolfill']
            myparams['flaggedsymboloutline'] = flaggedsymboloutline = myf['flaggedsymboloutline']
            myparams['plotrange'] = plotrange = myf['plotrange']
            myparams['title'] = title = myf['title']
            myparams['xlabel'] = xlabel = myf['xlabel']
            myparams['ylabel'] = ylabel = myf['ylabel']
            myparams['showmajorgrid'] = showmajorgrid = myf['showmajorgrid']
            myparams['majorwidth'] = majorwidth = myf['majorwidth']
            myparams['majorstyle'] = majorstyle = myf['majorstyle']
            myparams['majorcolor'] = majorcolor = myf['majorcolor']
            myparams['showminorgrid'] = showminorgrid = myf['showminorgrid']
            myparams['minorwidth'] = minorwidth = myf['minorwidth']
            myparams['minorstyle'] = minorstyle = myf['minorstyle']
            myparams['minorcolor'] = minorcolor = myf['minorcolor']
            myparams['showlegend'] = showlegend = myf['showlegend']
            myparams['legendposition'] = legendposition = myf['legendposition']
            myparams['plotfile'] = plotfile = myf['plotfile']
            myparams['expformat'] = expformat = myf['expformat']
            myparams['exprange'] = exprange = myf['exprange']
            myparams['highres'] = highres = myf['highres']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['showgui'] = showgui = myf['showgui']
            myparams['clearplots'] = clearplots = myf['clearplots']

        if type(shift)==float: shift=[shift]
        if type(plotrange)==float: plotrange=[plotrange]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['gridrows'] = gridrows
        mytmp['gridcols'] = gridcols
        mytmp['rowindex'] = rowindex
        mytmp['colindex'] = colindex
        mytmp['plotindex'] = plotindex
        mytmp['xaxis'] = xaxis
        mytmp['xdatacolumn'] = xdatacolumn
        mytmp['yaxis'] = yaxis
        mytmp['ydatacolumn'] = ydatacolumn
        mytmp['yaxislocation'] = yaxislocation
        mytmp['selectdata'] = selectdata
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['timerange'] = timerange
        mytmp['uvrange'] = uvrange
        mytmp['antenna'] = antenna
        mytmp['scan'] = scan
        mytmp['correlation'] = correlation
        mytmp['array'] = array
        mytmp['observation'] = observation
        mytmp['msselect'] = msselect
        mytmp['averagedata'] = averagedata
        mytmp['avgchannel'] = avgchannel
        mytmp['avgtime'] = avgtime
        mytmp['avgscan'] = avgscan
        mytmp['avgfield'] = avgfield
        mytmp['avgbaseline'] = avgbaseline
        mytmp['avgantenna'] = avgantenna
        mytmp['avgspw'] = avgspw
        mytmp['scalar'] = scalar
        mytmp['transform'] = transform
        mytmp['freqframe'] = freqframe
        mytmp['restfreq'] = restfreq
        mytmp['veldef'] = veldef
        mytmp['shift'] = shift
        mytmp['extendflag'] = extendflag
        mytmp['extcorr'] = extcorr
        mytmp['extchannel'] = extchannel
        mytmp['iteraxis'] = iteraxis
        mytmp['xselfscale'] = xselfscale
        mytmp['yselfscale'] = yselfscale
        mytmp['xsharedaxis'] = xsharedaxis
        mytmp['ysharedaxis'] = ysharedaxis
        mytmp['customsymbol'] = customsymbol
        mytmp['symbolshape'] = symbolshape
        mytmp['symbolsize'] = symbolsize
        mytmp['symbolcolor'] = symbolcolor
        mytmp['symbolfill'] = symbolfill
        mytmp['symboloutline'] = symboloutline
        mytmp['coloraxis'] = coloraxis
        mytmp['customflaggedsymbol'] = customflaggedsymbol
        mytmp['flaggedsymbolshape'] = flaggedsymbolshape
        mytmp['flaggedsymbolsize'] = flaggedsymbolsize
        mytmp['flaggedsymbolcolor'] = flaggedsymbolcolor
        mytmp['flaggedsymbolfill'] = flaggedsymbolfill
        mytmp['flaggedsymboloutline'] = flaggedsymboloutline
        mytmp['plotrange'] = plotrange
        mytmp['title'] = title
        mytmp['xlabel'] = xlabel
        mytmp['ylabel'] = ylabel
        mytmp['showmajorgrid'] = showmajorgrid
        mytmp['majorwidth'] = majorwidth
        mytmp['majorstyle'] = majorstyle
        mytmp['majorcolor'] = majorcolor
        mytmp['showminorgrid'] = showminorgrid
        mytmp['minorwidth'] = minorwidth
        mytmp['minorstyle'] = minorstyle
        mytmp['minorcolor'] = minorcolor
        mytmp['showlegend'] = showlegend
        mytmp['legendposition'] = legendposition
        mytmp['plotfile'] = plotfile
        mytmp['expformat'] = expformat
        mytmp['exprange'] = exprange
        mytmp['highres'] = highres
        mytmp['overwrite'] = overwrite
        mytmp['showgui'] = showgui
        mytmp['clearplots'] = clearplots
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'plotms.xml')

        casalog.origin('plotms')
        if not trec.has_key('plotms') or not casac.utils().verify(mytmp, trec['plotms']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: plotms           #####')
          casalog.post('')
          result = plotms(vis, gridrows, gridcols, rowindex, colindex, plotindex, xaxis, xdatacolumn, yaxis, ydatacolumn, yaxislocation, selectdata, field, spw, timerange, uvrange, antenna, scan, correlation, array, observation, msselect, averagedata, avgchannel, avgtime, avgscan, avgfield, avgbaseline, avgantenna, avgspw, scalar, transform, freqframe, restfreq, veldef, shift, extendflag, extcorr, extchannel, iteraxis, xselfscale, yselfscale, xsharedaxis, ysharedaxis, customsymbol, symbolshape, symbolsize, symbolcolor, symbolfill, symboloutline, coloraxis, customflaggedsymbol, flaggedsymbolshape, flaggedsymbolsize, flaggedsymbolcolor, flaggedsymbolfill, flaggedsymboloutline, plotrange, title, xlabel, ylabel, showmajorgrid, majorwidth, majorstyle, majorcolor, showminorgrid, minorwidth, minorstyle, minorcolor, showlegend, legendposition, plotfile, expformat, exprange, highres, overwrite, showgui, clearplots)
          casalog.post('')
          casalog.post('##### End Task: plotms           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('plotms', 'plotms.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'plotms'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('plotms', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['gridrows']  = 1
        a['gridcols']  = 1
        a['rowindex']  = 0
        a['colindex']  = 0
        a['plotindex']  = 0
        a['xaxis']  = ''
        a['yaxis']  = '', []
        a['selectdata']  = True
        a['averagedata']  = True
        a['transform']  = False
        a['extendflag']  = False
        a['iteraxis']  = ''
        a['customsymbol']  = False
        a['coloraxis']  = ''
        a['customflaggedsymbol']  = False
        a['plotrange']  = []
        a['title']  = ''
        a['xlabel']  = ''
        a['ylabel']  = ''
        a['showmajorgrid']  = False
        a['showminorgrid']  = False
        a['showlegend']  = False
        a['plotfile']  = ''
        a['showgui']  = True

        a['xaxis'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'amp'}, {'xdatacolumn':''}]), 
                    2:odict([{'value':'amplitude'}, {'xdatacolumn':''}]), 
                    3:odict([{'value':'phase'}, {'xdatacolumn':''}]), 
                    4:odict([{'value':'real'}, {'xdatacolumn':''}]), 
                    5:odict([{'value':'imag'}, {'xdatacolumn':''}]), 
                    6:odict([{'value':'imaginary'}, {'xdatacolumn':''}]), 
                    7:{'value':'scan'}, 
                    8:{'value':'field'}, 
                    9:{'value':'time'}, 
                    10:{'value':'time_interval'}, 
                    11:{'value':'timeinterval'}, 
                    12:{'value':'timeint'}, 
                    13:{'value':'spw'}, 
                    14:{'value':'channel'}, 
                    15:{'value':'chan'}, 
                    16:{'value':'frequency'}, 
                    17:{'value':'freq'}, 
                    18:{'value':'correlation'}, 
                    19:{'value':'corr'}, 
                    20:{'value':'antenna1'}, 
                    21:{'value':'ant1'}, 
                    22:{'value':'antenna2'}, 
                    23:{'value':'ant2'}, 
                    24:{'value':'baseline'}, 
                    25:{'value':'uvdist'}, 
                    26:{'value':'uvwave'}, 
                    27:{'value':'uvdist_l'}, 
                    28:{'value':'uvdistl'}, 
                    29:{'value':'u'}, 
                    30:{'value':'v'}, 
                    31:{'value':'w'}, 
                    32:{'value':'uwave'}, 
                    33:{'value':'vwave'}, 
                    34:{'value':'wwave'}, 
                    35:{'value':'velocity'}, 
                    36:{'value':'vel'}, 
                    37:{'value':'wt'}, 
                    38:{'value':'wtsp'}, 
                    39:{'value':'weight'}, 
                    40:{'value':'weightspectrum'}, 
                    41:{'value':'imwt'}, 
                    42:{'value':'flag'}, 
                    43:{'value':'azimuth'}, 
                    44:{'value':'elevation'}, 
                    45:{'value':'hourang'}, 
                    46:{'value':'hourangle'}, 
                    47:{'value':'parang'}, 
                    48:{'value':'parangle'}, 
                    49:{'value':'parallacticangle'}, 
                    50:{'value':'antenna'}, 
                    51:{'value':'ant'}, 
                    52:{'value':'ant-azimuth'}, 
                    53:{'value':'ant-elevation'}, 
                    54:{'value':'ant-hourang'}, 
                    55:{'value':'ant-hourangle'}, 
                    56:{'value':'ant-parang'}, 
                    57:{'value':'ant-parangle'}, 
                    58:{'value':'ant-parallacticangle'}, 
                    59:{'value':'row'}, 
                    60:{'value':'flagrow'}}
        a['yaxis'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'amp'}, {'ydatacolumn':''}]), 
                    2:odict([{'value':'amplitude'}, {'ydatacolumn':''}]), 
                    3:odict([{'value':'phase'}, {'ydatacolumn':''}]), 
                    4:odict([{'value':'real'}, {'ydatacolumn':''}]), 
                    5:odict([{'value':'imag'}, {'ydatacolumn':''}]), 
                    6:odict([{'value':'imaginary'}, {'ydatacolumn':''}]), 
                    7:{'value':'scan'}, 
                    8:{'value':'field'}, 
                    9:{'value':'time'}, 
                    10:{'value':'time_interval'}, 
                    11:{'value':'timeinterval'}, 
                    12:{'value':'timeint'}, 
                    13:{'value':'spw'}, 
                    14:{'value':'channel'}, 
                    15:{'value':'chan'}, 
                    16:{'value':'frequency'}, 
                    17:{'value':'freq'}, 
                    18:{'value':'correlation'}, 
                    19:{'value':'corr'}, 
                    20:{'value':'antenna1'}, 
                    21:{'value':'ant1'}, 
                    22:{'value':'antenna2'}, 
                    23:{'value':'ant2'}, 
                    24:{'value':'baseline'}, 
                    25:{'value':'uvdist'}, 
                    26:{'value':'uvwave'}, 
                    27:{'value':'uvdist_l'}, 
                    28:{'value':'uvdistl'}, 
                    29:{'value':'u'}, 
                    30:{'value':'v'}, 
                    31:{'value':'w'}, 
                    32:{'value':'uwave'}, 
                    33:{'value':'vwave'}, 
                    34:{'value':'wwave'}, 
                    35:{'value':'velocity'}, 
                    36:{'value':'vel'}, 
                    37:{'value':'wt'}, 
                    38:{'value':'wtsp'}, 
                    39:{'value':'weight'}, 
                    40:{'value':'weightspectrum'}, 
                    41:{'value':'imwt'}, 
                    42:{'value':'flag'}, 
                    43:{'value':'azimuth'}, 
                    44:{'value':'elevation'}, 
                    45:{'value':'hourang'}, 
                    46:{'value':'hourangle'}, 
                    47:{'value':'parang'}, 
                    48:{'value':'parangle'}, 
                    49:{'value':'parallacticangle'}, 
                    50:{'value':'antenna'}, 
                    51:{'value':'ant'}, 
                    52:{'value':'ant-azimuth'}, 
                    53:{'value':'ant-elevation'}, 
                    54:{'value':'ant-hourang'}, 
                    55:{'value':'ant-hourangle'}, 
                    56:{'value':'ant-parang'}, 
                    57:{'value':'ant-parangle'}, 
                    58:{'value':'ant-parallacticangle'}, 
                    59:{'value':'row'}, 
                    60:{'value':'flagrow'}}
        a['selectdata'] = {
                    0:odict([{'value':True}, {'field':''}, {'spw':''}, {'timerange':''}, {'uvrange':''}, {'antenna':''}, {'scan':''}, {'correlation':''}, {'array':''}, {'observation':''}, {'msselect':''}]), 
                    1:{'value':False}}
        a['averagedata'] = {
                    0:odict([{'value':True}, {'avgchannel':''}, {'avgtime':''}, {'avgscan':False}, {'avgfield':False}, {'avgbaseline':False}, {'avgantenna':False}, {'avgspw':False}, {'scalar':False}]), 
                    1:{'value':False}}
        a['transform'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'freqframe':''}, {'restfreq':''}, {'veldef':'RADIO'}, {'shift':[0.0, 0.0]}])}
        a['extendflag'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'extcorr':False}, {'extchannel':False}])}
        a['iteraxis'] = {
                    0:odict([{'notvalue':''}, {'xselfscale':False}, {'yselfscale':False}, {'xsharedaxis':False}, {'ysharedaxis':False}])}
        a['customsymbol'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'symbolshape':'autoscaling'}, {'symbolsize':2}, {'symbolcolor':'0000ff'}, {'symbolfill':'fill'}, {'symboloutline':False}])}
        a['customflaggedsymbol'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'flaggedsymbolshape':'nosymbol'}, {'flaggedsymbolsize':2}, {'flaggedsymbolcolor':'ff0000'}, {'flaggedsymbolfill':'fill'}, {'flaggedsymboloutline':False}])}
        a['plotfile'] = {
                    0:odict([{'notvalue':''}, {'expformat':''}, {'exprange':''}, {'highres':False}, {'overwrite':False}])}
        a['showgui'] = {
                    0:odict([{'value':True}, {'clearplots':True}]), 
                    1:odict([{'value':False}, {'clearplots':True}])}
        a['showmajorgrid'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'majorwidth':0}, {'majorstyle':''}, {'majorcolor':''}])}
        a['showminorgrid'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'minorwidth':0}, {'minorstyle':''}, {'minorcolor':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='plotms', subkey=None):
        desc={'plotms': 'A plotter/interactive flagger for visibility data.',
               'vis': 'input MS (or CalTable) (blank for none)',
               'gridrows': 'Number of subplot rows (default 1).',
               'gridcols': 'Number of subplot columns (default 1).',
               'rowindex': 'Row location of the plot (0-based, default 0)',
               'colindex': 'Column location of the plot (0-based, default 0)',
               'plotindex': 'Index to address a subplot (0-based, default 0)',
               'xaxis': 'plot x-axis (blank for default/current)',
               'xdatacolumn': 'data column to use for x-axis (blank for default/current)',
               'yaxis': 'plot y-axis (blank for default/current)',
               'ydatacolumn': 'data column to use for y-axis (blank for default/current)',
               'yaxislocation': 'whether to use a left or right y-axis for the data (blank for default)',
               'selectdata': 'data selection parameters',
               'field': 'field names or field index numbers (blank for all)',
               'spw': 'spectral windows:channels (blank for all)',
               'timerange': 'time range (blank for all)',
               'uvrange': 'uv range (blank for all)',
               'antenna': 'antenna/baselines (blank for all)',
               'scan': 'scan numbers (blank for all)',
               'correlation': 'correlations (blank for all)',
               'array': '(sub)array numbers (blank for all)',
               'observation': 'Select by observation ID(s)',
               'msselect': 'MS selection (blank for all)',
               'averagedata': 'data averaging parameters',
               'avgchannel': 'average over channel?  (blank = False, otherwise value in channels)',
               'avgtime': 'average over time? (blank = False, other value in seconds)',
               'avgscan': 'only valid if time averaging is turned on.  average over scans?',
               'avgfield': 'only valid if time averaging is turned on.  average over fields?',
               'avgbaseline': 'average over all baselines?  (mutually exclusive with avgantenna)',
               'avgantenna': 'average by per-antenna?  (mutually exclusive with avgbaseline)',
               'avgspw': 'average over all spectral windows?',
               'scalar': 'Do scalar averaging?',
               'transform': 'transform data in various ways?',
               'freqframe': 'the frame in which to render frequency and velocity axes ',
               'restfreq': 'Rest frequency to use for velocity conversions ',
               'veldef': 'the definition in which to render velocity ',
               'shift': 'Adjust phases by this phase center shift [dx,dy] (arcsec)',
               'extendflag': 'have flagging extend to other data points?',
               'extcorr': 'extend flags based on correlation? ',
               'extchannel': 'extend flags based on channel?',
               'iteraxis': 'the axis over which to iterate',
               'xselfscale': 'If true, iterated plots should share a common x-axis label per column.',
               'yselfscale': 'If true, iterated plots should share a common y-axis label per row.',
               'xsharedaxis': 'Plots should share a common x-axis. ',
               'ysharedaxis': 'Plots should share a common y-axis.',
               'customsymbol': 'set a custom symbol(s) for unflagged points',
               'symbolshape': 'shape of plotted unflagged symbols',
               'symbolsize': 'size of plotted unflagged symbols',
               'symbolcolor': 'color of plotted unflagged symbols',
               'symbolfill': 'fill type of plotted unflagged symbols',
               'symboloutline': 'selects outlining plotted unflagged points',
               'coloraxis': 'selects which data to use for colorizing',
               'customflaggedsymbol': 'set a custom plot symbol for flagged points',
               'flaggedsymbolshape': 'shape of plotted flagged symbols',
               'flaggedsymbolsize': 'size of plotted flagged symbols',
               'flaggedsymbolcolor': 'color of plotted flagged symbols',
               'flaggedsymbolfill': 'fill type of plotted flagged symbols',
               'flaggedsymboloutline': 'selects outlining plotted flagged points',
               'plotrange': 'plot axes ranges: [xmin,xmax,ymin,ymax]',
               'title': 'Title written along top of plot',
               'xlabel': 'Text for horizontal axis. Blank for default.',
               'ylabel': 'Text for vertical axis. Blank for default.',
               'showmajorgrid': 'Show major grid lines (horiz and vert.)',
               'majorwidth': 'Line width in pixels of major grid lines',
               'majorstyle': 'Major grid line style: solid dash dot none',
               'majorcolor': 'Color as name or hex code of major grid lines',
               'showminorgrid': 'Show minor grid lines (horiz and vert.)',
               'minorwidth': 'Line width in pixels of minor grid lines',
               'minorstyle': 'Minor grid line style: solid dash dot none',
               'minorcolor': 'Color as name or hex code of minor grid lines',
               'showlegend': 'Show a legend on the plot.',
               'legendposition': 'Legend position.',
               'plotfile': 'Name of plot file to save automatically.',
               'expformat': 'Export format type (jpg, png, ps, pdf, txt), if not provided, plotfile extension will be used.',
               'exprange': 'Export all iteration plots or only the current one.',
               'highres': 'Use high resolution? Always true for jpg and png.',
               'overwrite': 'Overwrite plot file if it already exists?',
               'showgui': 'Show GUI',
               'clearplots': 'Remove any existing plots so new ones can replace them.',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['gridrows']  = 1
        a['gridcols']  = 1
        a['rowindex']  = 0
        a['colindex']  = 0
        a['plotindex']  = 0
        a['xaxis']  = ''
        a['xdatacolumn']  = ''
        a['yaxis']  = '', []
        a['ydatacolumn']  = ''
        a['yaxislocation']  = ''
        a['selectdata']  = True
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['uvrange']  = ''
        a['antenna']  = ''
        a['scan']  = ''
        a['correlation']  = ''
        a['array']  = ''
        a['observation']  = ''
        a['msselect']  = ''
        a['averagedata']  = True
        a['avgchannel']  = ''
        a['avgtime']  = ''
        a['avgscan']  = False
        a['avgfield']  = False
        a['avgbaseline']  = False
        a['avgantenna']  = False
        a['avgspw']  = False
        a['scalar']  = False
        a['transform']  = False
        a['freqframe']  = ''
        a['restfreq']  = ''
        a['veldef']  = 'RADIO'
        a['shift']  = [0.0, 0.0]
        a['extendflag']  = False
        a['extcorr']  = False
        a['extchannel']  = False
        a['iteraxis']  = ''
        a['xselfscale']  = False
        a['yselfscale']  = False
        a['xsharedaxis']  = False
        a['ysharedaxis']  = False
        a['customsymbol']  = False
        a['symbolshape']  = 'autoscaling'
        a['symbolsize']  = 2
        a['symbolcolor']  = '0000ff'
        a['symbolfill']  = 'fill'
        a['symboloutline']  = False
        a['coloraxis']  = ''
        a['customflaggedsymbol']  = False
        a['flaggedsymbolshape']  = 'nosymbol'
        a['flaggedsymbolsize']  = 2
        a['flaggedsymbolcolor']  = 'ff0000'
        a['flaggedsymbolfill']  = 'fill'
        a['flaggedsymboloutline']  = False
        a['plotrange']  = []
        a['title']  = ''
        a['xlabel']  = ''
        a['ylabel']  = ''
        a['showmajorgrid']  = False
        a['majorwidth']  = 1
        a['majorstyle']  = ''
        a['majorcolor']  = 'B0B0B0'
        a['showminorgrid']  = False
        a['minorwidth']  = 1
        a['minorstyle']  = ''
        a['minorcolor']  = 'D0D0D0'
        a['showlegend']  = False
        a['legendposition']  = ''
        a['plotfile']  = ''
        a['expformat']  = ''
        a['exprange']  = ''
        a['highres']  = False
        a['overwrite']  = False
        a['showgui']  = True
        a['clearplots']  = True

        if a.has_key(paramname) :
	      return a[paramname]
plotms_pg = plotms_pg_()
