#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdbaselineold import sdbaselineold
from task_sdbaselineold import casalog

class sdbaselineold_pg_:
    __name__ = "sdbaselineold"

    def __init__(self) :
       self.__bases__ = (sdbaselineold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,fluxunit=None,telescopeparm=None,specunit=None,restfreq=None,frame=None,doppler=None,scanlist=None,field=None,iflist=None,pollist=None,tau=None,masklist=None,maskmode=None,thresh=None,avg_limit=None,edge=None,blfunc=None,order=None,npiece=None,applyfft=None,fftmethod=None,fftthresh=None,addwn=None,rejwn=None,clipthresh=None,clipniter=None,verify=None,verbose=None,bloutput=None,blformat=None,showprogress=None,minnrow=None,outfile=None,outform=None,overwrite=None,plotlevel=None,):

        """ASAP SD task: fit/subtract a spectral baseline 
        Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        fluxunit -- units for line flux
                options: 'K','Jy',''
                default: '' (keep current fluxunit)
                WARNING: For GBT data, see description below.
            >>> fluxunit expandable parameter
                 telescopeparm -- the telescope characteristics
                        options: (str) name or (list) list of gain info
                        default: '' (none set)
                        example: if telescopeparm='', it tries to get the telescope
                                 name from the data.
                                 Full antenna parameters (diameter,ap.eff.) known
                                 to ASAP are
                                 'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                                 'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                                 to 'K' first then convert to a new fluxunit.
                                 telescopeparm=[104.9,0.43] diameter(m), ap.eff.
                                 telescopeparm=[0.743] gain in Jy/K
                                 telescopeparm='FIX' to change default fluxunit
                                 see description below
        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz',''
                default: '' (=current)
                example: this will be the units for masklist
            >>> specunit expandable parameters
                 restfreq -- rest frequency
                         available type includes float, int, string, list of float, 
                         list of int, list of string, and list of dictionary. the 
                         default unit of restfreq in case of float, int, or string 
                         without unit is Hz. string input can be a value only 
                         (treated as Hz) or a value followed by unit for which 'GHz',
                         'MHz','kHz',and 'Hz' are available. 
                         a list can be used to set different rest frequencies for 
                         each IF. the length of list input must be nIF. dictionary 
                         input should be a pair of molecule name and frequency with 
                         keys of 'name' and 'value', respectively. values in the 
                         dictionary input follows the same manner as for single 
                         float or string input. 
                         example: 345.796
                                  '1420MHz'
                                  [345.8, 347.0, 356.7]
                                  ['345.8MHz', '347.0MHz', '356.7MHz']
                                  [{'name':'CO','value':345}]
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
        tau -- atmospheric optical depth
                default: 0.0 (no correction)
        masklist -- list or string of mask regions to INCLUDE in BASELINE fit
                    a string masklist allows per IF selection of channels.
                default: [] (entire spectrum)
                example: [[1000,3000],[5000,7000]]
                         '0:1000~3000;5000~7000, 1:200~350;450~600'
                         when maskmode is 'auto' or 'interact', this mask 
                         will be applied first before fitting as base mask
        maskmode -- mode of setting additional channel masks
                options: (str) 'auto','list','interact'
                default: 'auto'
                example: maskmode='auto' runs linefinder to detect line regions 
                         to be excluded from fitting. this mode requires three 
                         expandable parameters: thresh, avg_limit, and edge.
                         USE WITH CARE! May need to tweak the expandable parameters.
                         maskmode='list' uses the given masklist only: no additional 
                         masks applied.
                         maskmode='interact' allows users to manually modify the 
                         mask regions by dragging mouse on the spectrum plotter GUI.
                         use LEFT or RIGHT button to add or delete regions, 
                         respectively.
               
            >>> maskmode expandable parameters
                 thresh -- S/N threshold for linefinder
                         default: 5
                         example: a single channel S/N ratio above which the channel is
                                  considered to be a detection
                 avg_limit -- channel averaging for broad lines
                         default: 4
                         example: a number of consecutive channels not greater than
                                  this parameter can be averaged to search for broad lines
                 edge -- channels to drop at beginning and end of spectrum
                         default: 0
                         example: [1000] drops 1000 channels at beginning AND end
                                  [1000,500] drops 1000 from beginning and 500 from end
                 Note: For bad baselines threshold should be increased,
                 and avg_limit decreased (or even switched off completely by
                 setting this parameter to 1) to avoid detecting baseline
                 undulations instead of real lines.
        blfunc -- baseline model function
                options: (str) 'poly','chebyshev','cspline','sinusoid'
                default: 'poly'
                example: blfunc='poly' uses a single polynomial line of 
                         any order which should be given as an expandable 
                         parameter 'order' to fit baseline. 
                         blfunc='chebyshev' uses Chebyshev polynomials. 
                         blfunc='cspline' uses a cubic spline function, a piecewise 
                         cubic polynomial having C2-continuity (i.e., the second 
                         derivative is continuous at the joining points). 
                         blfunc='sinusoid' uses a combination of sinusoidal curves. 
            >>> blfunc expandable parameters
                 order -- order of baseline polynomial
                         options: (int) (<0 turns off baseline fitting)
                         default: 5
                         example: typically in range 2-9 (higher values
                                  seem to be needed for GBT)
                 npiece -- number of the element polynomials of cubic spline curve
                         options: (int) (<0 turns off baseline fitting)
                         default: 2
                 applyfft -- automatically set wave numbers of sinusoidal functions 
                         for fitting by applying some method like FFT.
                         options: (bool) True, False
                         default: True
                 fftmethod -- method to be used when applyfft=True. Now only 
                         'fft' is available and it is the default.
                 fftthresh -- threshold to select wave numbers to be used for 
                         sinusoidal fitting. both (float) and (str) accepted.
                         given a float value, the unit is set to sigma.
                         for string values, allowed formats include:
                           'xsigma' or 'x' (= x-sigma level. e.g., '3sigma'), or
                           'topx' (= the x strongest ones, e.g. 'top5').
                         default is 3.0 (unit: sigma).
                 addwn -- additional wave number(s) of sinusoids to be used 
                         for fitting. 
                         (list) and (int) are accepted to specify every
                         wave numbers. also (str) can be used in case
                         you need to specify wave numbers in a certain range,
                         e.g., 'a-b' (= a, a+1, a+2, ..., b-1, b),
                               '<a'  (= 0,1,...,a-2,a-1),
                               '>=a' (= a, a+1, ... up to the maximum wave
                                      number corresponding to the Nyquist
                                      frequency for the case of FFT).
                         default: [0] (i.e., constant is subtracted at least)
                 rejwn -- wave number(s) of sinusoid NOT to be used for fitting.
                         can be set just as addwn but has higher priority:
                         wave numbers which are specified both in addwn
                         and rejwn will NOT be used. 
                         default: []
                 clipthresh -- clipping threshold for iterative fitting
                         default: 3
                 clipniter -- maximum iteration number
                         default: 0 (no iteration, i.e., no clipping)
        verify -- interactively verify the results of baseline fitting for each spectrum.
                  When verify = True, for each input spectrum the baseline
                  fit function and spectra before and after the fit are 
                  displayed in a plot window. At the prompt there are four
                  choices of action: 'Y' (accept the fit and continue to the
                  next input spectrum), 'N' (reject the fit and continue to the
                  next input spectrum), 'A' (accept the current fit and continue
                  non-interactively), and 'R' (reject the current fit and exit
                  from baseline fitting).
                  Note that when the baseline fit is rejected by 'N' or 'R',
                  no baseline fit is applied to the spectrum/spectra. 
                options: (bool) True,False
                default: False
                NOTE: Currently available only when blfunc='poly'
        verbose -- output fitting results to logger
                default: True
                example: If False, the fitting results including coefficients, 
                         residual rms, etc., are not output to the CASA logger, 
                         while the processing speed gets faster
        bloutput -- output fitting results to a text file
                default: True
                example: If False, the fitting results including coefficients, 
                         residual rms, etc., are not output to a text file 
                         (<outfile>_blparam.txt), while the processing 
                         speed gets faster
        blformat -- format of the logger output and text file specified with bloutput
                options: '', 'csv'
                default: '' (same as in the past, easy to read but huge)
        showprogress -- show progress status for large data
                default: True
        minnrow -- minimum number of input spectra to show progress status
                default: 1000
        outfile -- Name of output file
                default: '' (<infile>_bs)
        outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                         If ASCII, then will append some stuff to
                         the outfile name
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored
        plotlevel -- control for plotting of results
                options: (int) 0=none, 1=some, 2=more, <0=hardcopy
                default: 0 (no plotting)
                example: plotlevel<0 as abs(plotlevel), e.g.
                         -1 => hardcopy of final plot (will be named
                        <outfile>_bspec.eps)
                WARNING: be careful plotting in fsotf mode!



        DESCRIPTION:

        Task sdbaselineold performs baseline fitting/removal for single-dish spectra.
        The fit parameters, terms and rms of base-line are saved to an ascii 
        file, '<outfile>_blparam.txt'. 

        ASAP recognizes the data of the "AT" telescopes, but currently
        does not know about the GBT or any other telescope. This task
        does know about GBT. Telescope name is obtained from the data.
        If you wish to change the fluxunit (see below), and telescopeparm='',
        for the AT telescopes it will use internal telescope parameters for
        flux conversion. For GBT, it will use an approximate aperture
        efficiency conversion.  If you give telescopeparm a list , 
        then if the list has a single float it is assumed to
        be the gain in Jy/K, if two or more elements they are assumed
        to be telescope diameter (m) and aperture efficiency
        respectively.

        Note that sdbaselineold assumes that the fluxunit is set correctly in
        the data already.  If not, then set telescopeparm='FIX' and it
        will set the default units to fluxunit without conversion.
        NOTE: If the data in infile is an ms from GBT and the default flux
        unit is missing and this task automatically fixes the default fluxunit
        to 'K' before the conversion.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.         

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdbaselineold'
        myf['taskname'] = 'sdbaselineold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['fluxunit'] = fluxunit = myf['fluxunit']
            myparams['telescopeparm'] = telescopeparm = myf['telescopeparm']
            myparams['specunit'] = specunit = myf['specunit']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['tau'] = tau = myf['tau']
            myparams['masklist'] = masklist = myf['masklist']
            myparams['maskmode'] = maskmode = myf['maskmode']
            myparams['thresh'] = thresh = myf['thresh']
            myparams['avg_limit'] = avg_limit = myf['avg_limit']
            myparams['edge'] = edge = myf['edge']
            myparams['blfunc'] = blfunc = myf['blfunc']
            myparams['order'] = order = myf['order']
            myparams['npiece'] = npiece = myf['npiece']
            myparams['applyfft'] = applyfft = myf['applyfft']
            myparams['fftmethod'] = fftmethod = myf['fftmethod']
            myparams['fftthresh'] = fftthresh = myf['fftthresh']
            myparams['addwn'] = addwn = myf['addwn']
            myparams['rejwn'] = rejwn = myf['rejwn']
            myparams['clipthresh'] = clipthresh = myf['clipthresh']
            myparams['clipniter'] = clipniter = myf['clipniter']
            myparams['verify'] = verify = myf['verify']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['bloutput'] = bloutput = myf['bloutput']
            myparams['blformat'] = blformat = myf['blformat']
            myparams['showprogress'] = showprogress = myf['showprogress']
            myparams['minnrow'] = minnrow = myf['minnrow']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]
        if type(edge)==int: edge=[edge]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparm'] = telescopeparm
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['tau'] = tau
        mytmp['masklist'] = masklist
        mytmp['maskmode'] = maskmode
        mytmp['thresh'] = thresh
        mytmp['avg_limit'] = avg_limit
        mytmp['edge'] = edge
        mytmp['blfunc'] = blfunc
        mytmp['order'] = order
        mytmp['npiece'] = npiece
        mytmp['applyfft'] = applyfft
        mytmp['fftmethod'] = fftmethod
        mytmp['fftthresh'] = fftthresh
        mytmp['addwn'] = addwn
        mytmp['rejwn'] = rejwn
        mytmp['clipthresh'] = clipthresh
        mytmp['clipniter'] = clipniter
        mytmp['verify'] = verify
        mytmp['verbose'] = verbose
        mytmp['bloutput'] = bloutput
        mytmp['blformat'] = blformat
        mytmp['showprogress'] = showprogress
        mytmp['minnrow'] = minnrow
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdbaselineold.xml')

        casalog.origin('sdbaselineold')
        if not trec.has_key('sdbaselineold') or not casac.utils().verify(mytmp, trec['sdbaselineold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdbaselineold           #####')
          casalog.post('')
          result = sdbaselineold(infile, antenna, fluxunit, telescopeparm, specunit, restfreq, frame, doppler, scanlist, field, iflist, pollist, tau, masklist, maskmode, thresh, avg_limit, edge, blfunc, order, npiece, applyfft, fftmethod, fftthresh, addwn, rejwn, clipthresh, clipniter, verify, verbose, bloutput, blformat, showprogress, minnrow, outfile, outform, overwrite, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdbaselineold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdbaselineold', 'sdbaselineold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdbaselineold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdbaselineold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['tau']  = 0.0
        a['masklist']  = []
        a['maskmode']  = ''
        a['blfunc']  = 'poly'
        a['verify']  = False
        a['verbose']  = True
        a['bloutput']  = True
        a['blformat']  = ''
        a['showprogress']  = True
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparm':''}]), 
                    2:odict([{'value':'k'}, {'telescopeparm':''}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparm':''}]), 
                    4:odict([{'value':'jy'}, {'telescopeparm':''}])}
        a['specunit'] = {
                    0:{'value':''}, 
                    1:{'value':'channel'}, 
                    2:{'value':'GHz'}, 
                    3:{'value':'MHz'}, 
                    4:{'value':'kHz'}, 
                    5:{'value':'Hz'}, 
                    6:odict([{'value':'km/s'}, {'restfreq':''}])}
        a['maskmode'] = {
                    0:odict([{'value':'auto'}, {'thresh':5.0}, {'avg_limit':4}, {'edge':[0]}]), 
                    1:{'value':'list'}, 
                    2:{'value':'interact'}}
        a['blfunc'] = {
                    0:odict([{'value':'poly'}, {'order':5}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    1:odict([{'value':'chebyshev'}, {'order':5}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    2:odict([{'value':'cspline'}, {'npiece':2}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    3:odict([{'value':'sinusoid'}, {'applyfft':True}, {'fftmethod':'fft'}, {'fftthresh':3.0}, {'addwn':[0]}, {'rejwn':[]}, {'clipthresh':3.0}, {'clipniter':0}])}
        a['showprogress'] = {
                    0:odict([{'value':True}, {'minnrow':1000}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdbaselineold', subkey=None):
        desc={'sdbaselineold': 'ASAP SD task: fit/subtract a spectral baseline ',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'fluxunit': 'units for line flux (K,Jy) (''=current)',
               'telescopeparm': 'param of telescope for flux conversion',
               'specunit': 'units for spectral axis (channel,km/s,GHz)',
               'restfreq': 'rest frequency (default unit: Hz)',
               'frame': 'frequency reference frame, e.g. LSRK (''=current)',
               'doppler': 'doppler convention, e.g. RADIO (''=current)',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'tau': 'atmospheric optical depth for correction',
               'masklist': 'list or string of mask regions to INCLUDE in BASELINE fit',
               'maskmode': 'mode of setting additional channel masks',
               'thresh': 'S/N threshold for linefinder',
               'avg_limit': 'channel averaging for broad lines',
               'edge': 'channels to drop at beginning and end of spectrum',
               'blfunc': 'baseline model function',
               'order': 'order of baseline polynomial',
               'npiece': 'number of element polynomials of spline',
               'applyfft': 'automatically set wave numbers of sinusoids',
               'fftmethod': 'method for automatically set wave numbers of sinusoids',
               'fftthresh': 'threshold to select wave numbers of sinusoids',
               'addwn': 'additional wave numbers to use',
               'rejwn': 'wave numbers NOT to use',
               'clipthresh': 'clipping threshold for iterative fitting',
               'clipniter': 'maximum iteration number',
               'verify': 'interactively verify the results of baseline fitting for each spectrum. (only for blfunc="poly")',
               'verbose': 'output fitting results to logger',
               'bloutput': 'output fitting results to a text file',
               'blformat': 'format of the text file specified with bloutput',
               'showprogress': 'show progress status for large data',
               'minnrow': 'minimum number of input spectra to show progress status',
               'outfile': 'output file name',
               'outform': 'output file format (ASCII,MS,SDFITS,ASAP)',
               'overwrite': 'overwrite the output file if already exists',
               'plotlevel': 'plot results (0=none,1+=some,<0=hardcopy)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparm']  = ''
        a['specunit']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['tau']  = 0.0
        a['masklist']  = []
        a['maskmode']  = ''
        a['thresh']  = 5.0
        a['avg_limit']  = 4
        a['edge']  = [0]
        a['blfunc']  = 'poly'
        a['order']  = 5
        a['npiece']  = 2
        a['applyfft']  = True
        a['fftmethod']  = 'fft'
        a['fftthresh']  = 3.0
        a['addwn']  = [0]
        a['rejwn']  = []
        a['clipthresh']  = 3.0
        a['clipniter']  = 0
        a['verify']  = False
        a['verbose']  = True
        a['bloutput']  = True
        a['blformat']  = ''
        a['showprogress']  = True
        a['minnrow']  = 1000
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdbaselineold_pg = sdbaselineold_pg_()
