#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdcal2old import sdcal2old
from task_sdcal2old import casalog

class sdcal2old_pg_:
    __name__ = "sdcal2old"

    def __init__(self) :
       self.__bases__ = (sdcal2old_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,calmode=None,fraction=None,noff=None,width=None,elongated=None,tsysiflist=None,applytable=None,interp=None,ifmap=None,scanlist=None,field=None,iflist=None,pollist=None,outfile=None,overwrite=None,):

        """ASAP SD task: generate and/or apply caltables
  Keyword arguments:
        infile -- Name of input SD dataset
        calmode -- Calibration mode. If you want to generate calibration 
                   table or apply existing calibration tables, set calmode 
                   to simple string. On the other hand, if you want to 
                   calibrate data on-the-fly, you have to set calmode 
                   to a composite calmode string separated by comma.
                   So far, sky calibration has three types, 'ps', 'otf',
                   and 'otfraster'. If observation is configured to 
                   observe reference position, calmode must be 'ps'. 
                   Otherwise, 'otf' or 'otfraster' should be used
                   depending on observing pattern. If observing pattern
                   is raster scan, calmode must be 'otfraster' while 
                   'otf' must be used when observing pattern is non-
                   raster (e.g., Lissajous).
                options: 'ps','otf','otfraster','tsys','apply'
                default: 'ps'
                example: Here is an example for composite calmode.
                         'ps,apply' (do sky cal and apply)
                         'ps,tsys,apply' (do sky and Tsys cal and apply)
            >>> calmode expandable parameter
                 fraction -- Edge marker parameter of 'otf' and 'otfraster'.
                             Specify a number of OFF integrations (at each
                             side of the raster rows in 'otfraster' mode)
                             as a fraction of total number of integrations.
                             In 'otfraster' mode, number of integrations 
                             to be marked as OFF, n_off, is determined by 
                             the following formula,

                                n_off = floor(fraction * n),

                             where n is number of integrations per raster 
                             row. Note that n_off from both sides will be  
                             marked as OFF so that twice of specified 
                             fraction will be marked at most. For example, 
                             if you specify fraction='10%', resultant 
                             fraction of OFF integrations will be 20% at 
                             most.
                             In 'otf' mode, n_off is given by,

                                n_off = floor(fraction * n),

                             where n is number of total integrations.
                             n_off is used as criterion of iterative marking
                             process. Therefore, resulting total number of 
                             OFFs will be larger than n_off. In practice,
                             fraction is a geometrical fraction of edge
                             region. Thus, if integrations are concentrated
                             on edge region (e.g. some of Lissajous
                             patterns), then resulting n_off may be 
                             unexpectedly large.
                         default: '10%'
                         options: '20%' in string style or float value less 
                                  than 1.0 (e.g. 0.15).
                                  'auto' is available only for 'otfraster'. 
                 noff -- Edge marking parameter for 'otfraster'.
                         It is used to specify a number of OFF spectra near 
                         edge directly. Value of noff comes before setting 
                         by fraction. Note that n_off from both sides will 
                         be marked as OFF so that twice of specified noff 
                         will be marked at most.
                         default: -1 (use fraction)
                         options: any positive integer
                 width -- Edge marking parameter for 'otf'.
                          Pixel width with respect to a median spatial 
                          separation between neighboring two data in time.
                          Default will be fine in most cases.
                         default: 0.5
                         options: float value
                 elongated -- Edge marking parameter for 'otf'.
                              Set True only if observed area is elongated 
                              in one direction.
                         default: False
                 tsysiflist -- List of IFNOs for Tsys calibration. It does 
                               no effect if you don't want to do Tsys 
                               calibration.
                         default: [] (auto-detect tsys spws)
                 applytable -- List of sky/Tsys calibration tables you want to 
                               apply.
                         default: ''
                 interp -- Interpolation method in time and frequency axis. 
                           Set comma separated method strings if you want 
                           to use different interpolation in time and 
                           frequency. 
                         options: 'linear', 'cspline', 'nearest', 
                                  any numeric string indicating an order 
                                  of polynomial.
                         default: '' (linear in time and frequency)
                         example: 'linear,cspline' (linear in time, cubic 
                                                    spline in frequency)
                                  'linear,3' (linear in time, third order 
                                              polynomial in frequency)
                                  'nearest' (nearest in time and frequency)
                 ifmap -- Dictionary defining transfer of Tsys calibration. 
                          Key must be IFNO for Tsys and its value must be 
                          a list of IFNOs for science target.
                         default: {}
                         example: {1: [5,6], 3: [7,8]}
                                  Tsys in IFNO1 is transferred to IFNO5, 6 
                                  while Tsys in IFNO3 is to IFNO7, 8.
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
        outfile -- Name of output file. If you omit, behavior of the task 
                   depends on calmode. If calmode includes 'apply', then 
                   omitting outfile indicates that infile is overwritten 
                   by the calibrated data. In this case, you have to set 
                   overwrite to True. If calmode doesn't include 'apply', 
                   omitting outfile indicates that the task will use default 
                   outfile name based on infile and predefined suffix 
                   ('_sky' for sky, '_tsys' for Tsys).
                default: '' (<infile>_<suffix> for calibration 
                             while overwrite infile for apply mode)
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False


        DESCRIPTION:

        Task sdcal2old is an implementation of a calibration scheme like as 
        interferometry, i.e., generate caltables and apply them. Available 
        calibration modes are 'ps', 'otf', 'otfraster', and 'tsys'. Those 
        modes generates caltables for sky or Tsys calibration. Those 
        caltables can be applied to the data by using calmode 'apply'.

        The above three calibration modes, 'ps', 'otf', and 'otfraster',
        generate sky calibration tables. The user should choose
        appropriate calibration mode depending on the data. Use case
        for each mode is as follows:

            'ps': position switch (including OTF) with explicit
                  reference (OFF) spectra
            'otf': non-raster OTF scan without explicit OFFs
                   (e.g. Lissajous, double circle, etc.)
                   intends to calibrate fast scan data
            'otfraster': raster OTF scan without explicit OFFs

        So, if the data contains explicit reference spectra, 'ps' should
        be used. Otherwise, 'otfraster' and 'otf' are appropriate for
        raster OTF and non-raster OTF, respectively. In 'otf' and
        'otfraster' modes, the task first try to find several integrations 
        near edge as OFF spectra, then the data are calibrated using those 
        OFFs. If the observing pattern is raster, you 
        should use the 'otfraster' mode to calibrate data. Otherwise, the 
        'otf' mode should be used. For detail about edge marking, see 
        inline help of sd.edgemarker module and its methods.
        Those modes are designed for OTF observations without 
        explicit OFF spectra. However, these modes should work even if
        explicit reference spectra exist. In this case, these spectra
        will be ignored and spectra near edges detected by edge
        marker will be used as reference.

        Except for how to choose OFFs, the procedure to derive calibrated
        spectra is common for the
        above three modes. Selected (or preset) OFF integrations are
        separated by its continuity in time domain, averaged in
        each segment, then interpolated to timestamps for ON integrations.
        Effectively, it means that OFF integrations are averaged by each
        OFF spectrum for 'ps' mode, averaged by either ends of each raster
        row for 'otfraster' mode, averaged by each temporal segments of
        detected edges for 'otf' mode. The formula for calibrated spectrum
        is

            Tsys * (ON - OFF) / OFF. 
  
        You can calibrate data on-the-fly like sdcalold task by setting 
        calmode to a composite calmode string separated by comma. 
        For example, calmode='ps,apply' means doing sky calibration and 
        apply it on-the-fly. In this case, caltable is generated as a 
        temporary plain table and will be deleted at the end.
        Allowed calibration modes in this task is as follows:
  
            ps
                generate sky caltable using 'ps' mode
            otf
                generate sky caltable using 'otf' mode
            otfraster
                generate sky caltable using 'otfraster' mode
            tsys
                generate tsys caltable
            apply
                apply caltables specified by applytable parameter
            ps,apply
                generate temporary sky caltable using 'ps' mode and
                apply it. also apply caltables specified by applytable 
            ps,tsys,apply
                generate temporary sky caltable using 'ps' mode as well
                as temporary tsys caltable, and apply them. 
            otf,apply
                generate temporary sky caltable using 'otf' mode and
                apply it. also apply caltables specified by applytable 
            otf,tsys,apply
                generate temporary sky caltable using 'otf' mode as well
                as temporary tsys caltable, and apply them. 
            otfraster,apply
                generate temporary sky caltable using 'otfraster' mode
                and apply it. also apply caltables specified by applytable 
            otfraster,tsys,apply
                generate temporary sky caltable using 'otfraster' mode
                as well as temporary tsys caltable, and apply them. 

        There are several control parameters for sky/Tsys calibration and 
        application of caltables. See the above parameter description.
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdcal2old'
        myf['taskname'] = 'sdcal2old'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['calmode'] = calmode = myf['calmode']
            myparams['fraction'] = fraction = myf['fraction']
            myparams['noff'] = noff = myf['noff']
            myparams['width'] = width = myf['width']
            myparams['elongated'] = elongated = myf['elongated']
            myparams['tsysiflist'] = tsysiflist = myf['tsysiflist']
            myparams['applytable'] = applytable = myf['applytable']
            myparams['interp'] = interp = myf['interp']
            myparams['ifmap'] = ifmap = myf['ifmap']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(tsysiflist)==int: tsysiflist=[tsysiflist]
        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['calmode'] = calmode
        mytmp['fraction'] = fraction
        mytmp['noff'] = noff
        mytmp['width'] = width
        mytmp['elongated'] = elongated
        mytmp['tsysiflist'] = tsysiflist
        mytmp['applytable'] = applytable
        mytmp['interp'] = interp
        mytmp['ifmap'] = ifmap
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdcal2old.xml')

        casalog.origin('sdcal2old')
        if not trec.has_key('sdcal2old') or not casac.utils().verify(mytmp, trec['sdcal2old']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdcal2old           #####')
          casalog.post('')
          result = sdcal2old(infile, calmode, fraction, noff, width, elongated, tsysiflist, applytable, interp, ifmap, scanlist, field, iflist, pollist, outfile, overwrite)
          casalog.post('')
          casalog.post('##### End Task: sdcal2old           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdcal2old', 'sdcal2old.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdcal2old'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdcal2old', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['calmode']  = 'ps'
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['outfile']  = ''
        a['overwrite']  = False

        a['calmode'] = {
                    0:{'value':'ps'}, 
                    1:odict([{'value':'otf'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}]), 
                    2:odict([{'value':'otfraster'}, {'fraction':'10%'}, {'noff':-1}]), 
                    3:odict([{'value':'tsys'}, {'tsysiflist':[]}]), 
                    4:odict([{'value':'apply'}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    5:odict([{'value':'ps,apply'}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    6:odict([{'value':'ps,tsys,apply'}, {'tsysiflist':[]}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    7:odict([{'value':'otf,apply'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    8:odict([{'value':'otf,tsys,apply'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}, {'tsysiflist':[]}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    9:odict([{'value':'otfraster,apply'}, {'fraction':'10%'}, {'noff':-1}, {'applytable':''}, {'interp':''}, {'ifmap':{}}]), 
                    10:odict([{'value':'otfraster,tsys,apply'}, {'fraction':'10%'}, {'noff':-1}, {'tsysiflist':[]}, {'applytable':''}, {'interp':''}, {'ifmap':{}}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdcal2old', subkey=None):
        desc={'sdcal2old': 'ASAP SD task: generate and/or apply caltables',
               'infile': 'name of input SD dataset (must be in scantable format)',
               'calmode': 'SD calibration mode ("ps","otf","otfraster","tsys","apply",and their possible combinations)',
               'fraction': 'fraction of the OFF data to mark',
               'noff': 'number of the OFF data to mark',
               'width': 'width of the pixel for edge detection',
               'elongated': 'whether observed area is elongated in one direction or not',
               'tsysiflist': 'List of IFNOs for Tsys calibration.',
               'applytable': '(List of) sky and/or tsys tables',
               'interp': 'Interpolation type in time[,freq]. Valid options are "nearest", "linear", "cspline", or any numeric string that indicates an order of polynomial interpolation. You can specify interpolation type for time and frequency separately by joining two of the above options by comma (e.g., "linear,cspline").',
               'ifmap': 'A dictionary indicating IFNO combinations to apply Tsys calibration to target. The key should be IFNO for Tsys calibration and its associated value must be a list of science IFNOs to be applied.',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'outfile': 'output file name',
               'overwrite': 'overwrite the output file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['calmode']  = 'ps'
        a['fraction']  = '10%'
        a['noff']  = -1
        a['width']  = 0.5
        a['elongated']  = False
        a['tsysiflist']  = []
        a['applytable']  = ''
        a['interp']  = ''
        a['ifmap']  = {}
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['outfile']  = ''
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdcal2old_pg = sdcal2old_pg_()
