#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
#from taskmanager import tm
import task_sdflag2old
def sdflag2old(infile='', antenna=0, specunit='', restfreq='', frame='', doppler='', mode='manual', unflag=False, scans=[], field='', timerange='', ifs=[], pols=[], maskflag=[], clipminmax=[], clipoutside=True, showflagged=False, rows=[], outfile='', outform='ASAP', overwrite=True, plotlevel=0):

        """ASAP SD spectral flagging task 
        Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz',''
                default: '' (=current)
                example: this will be the units for maskflag
            >>> specunit expandable parameters
                 restfreq -- rest frequency
                         available type includes float, int, string, list of float, 
                         list of int, list of string, and list of dictionary. the 
                         default unit of restfreq in case of float, int, or string 
                         without unit is Hz. string input can be a value only 
                         (treated as Hz) or a value followed by unit for which 'GHz',
                         'MHz','kHz',and 'Hz' are available. 
                         a list can be used to set different rest frequencies for 
                         each IF. the length of list input must be nIF. dictionary 
                         input should be a pair of molecule name and frequency with 
                         keys of 'name' and 'value', respectively. values in the 
                         dictionary input follows the same manner as for single 
                         float or string input. 
                         example: 345.796
                                  '1420MHz'
                                  [345.8, 347.0, 356.7]
                                  ['345.8MHz', '347.0MHz', '356.7MHz']
                                  [{'name':'CO','value':345}]
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        mode -- type of flag operation
                options: (str) 'manual', 'clip', 'interactive', 'rowid'
                default: 'manual'

             >>> common data selection parameters for all modes except mode='rowid'
                scans -- list of scan numbers to process
                        default: [] (use all scans)
                        example: [21,22,23,24]
                        this selection is in addition to field, timerange,
                        ifs and pols
                field -- selection string for selecting scans by name
                        default: '' (no name selection)
                        example: 'FLS3a*'
                        this selection is in addition to scans, timerange,
                        ifs, and pols
                timerange -- Select data based on time range:
                    default = '' (all); example,
                    timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                    Note: YYYY/MM/DD can be dropped as needed:
                    timerange='09:14:0~09:54:0' # this time range
                    timerange='09:44:00' # data within one integration of time
                    timerange='>10:24:00' # data after this time
                    timerange='09:44:00+00:13:00' #data 13 minutes after time
                ifs -- list of IF id numbers to select
                        default: [] (use all IFs)
                        example: [15]
                        this selection is in addition to scans, field,
                        timerange, ifs, and pols
                pols -- list of polarization id numbers to select
                        default: [] (use all polarizations)
                        example: [1]
                        this selection is in addition to scans, field,
                        timerange, and ifs

             >>> mode='manual' expandable parameters
                maskflag -- list of mask regions to apply flag/unflag 
                            Note, this parameter is ignored if one or more rows are 
                            given in flagrow, or clip=True.
                        default: [] (entire spectrum)
                        example: [[1000,3000],[5000,7000]]

             >>> mode='clip' expandable parameters
                clipminmax -- range of data that will NOT be flagged
                        default: [] (means no clip operation)
                        example: [0.0,1.5]
                clipoutside -- clip OUTSIDE the range ?
                        options: (bool)True,False
                        default: True
                        example: clipoutside=False means flag data WITHIN the range.
             >>> mode='interactive' expandable parameters
                showflagged -- show flagged data on plots
                        default: False
             >>> mode='rowid' expandable parameters
                rows -- a list of row IDs to apply flag/unflag in the input scannable
                        Note, this parameter is effective only when one or more row 
                        IDs are given explicitly
                        default: [] (means no selection)
                        example: [0, 2, 3]
        unflag -- flag or unflag
                default: False (flag selected data)
                options: (bool) True, False
        outfile -- Name of output file
                default: ''
                Note: by default (outfile=''), actual output file name is set as follows: 
                      (1) if overwrite=True (default), infile (input) will be overwritten.
                      WARNING: If the formats of input and ouput files are different, 
                               this causes complete loss of input file.
                      (2) if overwrite=False, outfile will be <infile>_f. 
        outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                WARNING: Be sure outform is same as the input file format when you 
                         overwrite the input file by overwrite=True and outfile='' (default).
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: True
                WARNING: input file is overwritten if overwrite=True and outfile='' (default). 
                         This causes the complete loss of input file if the formats of
                         input and ouput files are different.
        plotlevel -- control for plotting of results
                options: (int) 0=none, 1=some, 2=more, <0=hardcopy
                default: 0 (no plotting)
                example: plotlevel<0 as abs(plotlevel), e.g.
                         -1 => hardcopy of final plot (will be named
                         <outfile>_flag.eps)
                WARNING: be careful plotting in fsotf mode!

        -------------------------------------------------------------------

        DESCRIPTION:

        Task sdflag2old performs either interactive or non-interactive channel/row 
        based flagging on spectra. 
        Currently, there are three ways of non-interactive flagging available: 
        (1) channel or row based flagging by selecting spectra by field,
        lists of scan numbers, IF numbers, and polarization idices in mode='manual',
        (2) channel based flagging by specifying a range of spectral values in
        mode='clip', and 
        (3) row based flagging by specifying a list of row numbers in 
        mode='rowid'. Note this is an EXPERT mode.

        In mode='manual', the channel based flagging are invoked when regions
        in channel, frequency, or velocity are specified to maskflag parameter. 
        Otherwise, the whole channels are flagged for the selected spectra.
        Note the mode='rowid' is an EXPERT mode since it might not be straight
        forward for general users to select data by row IDs in scantable.

        Interactive flagging is available when mode='interactive'. 
        The available ways of interactive flagging include: 
        (1) row based flagging by selecting 'panel' and (2) channel
        based flagging by selecting 'region's of channels on Flag plotter. 
        See the cookbook for details of how to select channel regions and spectra
        on the plotter.

        NOTE the task sdflag2old only modifies flag information, FLAGROW and FLAGTRA, 
        in the input scantable, and does not filter rows in the dataset unlike
        sdflagold task.

        If plotlevel>=1, the task asks you if you really apply the 
        flags before it is actually written to the data with a plot 
        indicating flagged regions.
        Please note that this task is still experimental.

        WARNING for overwrite option:
        Be sure 'outform' is the same as data format of input file when you
        overwrite it. The default value of the option 'overwrite'
        is True in this task, thereby the current dataset (infile) is 
        overwritten unless a different file name is set to outfile. 
        There is a known issue in overwriting infile. If 'outform' differs to the
        data format of infile, the data is overwritten with the new data format 
        (specified by 'outform') and the data in the original format will be lost.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  
 
  
        """
        if type(scans)==int: scans=[scans]
        if type(ifs)==int: ifs=[ifs]
        if type(maskflag)==int: maskflag=[maskflag]
        if type(rows)==int: rows=[rows]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['mode'] = mode
        mytmp['unflag'] = unflag
        mytmp['scans'] = scans
        mytmp['field'] = field
        mytmp['timerange'] = timerange
        mytmp['ifs'] = ifs
        mytmp['pols'] = pols
        mytmp['maskflag'] = maskflag
        mytmp['clipminmax'] = clipminmax
        mytmp['clipoutside'] = clipoutside
        mytmp['showflagged'] = showflagged
        mytmp['rows'] = rows
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdflag2old.xml')

        casalog.origin('sdflag2old')
        if trec.has_key('sdflag2old') and casac.utils().verify(mytmp, trec['sdflag2old']) :
	    result = task_sdflag2old.sdflag2old(infile, antenna, specunit, restfreq, frame, doppler, mode, unflag, scans, field, timerange, ifs, pols, maskflag, clipminmax, clipoutside, showflagged, rows, outfile, outform, overwrite, plotlevel)

	else :
	  result = False
        return result
