#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdflagold import sdflagold
from task_sdflagold import casalog

class sdflagold_pg_:
    __name__ = "sdflagold"

    def __init__(self) :
       self.__bases__ = (sdflagold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,specunit=None,restfreq=None,frame=None,doppler=None,scanlist=None,field=None,iflist=None,pollist=None,maskflag=None,flagrow=None,clip=None,clipminmax=None,clipoutside=None,flagmode=None,interactive=None,showflagged=None,outfile=None,outform=None,overwrite=None,plotlevel=None,):

        """ASAP SD spectral flagging task 
        Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz',''
                default: '' (=current)
                example: this will be the units for maskflag
            >>> specunit expandable parameters
                 restfreq -- rest frequency
                         available type includes float, int, string, list of float, 
                         list of int, list of string, and list of dictionary. the 
                         default unit of restfreq in case of float, int, or string 
                         without unit is Hz. string input can be a value only 
                         (treated as Hz) or a value followed by unit for which 'GHz',
                         'MHz','kHz',and 'Hz' are available. 
                         a list can be used to set different rest frequencies for 
                         each IF. the length of list input must be nIF. dictionary 
                         input should be a pair of molecule name and frequency with 
                         keys of 'name' and 'value', respectively. values in the 
                         dictionary input follows the same manner as for single 
                         float or string input. 
                         example: 345.796
                                  '1420MHz'
                                  [345.8, 347.0, 356.7]
                                  ['345.8MHz', '347.0MHz', '356.7MHz']
                                  [{'name':'CO','value':345}]
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        scanlist -- list of scan numbers to be considered for flagging and output
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field and iflist
        field -- field name for selecting scans to be considered for flagging and output
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to be considered for flagging and output
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to be considered for flagging and output
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
        maskflag -- list of mask regions to apply flag/unflag 
                    Note, this parameter is ignored if one or more rows are 
                    given in flagrow, or clip=True.
                default: [] (entire spectrum)
                example: [[1000,3000],[5000,7000]]
        flagrow -- list of row numbers to apply flag/unflag
                   Note, this parameter is effective only when one or more row 
                   numbers are given explicitly and also clip=False
                default: [] (no row selection)
                example: [0, 2, 3]
        clip -- flag data that are outside a specified range
                options: (bool)True,False
                default: False
             >>> clip expandable parameters
                clipminmax -- range of data that will NOT be flagged
                        default: [] means do not use clip option
                        example: [0.0,1.5]
                clipoutside -- clip OUTSIDE the range ?
                        options: (bool)True,False
                        default: True
                        example: clipoutside=False means flag data WITHIN the range.
        flagmode -- flag mode
                default: 'flag'
                options: 'flag','unflag'
        interactive -- determines interactive flagging
                options: (bool) True,False
                default: False
             >>> interactive expandable parameters
                showflagged -- show flagged data on plots
                        default: False
        outfile -- Name of output file
                default: ''
                Note: by default (outfile=''), actual output file name is set as follows: 
                      (1) if overwrite=True (default), infile (input) will be overwritten.
                      WARNING: If the formats of input and ouput files are different, 
                               this causes complete loss of input file.
                      (2) if overwrite=False, outfile will be <infile>_f. 
        outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                WARNING: Be sure outform is same as the input file format when you 
                         overwrite the input file by overwrite=True and outfile='' (default).
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: True
                WARNING: input file is overwritten if overwrite=True and outfile='' (default). 
                         This causes the complete loss of input file if the formats of
                         input and ouput files are different.
        plotlevel -- control for plotting of results
                options: (int) 0=none, 1=some, 2=more, <0=hardcopy
                default: 0 (no plotting)
                example: plotlevel<0 as abs(plotlevel), e.g.
                         -1 => hardcopy of final plot (will be named
                         <outfile>_flag.eps)
                WARNING: be careful plotting in fsotf mode!

        -------------------------------------------------------------------

        DESCRIPTION:

        Task sdflagold performs both interactive and non-interactive channel/row 
        based flagging on spectra.
        Currently, the available ways of non-interactive flagging include: 
        (1) channel based flagging by specifying a range of spectral values 
        with clip=True, (2) row based flagging by specifying a list of row 
        numbers to the flagrow parameter, and (3) channel based flagging by 
        specifying regions in channel to the maskflag parameter. 
        These three ways of flagging can not be executed simultaneously. 
        If more than one parameter above are specified, the task looks for 
        them in the above order and operates the first specified way of 
        flagging operation.

        Interactive flagging is available when interactive=True. 
        The available ways of interactive flagging include: 
        (1) row based flagging by selecting 'panel' and (2) channel
        based flagging by selecting 'region's of channels on Flag plotter. 
        Note that the Flag plotter is loaded after carrying out the 
        non-interactive flag operation if any specified. See the cookbook for
        details of how to select channel regions and spectra on the plotter.

        If plotlevel>=1, the task asks you if you really apply the 
        flags before it is actually written to the data with a plot 
        indicating flagged regions.
        Please note that this task is still experimental.

        WARNING for overwrite option:
        Be sure 'outform' is the same as data format of input file when you
        overwrite it. Since CASA 3.1, the default value of the option 'overwrite'
        has been changed to True, thereby the current dataset (infile) is 
        overwritten unless a different file name is set to outfile. 
        There is a known issue in overwriting infile. If 'outform' differs to the
        data format of infile, the data is overwritten with the new data format 
        (specified by 'outform') and the data in the original format will be lost.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  
 
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdflagold'
        myf['taskname'] = 'sdflagold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['specunit'] = specunit = myf['specunit']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['maskflag'] = maskflag = myf['maskflag']
            myparams['flagrow'] = flagrow = myf['flagrow']
            myparams['clip'] = clip = myf['clip']
            myparams['clipminmax'] = clipminmax = myf['clipminmax']
            myparams['clipoutside'] = clipoutside = myf['clipoutside']
            myparams['flagmode'] = flagmode = myf['flagmode']
            myparams['interactive'] = interactive = myf['interactive']
            myparams['showflagged'] = showflagged = myf['showflagged']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(maskflag)==int: maskflag=[maskflag]
        if type(flagrow)==int: flagrow=[flagrow]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['maskflag'] = maskflag
        mytmp['flagrow'] = flagrow
        mytmp['clip'] = clip
        mytmp['clipminmax'] = clipminmax
        mytmp['clipoutside'] = clipoutside
        mytmp['flagmode'] = flagmode
        mytmp['interactive'] = interactive
        mytmp['showflagged'] = showflagged
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdflagold.xml')

        casalog.origin('sdflagold')
        if not trec.has_key('sdflagold') or not casac.utils().verify(mytmp, trec['sdflagold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdflagold           #####')
          casalog.post('')
          result = sdflagold(infile, antenna, specunit, restfreq, frame, doppler, scanlist, field, iflist, pollist, maskflag, flagrow, clip, clipminmax, clipoutside, flagmode, interactive, showflagged, outfile, outform, overwrite, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdflagold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdflagold', 'sdflagold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdflagold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdflagold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['maskflag']  = []
        a['flagrow']  = []
        a['clip']  = False
        a['flagmode']  = 'flag'
        a['interactive']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = True
        a['plotlevel']  = 0

        a['specunit'] = {
                    0:{'value':''}, 
                    1:{'value':'channel'}, 
                    2:{'value':'GHz'}, 
                    3:{'value':'MHz'}, 
                    4:{'value':'kHz'}, 
                    5:{'value':'Hz'}, 
                    6:odict([{'value':'km/s'}, {'restfreq':''}])}
        a['clip'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'clipminmax':[]}, {'clipoutside':True}])}
        a['interactive'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'showflagged':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdflagold', subkey=None):
        desc={'sdflagold': 'ASAP SD spectral flagging task ',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'specunit': 'units for spectral axis (channel,km/s,GHz)',
               'restfreq': 'rest frequency (default unit: Hz)',
               'frame': 'frequency reference frame, e.g. LSRK (''=current)',
               'doppler': 'doppler convention, e.g. RADIO (''=current)',
               'scanlist': 'list of scans to use and output (e.g. [1,2,3,4])',
               'field': 'string for selection by source name for flagging and output',
               'iflist': 'list of IF ids to use and output (e.g. [0,1])',
               'pollist': 'list of polarization by ids or names to use and output (e.g. [0,1])',
               'maskflag': 'list of mask regions to flag/unflag',
               'flagrow': 'list of row numbers to apply row-based flagging/unflagging',
               'clip': 'flag data outside a specified range',
               'clipminmax': 'range of data that will NOT be flagged',
               'clipoutside': 'clip outside the range, or within it',
               'flagmode': 'flag mode (flag,unflag)',
               'interactive': 'determines interactive flagging',
               'showflagged': 'show flagged data on plots',
               'outfile': 'name of output file (See a WARNING in help document)',
               'outform': 'output file format (ASCII,MS,SDFITS,ASAP) (See a WARNING in help document)',
               'overwrite': 'overwrite the output file if already exists (See a WARNING in help document)',
               'plotlevel': 'control for plotting of results',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['specunit']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['maskflag']  = []
        a['flagrow']  = []
        a['clip']  = False
        a['clipminmax']  = []
        a['clipoutside']  = True
        a['flagmode']  = 'flag'
        a['interactive']  = False
        a['showflagged']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = True
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdflagold_pg = sdflagold_pg_()
