#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdgridold import sdgridold
from task_sdgridold import casalog

class sdgridold_pg_:
    __name__ = "sdgridold"

    def __init__(self) :
       self.__bases__ = (sdgridold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infiles=None,antenna=None,scanlist=None,ifno=None,pollist=None,gridfunction=None,convsupport=None,truncate=None,gwidth=None,jwidth=None,weight=None,clipminmax=None,outfile=None,overwrite=None,npix=None,cell=None,center=None,plot=None,):

        """SD task: gridding single dish data
  Keyword arguments:
        infiles -- name of input SD dataset. can be list.
                example: 'testimage.asap' 
                         ['testimage1.asap','testimage2.asap']
        antenna -- select data based on antenna name(s) or id(s)
                default: -1
                example: 0, 'DV01'
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
        ifno -- IFNO to be gridded
                default: -1 (only process IFNO in the first row)
                example: 1
        pollist -- POLNO to be gridded
                default: [] (all polarizations)
                example: 1,[0,1]
        gridfunction -- gridding function 
                options: 'BOX' (Box-car), 'SF' (Spheroidal), 
                         'GAUSS' (Gaussian), 'PB' (Primary-beam)
                         'GJINC' (Gaussian*Jinc)
                default: 'BOX'
                example: 'SF'
            >>> gridfunction expandable parameter:
               convsupport -- convolution support for 'SF' 
                   default: -1 (use default for each gridfunction)
                   example: 3
               truncate -- truncattion radius of convolution kernel.
                           effective only for 'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               gwidth -- HWHM for gaussian. Effective only for 
                         'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               jwidth -- Width of jinc function. Effective only for 
                         'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
        weight -- weight type (both lower-case and upper-case are 
                  acceptable)
                options: 'UNIFORM',
                         'TSYS'  (1/Tsys**2 weighted)
                         'TINT'  (integration time weighted)
                         'TINTSYS'  (Tint/Tsys**2)
                default: 'UNIFORM'
        clipminmax -- do min/max cliping if True
                default: False
        outfile -- output data name
                default: '' (outfile will be set to infile[0]+'.grid')
                example: 'mydata.asap.grid'
        overwrite -- overwrite option for outfile
                default: False (not overwrite)
                options: True, False
                example: if True, existing file will be overwritten
        npix -- x and y image size in pixels, symmetric for single value
                default: -1 (automatically calculated from cell size and 
                             the data)
                example: npix=200 (equivalent to [200,200])
        cell -- x and y cell size. default unit arcsec
                default: '' (automatically calculated from npix if it is 
                             set, otherwise '1.0arcmin')
                example: cell=['0.2arcmin, 0.2arcmin']
                         cell='0.2arcmin' (equivalent to example above)
                         cell=12.0 (interpreted as '12.0arcsec'='0.2arcmin')
        center -- grid center
                default: '' (automatically calculated from the data)
                example: 'J2000 13h44m00 -17d02m00'
                         ['05:34:48.2', '-05.22.17.7'] (in J2000 frame)
                         [1.46, -0.09] (interpreted as radian in J2000 frame)
        plot -- Plot result or not
                default: False (not plot)
                example: if True, result will be plotted

        DESCRIPTION:

        The sdgridold task performs spatial gridding according to the user 
        specification of spatial grid, convolution function, etc.

        For grid configuration, the task supplements necessary information 
        by referring input data if any of gridding parameter ('npix', 
        'cell', or 'center') is not specified by the user. If 'center' is 
        default value (empty string), central position of the grid will be 
        set to the center of observed area, i.e. x=0.5*(xmax+xmin), 
        y=0.5*(ymax+ymin). If either 'cell' or 'npix' is set, unspecified 
        one will be calculated from the others. In that case, total extent of 
        the grid will be set to cover all observed position. If neither 'cell' 
        nor 'npix' is set, cell size will be set to 1.0 arcmin and number of 
        pixel will be calculated based on that cell size.
 
        Currently, only J2000 frame is supported.
 
        The parameter gridfunction sets gridding function for imaging. 
        Currently, the task supports 'BOX' (Box-car), 'SF' (Prolate 
        Spheroidal Wave Function), 'GAUSS' (Gaussian), 'GJINC' (Gaussian*
        Jinc), where Jinc(x) = J_1(pi*x/c)/(pi*x/c) with a first order 
        Bessel function J_1, and 'PB' (Primary Beam, not implemented yet). 
        For 'PB', correct antenna informations should be included in input 
        file. 
        
        There are four subparameters for gridfunction: convsupport, truncate, 
        gwidth, and jwidth. The convsupport is an integer specifying cut-off 
        radius for 'SF' in units of pixel. By default (convsupport=-1), 
        the cut-off radius is set to 3 pixels. The truncate is a cut-off 
        radius for 'GAUSS' or 'GJINC'. It accepts integer, float, and 
        string values of numeric plus unit. Allowed units are angular 
        units such as 'deg', 'arcmin', 'arcsec', and 'pixel'. Default unit 
        is 'pixel' so that string without unit or numerical values (integer 
        or float) will be interpreted as radius in pixel. Default value 
        for truncate, which is used when negative radius is set, is 3*HWHM 
        for 'GAUSS' and radius at first null for 'GJINC'. The gwidth is 
        the HWHM of gaussian for 'GAUSS' and 'GJINC'. Default value is 
        sqrt(log(2)) pixel for 'GAUSS' and 2.52*sqrt(log(2)) pixel for 
        'GJINC'. The jwidth specifies width of the jinc function (parameter 
        'c' in the definition above). Default is 1.55 pixel. Both gwidth 
        jwidth allows integer, float, or string of numeric plus unit. 
        Default values for gwidth and jwidth are taken from Mangum et al. 
        (2007). Formula for 'GAUSS' and 'GJINC' are taken from Table 1 in 
        the paper, and are written as below using gwidth and jwidth: 

           GAUSS: exp[-(|r|/gwidth)**2]

           GJINC: J_1(pi*|r|/jwidth)/(pi*|r|/jwidth) * exp[-(|r|/gwidth)^2]  


        Boolean parameter 'plot' controls whether gridded result is plotted 
        or not. If True, color map of gridded data will be shown. Pixel 
        center and observed position are overlayed as blue dot and red dot, 
        respectively. Currently, channel averaged value will be plotted.

        Reference: Mangum, et al. 2007, A&A, 474, 679-687 

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdgridold'
        myf['taskname'] = 'sdgridold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infiles'] = infiles = myf['infiles']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['ifno'] = ifno = myf['ifno']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['gridfunction'] = gridfunction = myf['gridfunction']
            myparams['convsupport'] = convsupport = myf['convsupport']
            myparams['truncate'] = truncate = myf['truncate']
            myparams['gwidth'] = gwidth = myf['gwidth']
            myparams['jwidth'] = jwidth = myf['jwidth']
            myparams['weight'] = weight = myf['weight']
            myparams['clipminmax'] = clipminmax = myf['clipminmax']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['npix'] = npix = myf['npix']
            myparams['cell'] = cell = myf['cell']
            myparams['center'] = center = myf['center']
            myparams['plot'] = plot = myf['plot']

        if type(scanlist)==int: scanlist=[scanlist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infiles'] = infiles
        mytmp['antenna'] = antenna
        mytmp['scanlist'] = scanlist
        mytmp['ifno'] = ifno
        mytmp['pollist'] = pollist
        mytmp['gridfunction'] = gridfunction
        mytmp['convsupport'] = convsupport
        mytmp['truncate'] = truncate
        mytmp['gwidth'] = gwidth
        mytmp['jwidth'] = jwidth
        mytmp['weight'] = weight
        mytmp['clipminmax'] = clipminmax
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['npix'] = npix
        mytmp['cell'] = cell
        mytmp['center'] = center
        mytmp['plot'] = plot
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdgridold.xml')

        casalog.origin('sdgridold')
        if not trec.has_key('sdgridold') or not casac.utils().verify(mytmp, trec['sdgridold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdgridold           #####')
          casalog.post('')
          result = sdgridold(infiles, antenna, scanlist, ifno, pollist, gridfunction, convsupport, truncate, gwidth, jwidth, weight, clipminmax, outfile, overwrite, npix, cell, center, plot)
          casalog.post('')
          casalog.post('##### End Task: sdgridold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdgridold', 'sdgridold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdgridold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdgridold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infiles']  = [""]
        a['antenna']  = -1
        a['scanlist']  = []
        a['ifno']  = -1
        a['pollist']  = []
        a['gridfunction']  = 'BOX'
        a['weight']  = 'UNIFORM'
        a['clipminmax']  = False
        a['outfile']  = ''
        a['overwrite']  = False
        a['npix']  = -1
        a['cell']  = ''
        a['center']  = ''
        a['plot']  = False

        a['gridfunction'] = {
                    0:{'value':'BOX'}, 
                    1:{'value':'box'}, 
                    2:odict([{'value':'SF'}, {'convsupport':-1}]), 
                    3:odict([{'value':'sf'}, {'convsupport':-1}]), 
                    4:odict([{'value':'GAUSS'}, {'truncate':-1}, {'gwidth':-1}]), 
                    5:odict([{'value':'gauss'}, {'truncate':-1}, {'gwidth':-1}]), 
                    6:odict([{'value':'GJINC'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}]), 
                    7:odict([{'value':'gjinc'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}]), 
                    8:{'value':'PB'}, 
                    9:{'value':'pb'}}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdgridold', subkey=None):
        desc={'sdgridold': 'SD task: gridding single dish data',
               'infiles': '(list of) input SD dataset',
               'antenna': 'antenna name(s) or id(s)',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'ifno': 'IFNO for gridding',
               'pollist': 'POLNO for gridding',
               'gridfunction': 'gridding function for imaging ("BOX","SF","PB","GAUSS","GJINC")',
               'convsupport': 'truncate of convolution kernel',
               'truncate': 'truncation radius of convolution kernel',
               'gwidth': 'HWHM for gaussian',
               'jwidth': 'c-parameter for jinc function',
               'weight': 'weight type',
               'clipminmax': 'clip minimum and maximum values during gridding',
               'outfile': 'output image name',
               'overwrite': 'overwrite option',
               'npix': 'number of pixels in x and y, symmetric for single value',
               'cell': 'x and y cell size. default unit arcsec',
               'center': 'Image center',
               'plot': 'Plot result or not',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infiles']  = [""]
        a['antenna']  = -1
        a['scanlist']  = []
        a['ifno']  = -1
        a['pollist']  = []
        a['gridfunction']  = 'BOX'
        a['convsupport']  = -1
        a['truncate']  = -1
        a['gwidth']  = -1
        a['jwidth']  = -1
        a['weight']  = 'UNIFORM'
        a['clipminmax']  = False
        a['outfile']  = ''
        a['overwrite']  = False
        a['npix']  = -1
        a['cell']  = ''
        a['center']  = ''
        a['plot']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdgridold_pg = sdgridold_pg_()
