#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
#from taskmanager import tm
import task_sdimagingold
def sdimagingold(infiles=[''], specunit='', restfreq='', scanlist=[], field=-1, spw=[0], antenna=-1, stokes='', gridfunction='BOX', convsupport=-1, truncate=-1, gwidth=-1, jwidth=-1, minweight=0.1, outfile='', overwrite=False, imsize=[

            ], cell='', dochannelmap=False, nchan=1, start=0, step=1, outframe='', phasecenter='', ephemsrcname='', pointingcolumn='direction'):

        """SD task: imaging for total power and spectral data
  Keyword arguments:
        infiles -- a list of names of input SD (MS) datasets
                example: 'm100.PM01.ms'
                         ['m100.PM01.ms','m100.PM03.ms']; multiple MSes
        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz'
                default: 'channel'
                example: this will be the units for nchan, start, and step
        restfreq -- rest frequency
                default: '' (refer input data)
                example: 1.0e11, '100GHz'
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field and spw
        field -- field id or selection string for selecting scans by name
                default: -1 (all fields)
                example: 'FLS3a', 0
                this selection is in addition to scanlist and spw
        spw -- a list of spectral window id
                default: 0
                example: 1
                         [1, 3, 5]
                this selection is in addition to scanlist and field
        antenna -- select data based on antenna name(s) or id(s)
                default: -1 (all baselines, i.e. all antenna in case of auto data)
                example: 0, 'DV01'
        stokes -- select data based on stokes or polarization type 
                default: '' (use all polarizations)
                example: 'XX'
        gridfunction -- gridding function for imaging
                options: 'BOX' (Box-car), 'SF' (Spheroidal), 
                         'PB' (Primary-beam), 'GAUSS' (Gaussian),
                         'GJINC' (Gaussian*Jinc)
                default: 'BOX'
                example: 'SF'
            >>> gridfunction expandable parameter:
               convsupport -- convolution support for 'SF' 
                   default: -1 (use default for each gridfunction)
                   example: 3
               truncate -- truncattion radius of convolution kernel.
                           effective only for 'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               gwidth -- HWHM for gaussian. Effective only for 
                         'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               jwidth -- Width of jinc function. Effective only for 
                         'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
        minweight -- Minimum weight ratio to the median of weight used in 
                     weight correction and weight based masking
                default: 0.1
                example: minweight = 0.
        outfile -- output image name
                default: none
                example: 'mySDimage.im'
        overwrite -- overwrite option for outfile
                default: False (not overwrite)
                options: True, False
                example: if True, existing file will be overwritten
        imsize -- x and y image size in pixels, symmetric for single value
                default: [] (=cover all pointings in MS)
                example: imsize=200 (equivalent to [200,200])
        cell -- x and y cell size. default unit arcmin
                default: '' (= 1/3 of FWHM of primary beam)
                example: cell=['0.2arcmin, 0.2arcmin']
                         cell='0.2arcmin' (equivalent to example above)
        dochannelmap -- channel map image or total power image
                default: False (total power)
                options: True (channel map), False
            >>> dochannelmap=True expandable parameters
               nchan -- number of spectral channel for created image
                   default: 1 
                   example: 100
               start -- reference value of start channel (in units of specunit)
                   default: 0 (0th channel if specunit='channel')
                   example: 100
               step -- width of each spectral channel for created image
                   default: 1 (width of 1 channel if specunit='channel')
                   example: 100
        outframe -- Velocity reference frame of output image
                Options: '','LSRK','LSRD','BARY','GEO','TOPO','GALACTO',
                         'LGROUP','CMB'
                default: ''; same as input data or 'LSRK' for multiple-MS inputs
                example: frame='bary' for Barycentric frame 
        phasecenter -- image phase center: direction measure or field id 
                default: '' (= the center of pointing directions in 
                             POINTING table of infiles)
                example: 6 (field id), 'J2000 13h44m00 -17d02m00',
                         'AZEL -123d48m29 15d41m41'
        ephemsrcname -- ephemeris source name for moving source
                default: ''
                if the source name in the data matches one of the known 
                solar objects by the system, this task automatically set 
                the source name. 
                example: 'moon' 
        pointingcolumn -- pointing data column to use
                option: 'direction', 'target', 'pointing_offset', 'source_offset', 'encoder' 
                default: 'direction'


        DESCRIPTION:

        Task sdimagingold create image from input single-dish data.
        The input can be either total power and spectral data. Currently,
        this task directly accesses the Measurement Set data only because of 
        the data access efficiency. So it differs from other single-dish 
        tasks that mostly operate on the ASAP scantable data format.
 
        Units of spectral axis can be specified via a parameter specunit.
        Allowed values for specunit are 'channel', 'GHz', 'MHz', 'kHz', 'Hz', 
        and 'km/s'. This parameter is also used as the units of the parameter 
        start and step that specify reference value of start channel and 
        width of each spectral channel for channel map, respectively. 
        The parameter nchan specifies number of channels for created image. 
        If you set nchan as -1, the task selects existing all channels and 
        combine those data into one channel to create continuum image.

        You can specify field id or name directly. By default, field is set 
        to -1 that means the task selects all fields in the data.

        Selection of the antennas can be made by setting antennaid(s) or 
        antenna name(s) in string (e.g. '0', 'DV01',etc.) or integer 
        (e.g. 0). Defalut value, -1, means that the task selects data from 
        all baseline, i.e., data from all antenna when data only contains 
        auto-correlation.

        The parameter gridfunction sets gridding function for imaging. 
        Currently, the task supports 'BOX' (Box-car), 'SF' (Prolate 
        Spheroidal Wave Function), 'GAUSS' (Gaussian), 'GJINC' (Gaussian*
        Jinc), where Jinc(x) = J_1(pi*x/c)/(pi*x/c) with a first order 
        Bessel function J_1, and 'PB' (Primary Beam). For 'PB', correct 
        antenna informations should be included in input file. 

        There are four subparameters for gridfunction: convsupport, truncate, 
        gwidth, and jwidth. The convsupport is an integer specifying cut-off 
        radius for 'SF' in units of pixel. By default (convsupport=-1), 
        the cut-off radius is set to 3 pixels. The truncate is a cut-off 
        radius for 'GAUSS' or 'GJINC'. It accepts integer, float, and 
        string values of numeric plus unit. Allowed units are angular 
        units such as 'deg', 'arcmin', 'arcsec', and 'pixel'. Default unit 
        is 'pixel' so that string without unit or numerical values (integer 
        or float) will be interpreted as radius in pixel. Default value 
        for truncate, which is used when negative radius is set, is 3*HWHM 
        for 'GAUSS' and radius at first null for 'GJINC'. The gwidth is 
        the HWHM of gaussian for 'GAUSS' and 'GJINC'. Default value is 
        sqrt(log(2)) pixel for 'GAUSS' and 2.52*sqrt(log(2)) pixel for 
        'GJINC'. The jwidth specifies width of the jinc function (parameter 
        'c' in the definition above). Default is 1.55 pixel. Both gwidth 
        jwidth allows integer, float, or string of numeric plus unit. 
        Default values for gwidth and jwidth are taken from Mangum et al. 
        (2007). Formula for 'GAUSS' and 'GJINC' are taken from Table 1 in 
        the paper, and are written as below using gwidth and jwidth: 

           GAUSS: exp[-log(2)*(|r|/gwidth)**2]

           GJINC: J_1(pi*|r|/jwidth)/(pi*|r|/jwidth)
                     * exp[-log(2)*(|r|/gwidth)^2]  


        Reference: Mangum, et al. 2007, A&A, 474, 679-687 

        The parameter minweight defines a threshold of weight values 
        to mask. The pixels in outfile whose weight is smaller than
        minweight*median(weight) are masked out. The task also creates
        a weight image with the name outfile.weight.

  
        """
        if type(infiles)==str: infiles=[infiles]
        if type(scanlist)==int: scanlist=[scanlist]
        if type(spw)==int: spw=[spw]
        if type(imsize)==int: imsize=[imsize]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infiles'] = infiles
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['antenna'] = antenna
        mytmp['stokes'] = stokes
        mytmp['gridfunction'] = gridfunction
        mytmp['convsupport'] = convsupport
        mytmp['truncate'] = truncate
        mytmp['gwidth'] = gwidth
        mytmp['jwidth'] = jwidth
        mytmp['minweight'] = minweight
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['imsize'] = imsize
        mytmp['cell'] = cell
        mytmp['dochannelmap'] = dochannelmap
        mytmp['nchan'] = nchan
        mytmp['start'] = start
        mytmp['step'] = step
        mytmp['outframe'] = outframe
        mytmp['phasecenter'] = phasecenter
        mytmp['ephemsrcname'] = ephemsrcname
        mytmp['pointingcolumn'] = pointingcolumn
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdimagingold.xml')

        casalog.origin('sdimagingold')
        if trec.has_key('sdimagingold') and casac.utils().verify(mytmp, trec['sdimagingold']) :
	    result = task_sdimagingold.sdimagingold(infiles, specunit, restfreq, scanlist, field, spw, antenna, stokes, gridfunction, convsupport, truncate, gwidth, jwidth, minweight, outfile, overwrite, imsize, cell, dochannelmap, nchan, start, step, outframe, phasecenter, ephemsrcname, pointingcolumn)

	else :
	  result = False
        return result
