#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdimagingold import sdimagingold
class sdimagingold_cli_:
    __name__ = "sdimagingold"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdimagingold_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infiles':None, 'specunit':None, 'restfreq':None, 'scanlist':None, 'field':None, 'spw':None, 'antenna':None, 'stokes':None, 'gridfunction':None, 'convsupport':None, 'truncate':None, 'gwidth':None, 'jwidth':None, 'minweight':None, 'outfile':None, 'overwrite':None, 'imsize':None, 'cell':None, 'dochannelmap':None, 'nchan':None, 'start':None, 'step':None, 'outframe':None, 'phasecenter':None, 'ephemsrcname':None, 'pointingcolumn':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infiles=None, specunit=None, restfreq=None, scanlist=None, field=None, spw=None, antenna=None, stokes=None, gridfunction=None, convsupport=None, truncate=None, gwidth=None, jwidth=None, minweight=None, outfile=None, overwrite=None, imsize=None, cell=None, dochannelmap=None, nchan=None, start=None, step=None, outframe=None, phasecenter=None, ephemsrcname=None, pointingcolumn=None, ):

        """SD task: imaging for total power and spectral data

	Detailed Description: 
SD task for imaging both total power and spectral data

        ### INTERFACE CHANGES in CASA 4.2 ###
        * infile -&gt; infiles (renamed)
        * outframe (added)
        * imsize (default value changed)
        * cell (default value changed)
        * spw (takes a list of integer)
        * minweight (added)
        #####################################

	Arguments :
		infiles:	a list of names of input SD dataset (only MS is allowed for this task)
		   Default Value: 

		specunit:	units for spectral axis
		   Default Value: 
		   Allowed Values:
				
				channel
				km/s
				GHz
				MHz
				kHz
				Hz

		restfreq:	rest frequency (e.g., \'114.5GHz\')
		   Default Value: 

		scanlist:	list of scans to use (e.g. [1,2,3,4])
		   Default Value: 

		field:	string for selection by field name or id
		   Default Value: -1

		spw:	a list of spectral window id for imaging (e.g., 15, [1, 3, 5])
		   Default Value: 0

		antenna:	antenna name(s) or id(s) (e.g., 0, \'PM03\') (-1 = use all antennas)
		   Default Value: -1

		stokes:	stokes or correlation name (e.g. \'I\', \'XX\')
		   Default Value: 

		gridfunction:	gridding function for imaging
		   Default Value: BOX
		   Allowed Values:
				BOX
				PB
				SF
				GAUSS
				GJINC
				box
				pb
				sf
				gauss
				gjinc

		convsupport:	convolution support for gridding
		   Default Value: -1

		truncate:	truncation radius for gridding
		   Default Value: -1

		gwidth:	HWHM for gaussian
		   Default Value: -1

		jwidth:	c-parameter for jinc function
		   Default Value: -1

		minweight:	Minimum weight ratio to the median of weight used in weight correction and weight beased masking
		   Default Value: 0.1
		   Allowed Values:
				0

		outfile:	output image name
		   Default Value: 

		overwrite:	overwrite option
		   Default Value: False

		imsize:	x and y image size in pixels, symmetric for single value (e.g., [128,128])
		   Default Value: 

            

		cell:	x and y cell size (e.g., [\'30arcsec\',\'30arcsec\']). default unit arcmin
		   Default Value: 

		dochannelmap:	True for channel map, False for total power imaging
		   Default Value: False

		nchan:	number of spectral channel for created image
		   Default Value: 1

		start:	reference value of start channel (in units of specunit)
		   Default Value: 0

		step:	width of each spectral channel (in units of specunit)
		   Default Value: 1

		outframe:	velocity frame of output image (''=current frame or LSRK for multiple-MS inputs)
		   Default Value: 
		   Allowed Values:
				lsrk
				lsrd
				bary
				geo
				topo
				galacto
				lgroup
				cmb
				

		phasecenter:	Image phase center: position or field index (e.g., \'J2000 17:30:15.0 -25.30.00.0\')
		   Default Value: 

		ephemsrcname:	ephemeris source name
		   Default Value: 

		pointingcolumn:	pointing data column to use
		   Default Value: direction
		   Allowed Values:
				target
				pointing_offset
				source_offset
				encoder
				direction

	Returns: void

	Example :

  Keyword arguments:
        infiles -- a list of names of input SD (MS) datasets
                example: 'm100.PM01.ms'
                         ['m100.PM01.ms','m100.PM03.ms']; multiple MSes
        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz'
                default: 'channel'
                example: this will be the units for nchan, start, and step
        restfreq -- rest frequency
                default: '' (refer input data)
                example: 1.0e11, '100GHz'
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field and spw
        field -- field id or selection string for selecting scans by name
                default: -1 (all fields)
                example: 'FLS3a', 0
                this selection is in addition to scanlist and spw
        spw -- a list of spectral window id
                default: 0
                example: 1
                         [1, 3, 5]
                this selection is in addition to scanlist and field
        antenna -- select data based on antenna name(s) or id(s)
                default: -1 (all baselines, i.e. all antenna in case of auto data)
                example: 0, 'DV01'
        stokes -- select data based on stokes or polarization type 
                default: '' (use all polarizations)
                example: 'XX'
        gridfunction -- gridding function for imaging
                options: 'BOX' (Box-car), 'SF' (Spheroidal), 
                         'PB' (Primary-beam), 'GAUSS' (Gaussian),
                         'GJINC' (Gaussian*Jinc)
                default: 'BOX'
                example: 'SF'
            >>> gridfunction expandable parameter:
               convsupport -- convolution support for 'SF' 
                   default: -1 (use default for each gridfunction)
                   example: 3
               truncate -- truncattion radius of convolution kernel.
                           effective only for 'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               gwidth -- HWHM for gaussian. Effective only for 
                         'GAUSS' and 'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
               jwidth -- Width of jinc function. Effective only for 
                         'GJINC'.
                   default: '-1' (use default for each gridfunction)
                   example: 3, '20arcsec', '3pixel'
        minweight -- Minimum weight ratio to the median of weight used in 
                     weight correction and weight based masking
                default: 0.1
                example: minweight = 0.
        outfile -- output image name
                default: none
                example: 'mySDimage.im'
        overwrite -- overwrite option for outfile
                default: False (not overwrite)
                options: True, False
                example: if True, existing file will be overwritten
        imsize -- x and y image size in pixels, symmetric for single value
                default: [] (=cover all pointings in MS)
                example: imsize=200 (equivalent to [200,200])
        cell -- x and y cell size. default unit arcmin
                default: '' (= 1/3 of FWHM of primary beam)
                example: cell=['0.2arcmin, 0.2arcmin']
                         cell='0.2arcmin' (equivalent to example above)
        dochannelmap -- channel map image or total power image
                default: False (total power)
                options: True (channel map), False
            >>> dochannelmap=True expandable parameters
               nchan -- number of spectral channel for created image
                   default: 1 
                   example: 100
               start -- reference value of start channel (in units of specunit)
                   default: 0 (0th channel if specunit='channel')
                   example: 100
               step -- width of each spectral channel for created image
                   default: 1 (width of 1 channel if specunit='channel')
                   example: 100
        outframe -- Velocity reference frame of output image
                Options: '','LSRK','LSRD','BARY','GEO','TOPO','GALACTO',
                         'LGROUP','CMB'
                default: ''; same as input data or 'LSRK' for multiple-MS inputs
                example: frame='bary' for Barycentric frame 
        phasecenter -- image phase center: direction measure or field id 
                default: '' (= the center of pointing directions in 
                             POINTING table of infiles)
                example: 6 (field id), 'J2000 13h44m00 -17d02m00',
                         'AZEL -123d48m29 15d41m41'
        ephemsrcname -- ephemeris source name for moving source
                default: ''
                if the source name in the data matches one of the known 
                solar objects by the system, this task automatically set 
                the source name. 
                example: 'moon' 
        pointingcolumn -- pointing data column to use
                option: 'direction', 'target', 'pointing_offset', 'source_offset', 'encoder' 
                default: 'direction'


        DESCRIPTION:

        Task sdimagingold create image from input single-dish data.
        The input can be either total power and spectral data. Currently,
        this task directly accesses the Measurement Set data only because of 
        the data access efficiency. So it differs from other single-dish 
        tasks that mostly operate on the ASAP scantable data format.
 
        Units of spectral axis can be specified via a parameter specunit.
        Allowed values for specunit are 'channel', 'GHz', 'MHz', 'kHz', 'Hz', 
        and 'km/s'. This parameter is also used as the units of the parameter 
        start and step that specify reference value of start channel and 
        width of each spectral channel for channel map, respectively. 
        The parameter nchan specifies number of channels for created image. 
        If you set nchan as -1, the task selects existing all channels and 
        combine those data into one channel to create continuum image.

        You can specify field id or name directly. By default, field is set 
        to -1 that means the task selects all fields in the data.

        Selection of the antennas can be made by setting antennaid(s) or 
        antenna name(s) in string (e.g. '0', 'DV01',etc.) or integer 
        (e.g. 0). Defalut value, -1, means that the task selects data from 
        all baseline, i.e., data from all antenna when data only contains 
        auto-correlation.

        The parameter gridfunction sets gridding function for imaging. 
        Currently, the task supports 'BOX' (Box-car), 'SF' (Prolate 
        Spheroidal Wave Function), 'GAUSS' (Gaussian), 'GJINC' (Gaussian*
        Jinc), where Jinc(x) = J_1(pi*x/c)/(pi*x/c) with a first order 
        Bessel function J_1, and 'PB' (Primary Beam). For 'PB', correct 
        antenna informations should be included in input file. 

        There are four subparameters for gridfunction: convsupport, truncate, 
        gwidth, and jwidth. The convsupport is an integer specifying cut-off 
        radius for 'SF' in units of pixel. By default (convsupport=-1), 
        the cut-off radius is set to 3 pixels. The truncate is a cut-off 
        radius for 'GAUSS' or 'GJINC'. It accepts integer, float, and 
        string values of numeric plus unit. Allowed units are angular 
        units such as 'deg', 'arcmin', 'arcsec', and 'pixel'. Default unit 
        is 'pixel' so that string without unit or numerical values (integer 
        or float) will be interpreted as radius in pixel. Default value 
        for truncate, which is used when negative radius is set, is 3*HWHM 
        for 'GAUSS' and radius at first null for 'GJINC'. The gwidth is 
        the HWHM of gaussian for 'GAUSS' and 'GJINC'. Default value is 
        sqrt(log(2)) pixel for 'GAUSS' and 2.52*sqrt(log(2)) pixel for 
        'GJINC'. The jwidth specifies width of the jinc function (parameter 
        'c' in the definition above). Default is 1.55 pixel. Both gwidth 
        jwidth allows integer, float, or string of numeric plus unit. 
        Default values for gwidth and jwidth are taken from Mangum et al. 
        (2007). Formula for 'GAUSS' and 'GJINC' are taken from Table 1 in 
        the paper, and are written as below using gwidth and jwidth: 

           GAUSS: exp[-log(2)*(|r|/gwidth)**2]

           GJINC: J_1(pi*|r|/jwidth)/(pi*|r|/jwidth)
                     * exp[-log(2)*(|r|/gwidth)^2]  


        Reference: Mangum, et al. 2007, A&A, 474, 679-687 

        The parameter minweight defines a threshold of weight values 
        to mask. The pixels in outfile whose weight is smaller than
        minweight*median(weight) are masked out. The task also creates
        a weight image with the name outfile.weight.

  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdimagingold'
        self.__globals__['taskname'] = 'sdimagingold'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infiles'] = infiles = self.parameters['infiles']
            myparams['specunit'] = specunit = self.parameters['specunit']
            myparams['restfreq'] = restfreq = self.parameters['restfreq']
            myparams['scanlist'] = scanlist = self.parameters['scanlist']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['stokes'] = stokes = self.parameters['stokes']
            myparams['gridfunction'] = gridfunction = self.parameters['gridfunction']
            myparams['convsupport'] = convsupport = self.parameters['convsupport']
            myparams['truncate'] = truncate = self.parameters['truncate']
            myparams['gwidth'] = gwidth = self.parameters['gwidth']
            myparams['jwidth'] = jwidth = self.parameters['jwidth']
            myparams['minweight'] = minweight = self.parameters['minweight']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']
            myparams['imsize'] = imsize = self.parameters['imsize']
            myparams['cell'] = cell = self.parameters['cell']
            myparams['dochannelmap'] = dochannelmap = self.parameters['dochannelmap']
            myparams['nchan'] = nchan = self.parameters['nchan']
            myparams['start'] = start = self.parameters['start']
            myparams['step'] = step = self.parameters['step']
            myparams['outframe'] = outframe = self.parameters['outframe']
            myparams['phasecenter'] = phasecenter = self.parameters['phasecenter']
            myparams['ephemsrcname'] = ephemsrcname = self.parameters['ephemsrcname']
            myparams['pointingcolumn'] = pointingcolumn = self.parameters['pointingcolumn']

        if type(infiles)==str: infiles=[infiles]
        if type(scanlist)==int: scanlist=[scanlist]
        if type(spw)==int: spw=[spw]
        if type(imsize)==int: imsize=[imsize]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infiles'] = infiles
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['antenna'] = antenna
        mytmp['stokes'] = stokes
        mytmp['gridfunction'] = gridfunction
        mytmp['convsupport'] = convsupport
        mytmp['truncate'] = truncate
        mytmp['gwidth'] = gwidth
        mytmp['jwidth'] = jwidth
        mytmp['minweight'] = minweight
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
        mytmp['imsize'] = imsize
        mytmp['cell'] = cell
        mytmp['dochannelmap'] = dochannelmap
        mytmp['nchan'] = nchan
        mytmp['start'] = start
        mytmp['step'] = step
        mytmp['outframe'] = outframe
        mytmp['phasecenter'] = phasecenter
        mytmp['ephemsrcname'] = ephemsrcname
        mytmp['pointingcolumn'] = pointingcolumn
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdimagingold.xml')

        casalog.origin('sdimagingold')
	try :
          #if not trec.has_key('sdimagingold') or not casac.casac.utils().verify(mytmp, trec['sdimagingold']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdimagingold'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdimagingold', 'sdimagingold.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdimagingold'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdimagingold(infiles, specunit, restfreq, scanlist, field, spw, antenna, stokes, gridfunction, convsupport, truncate, gwidth, jwidth, minweight, outfile, overwrite, imsize, cell, dochannelmap, nchan, start, step, outframe, phasecenter, ephemsrcname, pointingcolumn)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdimagingold'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdimagingold', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infiles']  = ['']
        a['specunit']  = ''
        a['restfreq']  = ''
        a['scanlist']  = []
        a['field']  = -1
        a['spw']  = [0]
        a['antenna']  = -1
        a['stokes']  = ''
        a['gridfunction']  = 'BOX'
        a['minweight']  = 0.1
        a['outfile']  = ''
        a['overwrite']  = False
        a['imsize']  = [

            ]
        a['cell']  = ''
        a['dochannelmap']  = False
        a['outframe']  = ''
        a['phasecenter']  = ''
        a['ephemsrcname']  = ''
        a['pointingcolumn']  = 'direction'

        a['dochannelmap'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'nchan':1}, {'start':0}, {'step':1}])}
        a['gridfunction'] = {
                    0:{'value':'BOX'}, 
                    1:{'value':'box'}, 
                    2:odict([{'value':'SF'}, {'convsupport':-1}]), 
                    3:odict([{'value':'sf'}, {'convsupport':-1}]), 
                    4:{'value':'PB'}, 
                    5:{'value':'pb'}, 
                    6:odict([{'value':'GAUSS'}, {'truncate':-1}, {'gwidth':-1}]), 
                    7:odict([{'value':'gauss'}, {'truncate':-1}, {'gwidth':-1}]), 
                    8:odict([{'value':'GJINC'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}]), 
                    9:odict([{'value':'gjinc'}, {'truncate':-1}, {'gwidth':-1}, {'jwidth':-1}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdimagingold', subkey=None):
        desc={'sdimagingold': 'SD task: imaging for total power and spectral data',
               'infiles': 'a list of names of input SD dataset (only MS is allowed for this task)',
               'specunit': 'units for spectral axis [\'channel\'|\'km/s\'|\'GHz\']',
               'restfreq': 'rest frequency (e.g., \'114.5GHz\')',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by field name or id',
               'spw': 'a list of spectral window id for imaging (e.g., 15, [1, 3, 5])',
               'antenna': 'antenna name(s) or id(s) (e.g., 0, \'PM03\') (-1 = use all antennas)',
               'stokes': 'stokes or correlation name (e.g. \'I\', \'XX\')',
               'gridfunction': 'gridding function for imaging [\'BOX\'|\'SF\'|\'PB\'|\'GAUSS\'|\'GJINC\']',
               'convsupport': 'convolution support for gridding',
               'truncate': 'truncation radius for gridding',
               'gwidth': 'HWHM for gaussian',
               'jwidth': 'c-parameter for jinc function',
               'minweight': 'Minimum weight ratio to use',
               'outfile': 'output image name',
               'overwrite': 'overwrite option',
               'imsize': 'x and y image size in pixels, symmetric for single value (e.g., [128,128])',
               'cell': 'x and y cell size (e.g., [\'30arcsec\',\'30arcsec\']). default unit arcmin',
               'dochannelmap': 'True for channel map, False for total power imaging',
               'nchan': 'number of spectral channel for created image',
               'start': 'reference value of start channel (in units of specunit)',
               'step': 'width of each spectral channel (in units of specunit)',
               'outframe': 'velocity frame of output image [\'lsrk\'|\'lsrd\'|\'bary\'|\'geo\'|\'topo\'|\'galacto\'|\'lgroup\'|\'cmb\'] (\'\'=current frame or LSRK for multiple-MS inputs) ',
               'phasecenter': 'Image phase center: position or field index (e.g., \'J2000 17:30:15.0 -25.30.00.0\')',
               'ephemsrcname': 'ephemeris source name',
               'pointingcolumn': 'pointing data column to use [\'direction\'|\'target\'|\'pointing_offset\'|\'source_offset\'|\'encoder\']',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infiles']  = ['']
        a['specunit']  = ''
        a['restfreq']  = ''
        a['scanlist']  = []
        a['field']  = -1
        a['spw']  = [0]
        a['antenna']  = -1
        a['stokes']  = ''
        a['gridfunction']  = 'BOX'
        a['convsupport']  = -1
        a['truncate']  = -1
        a['gwidth']  = -1
        a['jwidth']  = -1
        a['minweight']  = 0.1
        a['outfile']  = ''
        a['overwrite']  = False
        a['imsize']  = [

            ]
        a['cell']  = ''
        a['dochannelmap']  = False
        a['nchan']  = 1
        a['start']  = 0
        a['step']  = 1
        a['outframe']  = ''
        a['phasecenter']  = ''
        a['ephemsrcname']  = ''
        a['pointingcolumn']  = 'direction'

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['dochannelmap']  == True:
            a['nchan'] = 1
            a['start'] = 0
            a['step'] = 1

        if self.parameters['gridfunction']  == 'SF':
            a['convsupport'] = -1

        if self.parameters['gridfunction']  == 'sf':
            a['convsupport'] = -1

        if self.parameters['gridfunction']  == 'GAUSS':
            a['truncate'] = -1
            a['gwidth'] = -1

        if self.parameters['gridfunction']  == 'gauss':
            a['truncate'] = -1
            a['gwidth'] = -1

        if self.parameters['gridfunction']  == 'GJINC':
            a['truncate'] = -1
            a['gwidth'] = -1
            a['jwidth'] = -1

        if self.parameters['gridfunction']  == 'gjinc':
            a['truncate'] = -1
            a['gwidth'] = -1
            a['jwidth'] = -1

        if a.has_key(paramname) :
	      return a[paramname]
sdimagingold_cli = sdimagingold_cli_()
