#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdmathold import sdmathold
class sdmathold_cli_:
    __name__ = "sdmathold"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdmathold_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'expr':None, 'varlist':None, 'antenna':None, 'fluxunit':None, 'telescopeparm':None, 'specunit':None, 'frame':None, 'doppler':None, 'scanlist':None, 'field':None, 'iflist':None, 'pollist':None, 'outfile':None, 'outform':None, 'overwrite':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, expr=None, varlist=None, antenna=None, fluxunit=None, telescopeparm=None, specunit=None, frame=None, doppler=None, scanlist=None, field=None, iflist=None, pollist=None, outfile=None, outform=None, overwrite=None, ):

        """ASAP SD task: do simple arithmetic (subtraction, addition, multiplication, and division) for spectra

	Detailed Description: 
ASAP SD task:
        Do simple arithmetic (subtraction, addition, multiplication, and division) for spectra

        Task Version 2009-02-04 TT

	Arguments :
		expr:	Mathematical expression using spectra
		   Default Value: 

		varlist:	Dictionary of variables and their values used in expr.
		   Default Value: {}

		antenna:	antenna name or id (only effective for MS input)
		   Default Value: 0

		fluxunit:	units for line flux (K,Jy) (''=current)
		   Default Value: 

		telescopeparm:	param of telescope for flux conversion
		   Default Value: 

		specunit:	units for spectral axis (channel,km/s,GHz,''=current)
		   Default Value: 
		   Allowed Values:
				
				channel
				km/s
				GHz
				MHz
				kHz
				Hz

		frame:	frequency reference frame, e.g. LSRK (''=current)
		   Default Value: 

		doppler:	doppler convention, e.g. RADIO (''=current)
		   Default Value: 

		scanlist:	list of scans to use (e.g. [1,2,3,4])
		   Default Value: 

		field:	string for selection by source name
		   Default Value: 

		iflist:	list of IF ids to select (e.g. [0,1])
		   Default Value: 

		pollist:	list of polarization ids to select (e.g. [0,1])
		   Default Value: 

		outfile:	output file name (must be specified)
		   Default Value: 

		outform:	output file format (ASCII,MS,SDFITS,ASAP)
		   Default Value: ASAP
		   Allowed Values:
				ASCII
				ascii
				MS
				ms
				MS2
				ms2
				SDFITS
				sdfits
				ASAP
				asap

		overwrite:	overwrite the output file if already exists
		   Default Value: False

	Returns: void

	Example :

  Keyword arguments:
        expr -- Mathematical expression using scantables 
        varlist -- Dictionary of variables in expr and their values.
                Keys must be coincide with variables used in expr.
                Values are substituted in each value in expr.
        antenna -- antenna name or id (only effective for MS input). 
        fluxunit -- units for line flux
                options: 'K','Jy',''
                default: '' (keep current fluxunit)
                WARNING: For GBT data, see description below.
            >>> fluxunit expandable parameter
                 telescopeparm -- the telescope characteristics
                        options: (str) name or (list) list of gain info
                        default: '' (none set)
                        example: if telescopeparm='', it tries to get the telescope
                                 name from the data.
                                 Full antenna parameters (diameter,ap.eff.) known
                                 to ASAP are
                                 'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                                 'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                                 to 'K' first then convert to a new fluxunit.
                                 telescopeparm=[104.9,0.43] diameter(m), ap.eff.
                                 telescopeparm=[0.743] gain in Jy/K
                                 telescopeparm='FIX' to change default fluxunit
                                 see description below

        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz'
                default: '' (=current)
                example: this will be the units for masklist
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
        outfile -- Name of output file
                default: '' (must be specified)
        outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                         If ASCII, then will append some stuff to
                         the outfile name
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored


        DESCRIPTION:

        Task sdmathold execute a mathematical expression for single dish spectra.
        The spectral data file can be any of the formats supported by
        ASAP (scantable, MS, rpfits, and SDFITS). In the expression, 
        these file names should be put inside of sigle or double quotes.

        You can use variables in the expression. If you want to use, you 
        must define varlist dictionary. Name of variables should be simple, 
        e.g. V0, V1, etc., to avoid unexpected error. Keys of varlist must 
        be name of variables that you used in the expression, and their 
        values will be substituted for variables in the expression. Allowed 
        type for the value is numerical values, one- or two-dimensional lists 
        (Python list or numpy.ndarray), and filename strings that indicate 
        spectral data or ASCII text, which is space-separated list of 
        numerical values consisting of adequate number of rows and columns.  

        The fluxunit, specunit, and frame can be set, otherwise, the current
        settings of the first spectral data in the expression are used.  
        Other selections (e.g. scan No, . IF, Pol) also apply to all 
        the spectral data in the expression, so if any of the data does
        not contains selection, the task will produce no output. 
       
        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  
        
        Example:
        # do on-off/off calculation
        expr='("orion_on_data.asap"-"orion_off_data.asap")/"orion_off_data.asap"
        outfile='orion_cal.asap'
        sdmathold()

        # do on-off/off calculation using varlist
        expr='V0/V1-V2'
        varlist['V0']="orion_on_data.asap"
        varlist['V1']="orion_off_data.asap"
        varlist['V2']=1.0
        outfile='orion_cal.asap'
        sdmathold()

        # interpretation of ASCII file value for varlist
        If the contents of input ASCII file is shown as, 

           0.5 0.3 0.2
           1.0 0.2 0.9

        it is interpreted as a list [[0.5, 0.3, 0.2],[1.0, 0.2, 0.9]]. 

  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdmathold'
        self.__globals__['taskname'] = 'sdmathold'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['expr'] = expr = self.parameters['expr']
            myparams['varlist'] = varlist = self.parameters['varlist']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['fluxunit'] = fluxunit = self.parameters['fluxunit']
            myparams['telescopeparm'] = telescopeparm = self.parameters['telescopeparm']
            myparams['specunit'] = specunit = self.parameters['specunit']
            myparams['frame'] = frame = self.parameters['frame']
            myparams['doppler'] = doppler = self.parameters['doppler']
            myparams['scanlist'] = scanlist = self.parameters['scanlist']
            myparams['field'] = field = self.parameters['field']
            myparams['iflist'] = iflist = self.parameters['iflist']
            myparams['pollist'] = pollist = self.parameters['pollist']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['outform'] = outform = self.parameters['outform']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['expr'] = expr
        mytmp['varlist'] = varlist
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparm'] = telescopeparm
        mytmp['specunit'] = specunit
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdmathold.xml')

        casalog.origin('sdmathold')
	try :
          #if not trec.has_key('sdmathold') or not casac.casac.utils().verify(mytmp, trec['sdmathold']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdmathold'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdmathold', 'sdmathold.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdmathold'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdmathold(expr, varlist, antenna, fluxunit, telescopeparm, specunit, frame, doppler, scanlist, field, iflist, pollist, outfile, outform, overwrite)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdmathold'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdmathold', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['expr']  = ''
        a['varlist']  = {}
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparm':''}]), 
                    2:odict([{'value':'k'}, {'telescopeparm':''}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparm':''}]), 
                    4:odict([{'value':'jy'}, {'telescopeparm':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdmathold', subkey=None):
        desc={'sdmathold': 'ASAP SD task: do simple arithmetic (subtraction, addition, multiplication, and division) for spectra',
               'expr': 'Mathematical expression using spectra',
               'varlist': 'Dictionary of variables and their values used in expr.',
               'antenna': 'antenna name or id (only effective for MS input)',
               'fluxunit': 'units for line flux (K,Jy) (''=current)',
               'telescopeparm': 'param of telescope for flux conversion',
               'specunit': 'units for spectral axis (channel,km/s,GHz,''=current)',
               'frame': 'frequency reference frame, e.g. LSRK (''=current)',
               'doppler': 'doppler convention, e.g. RADIO (''=current)',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'outfile': 'output file name (must be specified)',
               'outform': 'output file format (ASCII,MS,SDFITS,ASAP)',
               'overwrite': 'overwrite the output file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['expr']  = ''
        a['varlist']  = {}
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparm']  = ''
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['fluxunit']  == 'K':
            a['telescopeparm'] = ''

        if self.parameters['fluxunit']  == 'k':
            a['telescopeparm'] = ''

        if self.parameters['fluxunit']  == 'Jy':
            a['telescopeparm'] = ''

        if self.parameters['fluxunit']  == 'jy':
            a['telescopeparm'] = ''

        if a.has_key(paramname) :
	      return a[paramname]
sdmathold_cli = sdmathold_cli_()
