#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdplotold import sdplotold
from task_sdplotold import casalog

class sdplotold_pg_:
    __name__ = "sdplotold"

    def __init__(self) :
       self.__bases__ = (sdplotold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,fluxunit=None,telescopeparm=None,specunit=None,restfreq=None,frame=None,doppler=None,scanlist=None,field=None,iflist=None,pollist=None,beamlist=None,scanaverage=None,timeaverage=None,tweight=None,polaverage=None,pweight=None,kernel=None,kwidth=None,plottype=None,stack=None,panel=None,flrange=None,sprange=None,linecat=None,linedop=None,subplot=None,colormap=None,linestyles=None,linewidth=None,histogram=None,center=None,cell=None,scanpattern=None,header=None,headsize=None,plotstyle=None,margin=None,legendloc=None,outfile=None,overwrite=None,):

        """ASAP SD plotting task 
  Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        fluxunit -- units for line flux
                options: 'K','Jy',''
                default: '' (keep current fluxunit)
                WARNING: For GBT data, see description below.
            >>> fluxunit expandable parameter
                 telescopeparm -- the telescope characteristics
                        options: (str) name or (list) list of gain info
                        default: '' (none set)
                        example: if telescopeparm='', it tries to get the telescope
                                 name from the data.
                                 Full antenna parameters (diameter,ap.eff.) known
                                 to ASAP are
                                 'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                                 'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                                 to 'K' first then convert to a new fluxunit.
                                 telescopeparm=[104.9,0.43] diameter(m), ap.eff.
                                 telescopeparm=[0.743] gain in Jy/K
                                 telescopeparm='FIX' to change default fluxunit
                                 see description below

        specunit -- units for spectral axis
                options: (str) 'channel','km/s','GHz','MHz','kHz','Hz'
                default: '' (=current)
                example: this will be the units for masklist
            >>> specunit expandable parameter
                 restfreq -- rest frequency
                         default: '' (use current setting)
                         example: 4.6e10 (float value in Hz),
                                  '46GHz' (string with unit),
                                  ['345.8GHz', 347.0e9, 356.7e9] (for each IF)
                                  [{'name':'CO','value':345e9}] (a value with name)
        frame -- frequency frame for spectral axis
                options: (str) 'LSRK','REST','TOPO','LSRD','BARY',
                         'GEO','GALACTO','LGROUP','CMB'
                default: currently set frame in scantable
                WARNING: frame='REST' not yet implemented
        doppler -- doppler mode
                options: (str) 'RADIO','OPTICAL','Z','BETA','GAMMA'
                default: currently set doppler in scantable
        scanlist -- list or string of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24], or "21~24"
                this selection is in addition to field, iflist, pollist,
                and beamlist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, pollist,
                and beamlist
        iflist -- list or string of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, pollist,
                and beamlist
        pollist -- list or string of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, iflist,
                and beamlist
        beamlist -- list or string of beam id numbers to select
                default: [] (use all beams)
                example: [1]
                this selection is in addition to scanlist, field, iflist,
                and pollist
        scanaverage -- average integs within scans
                options: (bool) True,False
                default: False
        timeaverage -- average times for multiple scan cycles
                options: (bool) True,False
                default: False
                example: if True, this happens after calibration
                >>>timeaverage expandable parameter
                 tweight -- weighting for time average
                         options: 'var'   (1/var(spec) weighted)
                                  'tsys'  (1/Tsys**2 weighted)
                                  'tint'  (integration time weighted)
                                  'tintsys'  (Tint/Tsys**2)
                                  'median'  ( median averaging)
                         default: 'tintsys'
        polaverage -- average polarizations
                options: (bool) True,False
                default: False
                >>>polaverage expandable parameter
                 pweight -- weighting for polarization average
                         options: 'var'  (1/var(spec) weighted)
                                  'tsys' (1/Tsys**2 weighted)
                         default: 'tsys'
        kernel -- type of spectral smoothing
                options: 'hanning','gaussian','boxcar', 'none'
                default: 'none'
                >>>kernel expandable parameter
                 kwidth -- width of spectral smoothing kernel
                         options: (int) in channels
                         default: 5
                         example: 5 or 10 seem to be popular for boxcar
                                  ignored for hanning (fixed at 5 chans)
                                  (0 will turn off gaussian or boxcar)
        plottype -- type of plot
                 options: 'spectra','totalpower','pointing','azel','grid'
                 default: 'spectra'
            >>> plottype expandable parameters
                stack -- code for stacking on single plot for spectral plotting
                        options: 'p','b','i','t','s','r' or
                                 'pol', 'beam', 'if', 'time', 'scan', 'row'
                        default: 'p'
                        example: maximum of 16 stacked spectra
                                 stack by pol, beam, if, time, scan
                        Note stack selection is ignored when panel='r'.
                panel -- code for splitting into multiple panels for spectral plotting
                        options: 'p','b','i','t','s','r' or
                                 'pol', 'beam', 'if', 'time', 'scan', 'row'
                        default: 'i'
                        example: maximum of 16 panels
                                 panel by pol, beam, if, time, scan
                        Note panel selection is ignored when stack='r'.
                flrange -- range for flux axis of plot for spectral plotting
                        options: (list) [min,max]
                        default: [] (full range)
                        example: flrange=[-0.1,2.0] if 'K'
                                 assumes current fluxunit
                sprange -- range for spectral axis of plot
                        options: (list) [min,max]
                        default: [] (full range)
                        example: sprange=[42.1,42.5] if 'GHz'
                                 assumes current specunit
                linecat -- control for line catalog plotting for spectral plotting
                        options: (str) 'all','none' or by molecule
                        default: 'none' (no lines plotted)
                        example: linecat='SiO' for SiO lines
                                 linecat='*OH' for alcohols
                                 uses sprange to limit catalog
                        WARNING: specunit must be in frequency (*Hz)
                                 to plot from the line catalog!
                                 and must be 'GHz' or 'MHz' to use
                                 sprange to limit catalog
                linedop -- doppler offset for line catalog plotting (spectral plotting)
                        options: (float) doppler velocity (km/s)
                        default: 0.0
                        example: linedop=-30.0
                center -- the central direction of gridding
                        default: '' (map center)
                        example: 'J2000 19h30m00 -40d00m00'
                        Note currently only supports 'J2000' as direction frame
                cell -- x and y cell size of gridding
                        default: [] (map extent devided by # of subplots in x and y)
                        example: cell=['1.0arcmin','1.0arcmin']
                                 cell='1.0arcmin' (equivalent to the example above)
                        Note default number of subplots is 1 x 1 in plottype='grid'.
                subplot -- number of subplots (row and column) on a page
                           NOTICE plotter will slow down when a large number is specified
                           default: -1 (auto. for plottype='spectra', 1x1 for plottype='grid')
                           example: 23 (2 rows by 3 columns)
                colormap -- the colours to be used for plot lines. 
                        default: None
                        example: colormap="green red black cyan magenta" (html standard)
                                 colormap="g r k c m" (abbreviation)
                                 colormap="#008000 #00FFFF #FF0090" (RGB tuple)
                                 The plotter will cycle through these colours 
                                 when lines are overlaid (stacking mode).
                linestyles -- the linestyles to be used for plot lines. 
                        default: None
                        example: linestyles="line dashed dotted dashdot dashdotdot dashdashdot". 
                                 The plotter will cycle through these linestyles 
                                 when lines are overlaid (stacking mode). 
                        WARNING: Linestyles can be specified only one color has been set. 
                linewidth -- width of plotted lines. 
                        default: 1
                        example: linewidth=1 (integer)
                                 linewidth=0.75 (double)
                histogram -- plot histogram
                        options: (bool) True, False
                        default: False
                scanpattern -- plot additional lines on the plot to indicate scan patterns
                               when plottype='pointing'
                        options: (bool) True, False
                        default: False
        header -- print header information on the plot
                options: (bool) True, False
                default: True
                The header information is printed only on the logger when 
                plottype = 'azel' and 'pointing'. 
            >>> header expandable parameter
                headsize -- header font size
                        options: (int)
                        default: 9
        plotstyle -- customise plot settings
                options: (bool) True, False
                default: False
            >>> plotstyle expandable parameter
                margin -- a list of subplot margins in figure coordinate (0-1), 
                          i.e., fraction of the figure width or height.
                          The order of elements should be:
                          [left, bottom, right, top, horizontal space btw panels,
                          vertical space btw panels]
                        example: margin = [0.125, 0.1, 0.9, 0.9, 0.2, 0.2]
                legendloc -- legend location on the axes (0-10)
                           options: (integer) 0 -10 
                                    see help of "sd.plotter.set_legend" for 
                                    the detail of location. Note that 0 ('best')
                                    is very slow. 
                           default: 1 ('upper right')
        outfile -- file name for hardcopy output
                options: (str) filename.eps,.ps,.png
                default: '' (no hardcopy)
                example: 'specplot.eps','specplot.png'
                Note this autodetects the format from the suffix (.eps,.ps,.png).
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False


        DESCRIPTION:

        Task sdplotold displays single-dish spectra, total power,
        or pointing direction of input data.
        It assumes that the spectra have been calibrated.
        It does allow selection of scans, IFs, polarizations, and
        some time and channel averaging/smoothing options also,
        but does not write out this data.

        This task adds an additional toolbar to Matplotlib plotter. 
        See the cookbook for details of its capability.

        *** Data selection ***
        This task allows data selection via field name, scan, IF,
        polarization and beam IDs. Selection of field allows pattern
        matching using asterisk, e.g., 'FLS3a*'. Selection of scans,
        IFs, polarizations, and beams, is possible either by a list
        of IDs or by a CASA type selection syntax using a string of 
        comma separated numbers with operaters, i.e., '~', '>', '>=',
        '<', and '<='.
        For example, the following two selections are equivalent:
        scanlist = [0, 1, 2, 7, 8, 9, 15]
        scanlist = "<3,7~9,15"

        *** control of plot lines in 'spectra' and 'grid' plottype ***
        Note that colormap and linestyles cannot be controlled at a time.
        The 'linestyles' is ignored if both of them are specified.
        Some plot options, like changing titles, legends, fonts,
        and the like are not supported in this task.  You should use
        sd.plotter from the ASAP toolkit directly for this.

        *** available plottypes ***
        * plottype = 'spectra' plots single dish spectra. Multiple scans,
          IFs, polarizations, and beams can be handles through stacking
          and panelling.
          This task uses the JPL line catalog as supplied by ASAP.
          If you wish to use a different catalog, or have it plot
          the line IDs from top or bottom (rather than alternating),
          then you will need to explore the sd toolkit also.
        * plottype = 'grid' plots spectra based on their pointing direction.
          The spectra are gridded by direction before plotting.
          Multiple IFs and polarizations are not handled in this mode. Only
          the first IF and polarizaion is gridded and plotted if data 
          includes multiple IDs after selections are applied. Hence, over
          plotting is not available

        Currently most of the parameters are ignored in the following modes.

        * plottype='totalpower' is used to plot the total power data.
          and only plot option is amplitude versus data row number.
        * plottype='azel' plots azimuth and elevation tracks of the source.
        * plottype='pointing' plots antenna poinitings.

        ASAP recognizes the data of the "AT" telescopes, but currently
        does not know about the GBT or any other telescope. This task
        does know about GBT. Telescope name is obtained from the data.
        If you wish to change the fluxunit (see below), and telescopeparm='',
        for the AT telescopes it will use internal telescope parameters for
        flux conversion. For GBT, it will use an approximate aperture
        efficiency conversion.  If you give telescopeparm a list,
        then if the list has a single float it is assumed to
        be the gain in Jy/K, if two or more elements they are assumed
        to be telescope diameter (m) and aperture efficiency
        respectively.

        WARNING: be careful plotting otf data with lots of fields!

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdplotold'
        myf['taskname'] = 'sdplotold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['fluxunit'] = fluxunit = myf['fluxunit']
            myparams['telescopeparm'] = telescopeparm = myf['telescopeparm']
            myparams['specunit'] = specunit = myf['specunit']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['beamlist'] = beamlist = myf['beamlist']
            myparams['scanaverage'] = scanaverage = myf['scanaverage']
            myparams['timeaverage'] = timeaverage = myf['timeaverage']
            myparams['tweight'] = tweight = myf['tweight']
            myparams['polaverage'] = polaverage = myf['polaverage']
            myparams['pweight'] = pweight = myf['pweight']
            myparams['kernel'] = kernel = myf['kernel']
            myparams['kwidth'] = kwidth = myf['kwidth']
            myparams['plottype'] = plottype = myf['plottype']
            myparams['stack'] = stack = myf['stack']
            myparams['panel'] = panel = myf['panel']
            myparams['flrange'] = flrange = myf['flrange']
            myparams['sprange'] = sprange = myf['sprange']
            myparams['linecat'] = linecat = myf['linecat']
            myparams['linedop'] = linedop = myf['linedop']
            myparams['subplot'] = subplot = myf['subplot']
            myparams['colormap'] = colormap = myf['colormap']
            myparams['linestyles'] = linestyles = myf['linestyles']
            myparams['linewidth'] = linewidth = myf['linewidth']
            myparams['histogram'] = histogram = myf['histogram']
            myparams['center'] = center = myf['center']
            myparams['cell'] = cell = myf['cell']
            myparams['scanpattern'] = scanpattern = myf['scanpattern']
            myparams['header'] = header = myf['header']
            myparams['headsize'] = headsize = myf['headsize']
            myparams['plotstyle'] = plotstyle = myf['plotstyle']
            myparams['margin'] = margin = myf['margin']
            myparams['legendloc'] = legendloc = myf['legendloc']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(flrange)==float: flrange=[flrange]
        if type(cell)==float: cell=[cell]
        if type(margin)==float: margin=[margin]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparm'] = telescopeparm
        mytmp['specunit'] = specunit
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['beamlist'] = beamlist
        mytmp['scanaverage'] = scanaverage
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['plottype'] = plottype
        mytmp['stack'] = stack
        mytmp['panel'] = panel
        mytmp['flrange'] = flrange
        mytmp['sprange'] = sprange
        mytmp['linecat'] = linecat
        mytmp['linedop'] = linedop
        mytmp['subplot'] = subplot
        mytmp['colormap'] = colormap
        mytmp['linestyles'] = linestyles
        mytmp['linewidth'] = linewidth
        mytmp['histogram'] = histogram
        mytmp['center'] = center
        mytmp['cell'] = cell
        mytmp['scanpattern'] = scanpattern
        mytmp['header'] = header
        mytmp['headsize'] = headsize
        mytmp['plotstyle'] = plotstyle
        mytmp['margin'] = margin
        mytmp['legendloc'] = legendloc
        mytmp['outfile'] = outfile
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdplotold.xml')

        casalog.origin('sdplotold')
        if not trec.has_key('sdplotold') or not casac.utils().verify(mytmp, trec['sdplotold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdplotold           #####')
          casalog.post('')
          result = sdplotold(infile, antenna, fluxunit, telescopeparm, specunit, restfreq, frame, doppler, scanlist, field, iflist, pollist, beamlist, scanaverage, timeaverage, tweight, polaverage, pweight, kernel, kwidth, plottype, stack, panel, flrange, sprange, linecat, linedop, subplot, colormap, linestyles, linewidth, histogram, center, cell, scanpattern, header, headsize, plotstyle, margin, legendloc, outfile, overwrite)
          casalog.post('')
          casalog.post('##### End Task: sdplotold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdplotold', 'sdplotold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdplotold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdplotold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['specunit']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['beamlist']  = []
        a['scanaverage']  = False
        a['timeaverage']  = False
        a['polaverage']  = False
        a['kernel']  = 'none'
        a['plottype']  = 'spectra'
        a['header']  = True
        a['plotstyle']  = False
        a['outfile']  = ''
        a['overwrite']  = False

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparm':""}]), 
                    2:odict([{'value':'k'}, {'telescopeparm':""}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparm':""}]), 
                    4:odict([{'value':'jy'}, {'telescopeparm':""}])}
        a['specunit'] = {
                    0:{'value':''}, 
                    1:{'value':'channel'}, 
                    2:odict([{'value':'km/s'}, {'restfreq':''}]), 
                    3:{'value':'GHz'}, 
                    4:{'value':'MHz'}, 
                    5:{'value':'kHz'}, 
                    6:{'value':'Hz'}}
        a['kernel'] = {
                    0:{'value':'none'}, 
                    1:{'value':'hanning'}, 
                    2:odict([{'value':'gaussian'}, {'kwidth':5}]), 
                    3:odict([{'value':'boxcar'}, {'kwidth':5}])}
        a['plottype'] = {
                    0:odict([{'value':'spectra'}, {'stack':'p'}, {'panel':'i'}, {'flrange':[]}, {'sprange':[]}, {'linecat':'none'}, {'linedop':0.0}, {'subplot':-1}, {'colormap':'none'}, {'linestyles':'none'}, {'linewidth':1}, {'histogram':False}]), 
                    1:odict([{'value':'grid'}, {'center':''}, {'cell':[]}, {'subplot':-1}, {'flrange':[]}, {'sprange':[]}, {'colormap':'none'}, {'linestyles':'none'}, {'linewidth':1}]), 
                    2:{'value':'totalpower'}, 
                    3:odict([{'value':'pointing'}, {'stack':'t'}, {'colormap':'none'}, {'scanpattern':False}]), 
                    4:{'value':'azel'}}
        a['timeaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'tweight':'tintsys'}])}
        a['polaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'pweight':'tsys'}])}
        a['header'] = {
                    0:odict([{'value':True}, {'headsize':9}]), 
                    1:{'value':False}}
        a['plotstyle'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'margin':[]}, {'legendloc':1}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdplotold', subkey=None):
        desc={'sdplotold': 'ASAP SD plotting task ',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'fluxunit': 'units for line flux (K,Jy) (''=current)',
               'telescopeparm': 'param of telescope for flux conversion',
               'specunit': 'units for spectral axis (channel,km/s,GHz)',
               'restfreq': 'rest frequency (default unit: Hz)',
               'frame': 'frequency reference frame, e.g. LSRK (''=current)',
               'doppler': 'doppler convention, e.g. RADIO (''=current)',
               'scanlist': 'list or string of scans to use (e.g. [1,2,3,4] or "1~4")',
               'field': 'string for selection by source name',
               'iflist': 'list or string of IF ids to select (e.g. [0,1] or "0~1")',
               'pollist': 'list or string of polarization ids to select (e.g. [0,1])',
               'beamlist': 'list or string of beam ids to select (e.g. [0,1])',
               'scanaverage': 'average integs within scans (True,False)',
               'timeaverage': 'average scans over time (True,False)',
               'tweight': 'weighting for time averaging',
               'polaverage': 'average over polarizations (True,False)',
               'pweight': '',
               'kernel': 'type of spectral smoothing',
               'kwidth': 'width of spectral kernel in channels',
               'plottype': 'type of plot (spectra, totalpower, azel, pointing, grid)',
               'stack': 'code for stacking on single plot for spectral plotting',
               'panel': 'code for splitting into multiple panels for spectral plotting',
               'flrange': 'range for flux axis of plot for spectral plotting',
               'sprange': 'range for spectral axis of plot',
               'linecat': 'control for line catalog plotting for spectral plotting',
               'linedop': 'doppler offset for line catalog plotting (spectral plotting)',
               'subplot': 'number of subplots (row and column)',
               'colormap': 'the colours to be used for plot lines',
               'linestyles': 'the linestyles to be used for plot lines',
               'linewidth': 'width of plotted lines',
               'histogram': 'plot histogram',
               'center': 'the central direction of gridding. (default: map center)',
               'cell': 'x and y cell size, e.g., ["1arcmin","1arcmin"]. (default map extent/subplot number)',
               'scanpattern': 'plot scan patterns.',
               'header': 'print header information on the plot',
               'headsize': 'header fontsize',
               'plotstyle': 'customise plot settings',
               'margin': 'subplot margins in figure coordinate',
               'legendloc': 'legend location',
               'outfile': 'file name for hardcopy output',
               'overwrite': 'overwrite the output file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparm']  = ''
        a['specunit']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['beamlist']  = []
        a['scanaverage']  = False
        a['timeaverage']  = False
        a['tweight']  = 'tintsys'
        a['polaverage']  = False
        a['pweight']  = 'tsys'
        a['kernel']  = 'none'
        a['kwidth']  = 5
        a['plottype']  = 'spectra'
        a['stack']  = 'p'
        a['panel']  = 'i'
        a['flrange']  = []
        a['sprange']  = []
        a['linecat']  = 'none'
        a['linedop']  = 0.0
        a['subplot']  = -1
        a['colormap']  = 'none'
        a['linestyles']  = 'none'
        a['linewidth']  = 1
        a['histogram']  = False
        a['center']  = ''
        a['cell']  = []
        a['scanpattern']  = False
        a['header']  = True
        a['headsize']  = 9
        a['plotstyle']  = False
        a['margin']  = []
        a['legendloc']  = 1
        a['outfile']  = ''
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdplotold_pg = sdplotold_pg_()
