#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdreduce import sdreduce
from task_sdreduce import casalog

class sdreduce_pg_:
    __name__ = "sdreduce"

    def __init__(self) :
       self.__bases__ = (sdreduce_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,fluxunit=None,telescopeparam=None,field=None,spw=None,restfreq=None,frame=None,doppler=None,timerange=None,scan=None,pol=None,calmode=None,fraction=None,noff=None,width=None,elongated=None,markonly=None,plotpointings=None,tau=None,average=None,timeaverage=None,tweight=None,scanaverage=None,averageall=None,polaverage=None,pweight=None,kernel=None,kwidth=None,chanwidth=None,maskmode=None,thresh=None,avg_limit=None,edge=None,blfunc=None,order=None,npiece=None,applyfft=None,fftmethod=None,fftthresh=None,addwn=None,rejwn=None,clipthresh=None,clipniter=None,verifycal=None,verifysm=None,verifybl=None,verbosebl=None,bloutput=None,blformat=None,showprogress=None,minnrow=None,outfile=None,outform=None,overwrite=None,plotlevel=None,):

        """ASAP SD task: do sdcal, sdaverage, and sdbaseline in one task
Keyword arguments:
infile -- name of input SD dataset
antenna -- select an antenna name or ID
        default: 0
        example: 'PM03'
        NOTE this parameter is effective only for MS input
fluxunit -- units for line flux
        options: 'K','Jy',''
        default: '' (keep current fluxunit in data)
        WARNING: For GBT data, see description below.
    >>> fluxunit expandable parameter
        telescopeparam -- parameters of telescope for flux conversion
                options: (str) name or (list) list of gain info
                default: '' (none set)
                example: if telescopeparam='', it tries to get the telescope
                         name from the data.
                         Full antenna parameters (diameter,ap.eff.) known
                         to ASAP are
                         'ATPKSMB', 'ATPKSHOH', 'ATMOPRA', 'DSS-43',
                         'CEDUNA','HOBART'. For GBT, it fixes default fluxunit
                         to 'K' first then convert to a new fluxunit.
                         telescopeparam=[104.9,0.43] diameter(m), ap.eff.
                         telescopeparam=[0.743] gain in Jy/K
                         telescopeparam='FIX' to change default fluxunit
                         see description below

field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)/channels
       NOTE channel range selections are interpreted as mask regions to
       INCLUDE in BASELINE fit, and ignored in the other operations.
       when maskmode is 'auto' or 'interact', the channel mask 
       will be applied first before fitting as base mask
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                 spw='0:5~61' (IF ID 0; channels 5 to 61)
                 spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                 spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                 spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
        this selection is in addition to the other selections to data
    >>> spw expandable parameters
        restfreq -- the rest frequency
                    available type includes float, int, string, list of float, 
                    list of int, list of string, and list of dictionary. the 
                    default unit of restfreq in case of float, int, or string 
                    without unit is Hz. string input can be a value only 
                    (treated as Hz) or a value followed by unit for which 'GHz',
                    'MHz','kHz',and 'Hz' are available. 
                    a list can be used to set different rest frequencies for 
                    each IF. the length of list input must be number of IFs. 
                    dictionary input should be a pair of line name and 
                    frequency with keys of 'name' and 'value', respectively. 
                    values in the dictionary input follows the same manner as 
                    as for single float or string input. 
                example: 345.796
                         '1420MHz'
                         [345.8, 347.0, 356.7]
                         ['345.8MHz', '347.0MHz', '356.7MHz']
                         [{'name':'CO','value':345}]
        frame -- frequency reference frame
                options: 'LSRK', 'TOPO', 'LSRD', 'BARY', 'GALACTO', 'LGROUP', 'CMB'
                default: '' (keep current frame in data)
        doppler -- doppler convention (effective only when spw is in
                   velocity unit)
                options: 'RADIO', 'OPTICAL', 'Z', 'BETA', or 'GAMMA'
                default: '' (keep current doppler setting in data)

timerange -- select data by time range
        default: '' (use all)
        example: timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                 Note: YYYY/MM/DD can be dropped as needed:
                 timerange='09:14:00~09:54:00' # this time range
                 timerange='09:44:00' # data within one integration of time
                 timerange='>10:24:00' # data after this time
                 timerange='09:44:00+00:13:00' #data 13 minutes after time
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data

calmode -- calibration mode
        options: 'ps','nod','otf','otfraster',
                 'fs','quotient','none'
        default: 'none'
        example: choose mode 'none' if you have already calibrated
                 and want to correct for atmospheric opacity defined
                 by tau, subtract baseline or average/smooth spectra.
    >>> calmode expandable parameter
         fraction -- edge marker parameter of 'otf' and 'otfraster'.
                     Specify a number of OFF integrations (at each
                     side of the raster rows in 'otfraster' mode)
                     as a fraction of total number of integrations.
                     In 'otfraster' mode, number of integrations 
                     to be marked as OFF, n_off, is determined by 
                     the following formula,

                        n_off = floor(fraction * n),

                     where n is number of integrations per raster 
                     row. Note that n_off from both sides will be  
                     marked as OFF so that twice of specified 
                     fraction will be marked at most. For example, 
                     if you specify fraction='10%', resultant 
                     fraction of OFF integrations will be 20% at 
                     most.
                     In 'otf' mode, n_off is given by,

                        n_off = floor(fraction * n),

                     where n is number of total integrations.
                     n_off is used as criterion of iterative marking
                     process. Therefore, resulting total number of 
                     OFFs will be larger than n_off. In practice,
                     fraction is a geometrical fraction of edge
                     region. Thus, if integrations are concentrated
                     on edge region (e.g. some of Lissajous
                     patterns), then resulting n_off may be 
                     unexpectedly large.
                 default: '10%'
                 options: '20%' in string style or float value less 
                          than 1.0 (e.g. 0.15).
                          'auto' is available only for 'otfraster'. 
         noff -- edge marking parameter for 'otfraster'.
                 It is used to specify a number of OFF scans near 
                 edge directly. Value of noff comes before setting 
                 by fraction. Note that n_off from both sides will 
                 be marked as OFF so that twice of specified noff 
                 will be marked at most.
                 default: -1 (use fraction)
                 options: any positive integer
         width -- edge marking parameter for 'otf'.
                  Pixel width with respect to a median spatial 
                  separation between neighboring two data in time.
                  Default will be fine in most cases.
                 default: 0.5
                 options: float value
         elongated -- edge marking parameter for 'otf'.
                      Set True only if observed area is elongeted 
                      in one direction.
                 options: (bool) True, False
                 default: False
         markonly -- set True if you want to save data just after 
                     edge marking (i.e. uncalibrated data) to see 
                     how OFF scans are defined.
                 options: (bool) True, False
                 default: False
         plotpointings -- load plotter and plot pointing directions of
                          ON and OFF scans.
                 options: (bool) True, False
                 default: False

tau -- the zenith atmospheric optical depth for correction 
        default: 0.0 (no correction)
average -- averaging on spectral data 
        options: (bool) True,False
        default: False

    >>>average expandable parameter
        timeaverage -- average spectra over time
                options: (bool) True, False
                default: False
                example: if True, this happens after calibration
        tweight -- weighting for time averaging (effective only when
                   timeaverage=True)
                options: 'var'   (1/var(spec) weighted)
                         'tsys'  (1/Tsys**2 weighted)
                         'tint'  (integration time weighted)
                         'tintsys'  (Tint/Tsys**2)
                         'median'  ( median averaging)
                default: 'tintsys'
        scanaverage -- average spectra within a scan number (effective
                       only when timeaverage=True)
                       when True, spectra are NOT averaged over
                       different scan numbers. 
                options: (bool) True, False
                default: False
        averageall -- average multi-resolution spectra (effective only
                      when timeaverage=True)
                      spectra are averaged by referring their frequency
                      coverage 
                 default: False
        polaverage -- average spectra over polarizations
                options: (bool) True, False
                default: False
        pweight -- weighting for polarization averaging (effective only
                   when polaverage=True)
                options: 'var'  (1/var(spec) weighted)
                         'tsys' (1/Tsys**2 weighted)
                default: 'tsys'

kernel -- type of spectral smoothing kernel
        options: 'none', 'hanning','gaussian','boxcar','regrid', ''(='none')
        default: 'none' (no smoothing)

    >>>kernel expandable parameter
        kwidth -- width of spectral smoothing kernel
                options: (int) in channels 
                default: 5
	        example: 5 or 10 seem to be popular for boxcar
	                 ignored for hanning (fixed at 5 chans)
                         (0 will turn off gaussian or boxcar)
        chanwidth -- channel width of regridded spectra
	         default: '5' (in channels)
	         example: '500MHz', '0.2km/s'

maskmode -- mode of setting additional channel masks
        options: 'auto', 'list', or 'interact'
        default: 'auto'
        example: maskmode='auto' runs linefinder to detect line regions 
                 to be excluded from fitting. this mode requires three 
                 expandable parameters: thresh, avg_limit, and edge.
                 USE WITH CARE! May need to tweak the expandable parameters.
                 maskmode='list' uses the given masklist only: no additional 
                 masks applied.
                 maskmode='interact' allows users to manually modify the 
                 mask regions by dragging mouse on the spectrum plotter GUI.
                 use LEFT or RIGHT button to add or delete regions, 
                 respectively.
       
    >>> maskmode expandable parameters
        thresh -- S/N threshold for linefinder. a single channel S/N ratio
                  above which the channel is considered to be a detection. 
                default: 5
        avg_limit -- channel averaging for broad lines. a number of
                     consecutive channels not greater than this parameter
                     can be averaged to search for broad lines.
                default: 4
        edge -- channels to drop at beginning and end of spectrum
                default: 0
                example: edge=[1000] drops 1000 channels at beginning AND end.
                         edge=[1000,500] drops 1000 from beginning and 500
                         from end.
        Note: For bad baselines threshold should be increased,
        and avg_limit decreased (or even switched off completely by
        setting this parameter to 1) to avoid detecting baseline
        undulations instead of real lines.

blfunc -- baseline model function
        options: 'poly', 'chebyshev', 'cspline', or 'sinusoid'
        default: 'none' (no baseline fit)
        example: blfunc='poly' uses a single polynomial line of 
                 any order which should be given as an expandable 
                 parameter 'order' to fit baseline. 
                 blfunc='chebyshev' uses Chebyshev polynomials. 
                 blfunc='cspline' uses a cubic spline function, a piecewise 
                 cubic polynomial having C2-continuity (i.e., the second 
                 derivative is continuous at the joining points). 
                 blfunc='sinusoid' uses a combination of sinusoidal curves. 
    >>> blfunc expandable parameters
        order -- order of baseline model function
                options: (int) (<0 turns off baseline fitting)
                default: 5
                example: typically in range 2-9 (higher values
                         seem to be needed for GBT)
        npiece -- number of the element polynomials of cubic spline curve
                options: (int) (<0 turns off baseline fitting)
                default: 2
        applyfft -- automatically set wave numbers of sinusoidal functions 
                    for fitting by applying some method like FFT.
                options: (bool) True, False
                default: True
        fftmethod -- method to be used when applyfft=True. Now only 
                     'fft' is available and it is the default.
        fftthresh -- threshold to select wave numbers to be used for 
                     sinusoidal fitting. both (float) and (str) accepted.
                     given a float value, the unit is set to sigma.
                     for string values, allowed formats include:
                     'xsigma' or 'x' (= x-sigma level. e.g., '3sigma'), or
                     'topx' (= the x strongest ones, e.g. 'top5').
                default is 3.0 (unit: sigma).
        addwn -- additional wave number(s) of sinusoids to be used 
                 for fitting. 
                 (list) and (int) are accepted to specify every
                 wave numbers. also (str) can be used in case
                 you need to specify wave numbers in a certain range.
                 default: [0] (i.e., constant is subtracted at least)
                 example: 0
                          [0,1,2]
                          'a-b' (= a, a+1, a+2, ..., b-1, b),
                          '<a'  (= 0,1,...,a-2,a-1),
                          '>=a' (= a, a+1, ... up to the maximum wave
                                   number corresponding to the Nyquist
                                   frequency for the case of FFT).
        rejwn -- wave number(s) of sinusoid NOT to be used for fitting.
                 can be set just as addwn but has higher priority:
                 wave numbers which are specified both in addwn
                 and rejwn will NOT be used. 
                 default: []
        clipthresh -- clipping threshold for iterative fitting
                 default: 3
        clipniter -- maximum iteration number for iterative fitting
                 default: 0 (no iteration, i.e., no clipping)

verifycal -- interactively verify the results of calibration
          See description of verify parameter in the task, sdcal,
          for details.
        options: (bool) True,False
        default: False
verifysm -- interactively verify the results of smoothing for each
            spectrum.
          See description of verify parameter in the task, sdaverage,
          for details.
        options: (bool) True,False
        default: False
        Note: verification is not yet available for kernel='regrid'
verifybl -- interactively verify the results of baseline fitting for
            each spectrum.
          See description of verify parameter in the task, sdbaseline,
          for details.
        options: (bool) True,False
        default: False
        NOTE: Currently available only when blfunc='poly'
verbosebl -- output fitting results to logger. If False, the fitting results
             including coefficients, residual rms, etc., are not output to
             the CASA logger, while the processing speed gets faster.
        options: (bool) True, False
        default: True
bloutput -- output fitting results to a text file. if False, the fitting
            results including coefficients, residual rms, etc., are not
            output to a text file (<outfile>_blparam.txt), while
            the processing speed gets faster.
        options: (bool) True, False
        default: True
blformat -- format of the logger output and text file specified with bloutput
        options: '', 'csv'
        default: '' (same as in the past, easy to read but huge)
showprogress -- show progress status for large data
        options: (bool) True, False
        default: True
    >>> showprogress expandable parameter
        minnrow -- minimum number of input spectra to show progress status
                 default: 1000
outfile -- name of output file
        default: '' (<infile>_cal)
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
overwrite -- overwrite the output file if already exists
        options: (bool) True,False
        default: False
        NOTE this parameter is ignored when outform='ASCII'
plotlevel -- control for plotting and summary of results
        options: (int) 0, 1, 2, and their negative counterparts
	default: 0 (no plotting)
	example: plotlevel=1; show plot of spectra (see description of
                              the parameter in sdcal, sdaverage, and sdbaseline)
                 plotlevel=2; additionally list data before and after operation.
                 plotlevel<0 as abs(plotlevel), e.g.
                 -1 => hardcopy of final plot at each step, i.e.,
                 <outfile>_calspec.eps, <outfile>_smspec.eps, and/or
                 <outfile>_bsspec.eps

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the 
data filler is available. However, the functionality is not well 
tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdreduce'
        myf['taskname'] = 'sdreduce'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['fluxunit'] = fluxunit = myf['fluxunit']
            myparams['telescopeparam'] = telescopeparam = myf['telescopeparam']
            myparams['field'] = field = myf['field']
            myparams['spw'] = spw = myf['spw']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['frame'] = frame = myf['frame']
            myparams['doppler'] = doppler = myf['doppler']
            myparams['timerange'] = timerange = myf['timerange']
            myparams['scan'] = scan = myf['scan']
            myparams['pol'] = pol = myf['pol']
            myparams['calmode'] = calmode = myf['calmode']
            myparams['fraction'] = fraction = myf['fraction']
            myparams['noff'] = noff = myf['noff']
            myparams['width'] = width = myf['width']
            myparams['elongated'] = elongated = myf['elongated']
            myparams['markonly'] = markonly = myf['markonly']
            myparams['plotpointings'] = plotpointings = myf['plotpointings']
            myparams['tau'] = tau = myf['tau']
            myparams['average'] = average = myf['average']
            myparams['timeaverage'] = timeaverage = myf['timeaverage']
            myparams['tweight'] = tweight = myf['tweight']
            myparams['scanaverage'] = scanaverage = myf['scanaverage']
            myparams['averageall'] = averageall = myf['averageall']
            myparams['polaverage'] = polaverage = myf['polaverage']
            myparams['pweight'] = pweight = myf['pweight']
            myparams['kernel'] = kernel = myf['kernel']
            myparams['kwidth'] = kwidth = myf['kwidth']
            myparams['chanwidth'] = chanwidth = myf['chanwidth']
            myparams['maskmode'] = maskmode = myf['maskmode']
            myparams['thresh'] = thresh = myf['thresh']
            myparams['avg_limit'] = avg_limit = myf['avg_limit']
            myparams['edge'] = edge = myf['edge']
            myparams['blfunc'] = blfunc = myf['blfunc']
            myparams['order'] = order = myf['order']
            myparams['npiece'] = npiece = myf['npiece']
            myparams['applyfft'] = applyfft = myf['applyfft']
            myparams['fftmethod'] = fftmethod = myf['fftmethod']
            myparams['fftthresh'] = fftthresh = myf['fftthresh']
            myparams['addwn'] = addwn = myf['addwn']
            myparams['rejwn'] = rejwn = myf['rejwn']
            myparams['clipthresh'] = clipthresh = myf['clipthresh']
            myparams['clipniter'] = clipniter = myf['clipniter']
            myparams['verifycal'] = verifycal = myf['verifycal']
            myparams['verifysm'] = verifysm = myf['verifysm']
            myparams['verifybl'] = verifybl = myf['verifybl']
            myparams['verbosebl'] = verbosebl = myf['verbosebl']
            myparams['bloutput'] = bloutput = myf['bloutput']
            myparams['blformat'] = blformat = myf['blformat']
            myparams['showprogress'] = showprogress = myf['showprogress']
            myparams['minnrow'] = minnrow = myf['minnrow']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']

        if type(edge)==int: edge=[edge]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['fluxunit'] = fluxunit
        mytmp['telescopeparam'] = telescopeparam
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['restfreq'] = restfreq
        mytmp['frame'] = frame
        mytmp['doppler'] = doppler
        mytmp['timerange'] = timerange
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['calmode'] = calmode
        mytmp['fraction'] = fraction
        mytmp['noff'] = noff
        mytmp['width'] = width
        mytmp['elongated'] = elongated
        mytmp['markonly'] = markonly
        mytmp['plotpointings'] = plotpointings
        mytmp['tau'] = tau
        mytmp['average'] = average
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['scanaverage'] = scanaverage
        mytmp['averageall'] = averageall
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['maskmode'] = maskmode
        mytmp['thresh'] = thresh
        mytmp['avg_limit'] = avg_limit
        mytmp['edge'] = edge
        mytmp['blfunc'] = blfunc
        mytmp['order'] = order
        mytmp['npiece'] = npiece
        mytmp['applyfft'] = applyfft
        mytmp['fftmethod'] = fftmethod
        mytmp['fftthresh'] = fftthresh
        mytmp['addwn'] = addwn
        mytmp['rejwn'] = rejwn
        mytmp['clipthresh'] = clipthresh
        mytmp['clipniter'] = clipniter
        mytmp['verifycal'] = verifycal
        mytmp['verifysm'] = verifysm
        mytmp['verifybl'] = verifybl
        mytmp['verbosebl'] = verbosebl
        mytmp['bloutput'] = bloutput
        mytmp['blformat'] = blformat
        mytmp['showprogress'] = showprogress
        mytmp['minnrow'] = minnrow
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdreduce.xml')

        casalog.origin('sdreduce')
        if not trec.has_key('sdreduce') or not casac.utils().verify(mytmp, trec['sdreduce']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdreduce           #####')
          casalog.post('')
          result = sdreduce(infile, antenna, fluxunit, telescopeparam, field, spw, restfreq, frame, doppler, timerange, scan, pol, calmode, fraction, noff, width, elongated, markonly, plotpointings, tau, average, timeaverage, tweight, scanaverage, averageall, polaverage, pweight, kernel, kwidth, chanwidth, maskmode, thresh, avg_limit, edge, blfunc, order, npiece, applyfft, fftmethod, fftthresh, addwn, rejwn, clipthresh, clipniter, verifycal, verifysm, verifybl, verbosebl, bloutput, blformat, showprogress, minnrow, outfile, outform, overwrite, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdreduce           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdreduce', 'sdreduce.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdreduce'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdreduce', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['calmode']  = 'none'
        a['tau']  = 0.0
        a['average']  = False
        a['kernel']  = 'none'
        a['maskmode']  = 'auto'
        a['blfunc']  = 'none'
        a['verifycal']  = False
        a['verifysm']  = False
        a['verifybl']  = False
        a['verbosebl']  = True
        a['bloutput']  = True
        a['blformat']  = ''
        a['showprogress']  = True
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        a['fluxunit'] = {
                    0:{'value':''}, 
                    1:odict([{'value':'K'}, {'telescopeparam':""}]), 
                    2:odict([{'value':'k'}, {'telescopeparam':""}]), 
                    3:odict([{'value':'Jy'}, {'telescopeparam':""}]), 
                    4:odict([{'value':'jy'}, {'telescopeparam':""}])}
        a['spw'] = {
                    0:odict([{'notvalue':''}, {'restfreq':''}, {'frame':''}, {'doppler':''}])}
        a['average'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'timeaverage':False}, {'tweight':'tintsys'}, {'scanaverage':False}, {'averageall':False}, {'polaverage':False}, {'pweight':'tsys'}])}
        a['kernel'] = {
                    0:{'value':'none'}, 
                    1:{'value':''}, 
                    2:{'value':'hanning'}, 
                    3:odict([{'value':'gaussian'}, {'kwidth':5}]), 
                    4:odict([{'value':'boxcar'}, {'kwidth':5}]), 
                    5:odict([{'value':'regrid'}, {'chanwidth':'5'}])}
        a['maskmode'] = {
                    0:odict([{'value':'auto'}, {'thresh':5.0}, {'avg_limit':4}, {'edge':[0]}]), 
                    1:{'value':'list'}, 
                    2:{'value':'interact'}}
        a['blfunc'] = {
                    0:{'value':'none'}, 
                    1:odict([{'value':'poly'}, {'order':5}]), 
                    2:odict([{'value':'chebyshev'}, {'order':5}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    3:odict([{'value':'cspline'}, {'npiece':2}, {'clipthresh':3.0}, {'clipniter':0}]), 
                    4:odict([{'value':'sinusoid'}, {'applyfft':True}, {'fftmethod':'fft'}, {'fftthresh':3.0}, {'addwn':[0]}, {'rejwn':[]}, {'clipthresh':3.0}, {'clipniter':0}])}
        a['showprogress'] = {
                    0:odict([{'value':True}, {'minnrow':1000}])}
        a['calmode'] = {
                    0:{'value':'none'}, 
                    1:{'value':'ps'}, 
                    2:{'value':'nod'}, 
                    3:{'value':'fs'}, 
                    4:{'value':'fsotf'}, 
                    5:{'value':'quotient'}, 
                    6:odict([{'value':'otf'}, {'fraction':'10%'}, {'width':0.5}, {'elongated':False}, {'markonly':False}, {'plotpointings':False}]), 
                    7:odict([{'value':'otfraster'}, {'fraction':'10%'}, {'noff':-1}, {'markonly':False}, {'plotpointings':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdreduce', subkey=None):
        desc={'sdreduce': 'ASAP SD task: do sdcal, sdaverage, and sdbaseline in one task',
               'infile': 'name of input SD dataset',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'fluxunit': 'units of the flux (\'\'=current)',
               'telescopeparam': 'parameters of telescope for flux conversion (see description in help of sdcal)',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'restfreq': 'the rest frequency, e.g. \'1.41GHz\' (default unit: Hz) (see examples in help)',
               'frame': 'frequency reference frame (\'\'=current)',
               'doppler': 'doppler convention (\'\'=current). Effective only when spw selection is in velocity unit.',
               'timerange': 'select data by time range, e.g. \'09:14:0~09:54:0\' (\'\'=all) (see examples in help of sdcal)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'calmode': 'SD calibration mode (\'none\' = skip calibration)',
               'fraction': 'fraction of the OFF data to mark as OFF spectra, e.g., \'10%\'',
               'noff': 'number of the OFF data to mark (-1 = use fraction instead of number)',
               'width': 'width of the pixel for edge detection',
               'elongated': 'the observed area is elongated in one direction',
               'markonly': 'do calibration (False) or just mark OFF (True)',
               'plotpointings': 'plot pointing direction for ON and OFF',
               'tau': 'the zenith atmospheric optical depth for correction (0. = no correction)',
               'average': 'data averaging [True, False] ',
               'timeaverage': 'average spectra over time [True, False] (see examples in help of sdaverage)',
               'tweight': 'weighting for time averaging',
               'scanaverage': 'average spectra within a scan number [True, False] (see examples in help of sdaverage)',
               'averageall': 'set True only when averaging spectra with different spectral resolutions',
               'polaverage': 'average spectra over polarizations [True, False]',
               'pweight': 'weighting for polarization averaging',
               'kernel': 'type of spectral smoothing kernel  (\'none\'=no smoothing)',
               'kwidth': 'width of smoothing kernel in channels',
               'chanwidth': 'width of regridded channels',
               'maskmode': 'mode of setting additional channel masks',
               'thresh': 'S/N threshold for linefinder',
               'avg_limit': 'channel averaging for broad lines',
               'edge': 'channels to drop at beginning and end of spectrum',
               'blfunc': 'baseline model function  (\'none\' = skip baseline fit)',
               'order': 'order of baseline model function',
               'npiece': 'number of element polynomials for cubic spline curve',
               'applyfft': 'automatically set wave numbers of sinusoids [True, False]',
               'fftmethod': 'method for automatically set wave numbers of sinusoids',
               'fftthresh': 'threshold to select wave numbers of sinusoids',
               'addwn': 'additional wave numbers to use',
               'rejwn': 'wave numbers NOT to use',
               'clipthresh': 'clipping threshold for iterative fitting',
               'clipniter': 'maximum iteration number for iterative fitting',
               'verifycal': 'interactively verify the results of calibration [True, False] (see description in sdcal)',
               'verifysm': 'interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]',
               'verifybl': 'interactively verify the results of baseline fitting for each spectrum (only for blfunc="poly". see description in help)',
               'verbosebl': 'output baseline fitting results to logger [True, False]',
               'bloutput': 'output baseline fitting results to a text file [True, False]',
               'blformat': 'format of the text file specified with bloutput',
               'showprogress': 'show progress status for large data [True, False]',
               'minnrow': 'minimum number of input spectra to show progress status in baseline fitting',
               'outfile': 'name of output file (See a WARNING in help)',
               'outform': 'output file format (See a WARNING in help)',
               'overwrite': 'overwrite the output file if already exists  [True, False]',
               'plotlevel': 'plot and summarize results (0=none). See description in each task',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['fluxunit']  = ''
        a['telescopeparam']  = ''
        a['field']  = ''
        a['spw']  = ''
        a['restfreq']  = ''
        a['frame']  = ''
        a['doppler']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['calmode']  = 'none'
        a['fraction']  = '10%'
        a['noff']  = -1
        a['width']  = 0.5
        a['elongated']  = False
        a['markonly']  = False
        a['plotpointings']  = False
        a['tau']  = 0.0
        a['average']  = False
        a['timeaverage']  = False
        a['tweight']  = 'tintsys'
        a['scanaverage']  = False
        a['averageall']  = False
        a['polaverage']  = False
        a['pweight']  = 'tsys'
        a['kernel']  = 'none'
        a['kwidth']  = 5
        a['chanwidth']  = '5'
        a['maskmode']  = 'auto'
        a['thresh']  = 5.0
        a['avg_limit']  = 4
        a['edge']  = [0]
        a['blfunc']  = 'none'
        a['order']  = 5
        a['npiece']  = 2
        a['applyfft']  = True
        a['fftmethod']  = 'fft'
        a['fftthresh']  = 3.0
        a['addwn']  = []
        a['rejwn']  = []
        a['clipthresh']  = 3.0
        a['clipniter']  = 0
        a['verifycal']  = False
        a['verifysm']  = False
        a['verifybl']  = False
        a['verbosebl']  = True
        a['bloutput']  = True
        a['blformat']  = ''
        a['showprogress']  = True
        a['minnrow']  = 1000
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdreduce_pg = sdreduce_pg_()
