#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdsave import sdsave
class sdsave_cli_:
    __name__ = "sdsave"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdsave_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'splitant':None, 'antenna':None, 'getpt':None, 'field':None, 'spw':None, 'timerange':None, 'scan':None, 'pol':None, 'beam':None, 'restfreq':None, 'outfile':None, 'outform':None, 'fillweight':None, 'overwrite':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, splitant=None, antenna=None, getpt=None, field=None, spw=None, timerange=None, scan=None, pol=None, beam=None, restfreq=None, outfile=None, outform=None, fillweight=None, overwrite=None, ):

        """Save the sd spectra in various format

	Detailed Description: 

Task sdsave writes the single dish data to a disk file in 
specified format (ASAP, MS2, SDFITS, ASCII). It is possible to 
save the subset of the data by selecting field names, spw ids,
time ranges, scan numbers, and polarization ids. The ASAP
(scantable) format is recommended for further analysis using Sd
tool or tasks except imaging. For further imaging using imager
or sdimaging/sdtpimaging, save the data to the Measurement Set
(MS2).
  
	Arguments :
		infile:	name of input SD dataset
		   Default Value: 

		splitant:	split output file by antenna (only effective for MS input)
		   Default Value: False

		antenna:	select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)
		   Default Value: 0

		getpt:	fill DIRECTION column properly (True), or reuse POINTING table in original MS (False) (only effective for MS input)
		   Default Value: True

		field:	select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)
		   Default Value: 

		spw:	select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)
		   Default Value: 

		timerange:	select data by time range, e.g. \'09:14:0~09:54:0\' (\'\'=all) (see examples in help)
		   Default Value: 

		scan:	select data by scan numbers, e.g. \'21~23\' (\'\'=all)
		   Default Value: 

		pol:	select data by polarization IDs, e.g. \'0,1\' (\'\'=all)
		   Default Value: 

		beam:	select data by beam IDs, e.g. \'0,1\' (\'\'=all)
		   Default Value: 

		restfreq:	the rest frequency, e.g. \'1.41GHz\' (default unit: Hz) (see examples in help)
		   Default Value: 

		outfile:	name of output file (See a WARNING in help)
		   Default Value: 

		outform:	output file format (See a WARNING in help)
		   Default Value: ASAP
		   Allowed Values:
				ASCII
				MS2
				SDFITS
				ASAP

		fillweight:	fill the WEIGHT and SIGMA columns for output MS
		   Default Value: False

		overwrite:	overwrite the output file if already exists
		   Default Value: False

	Returns: void

	Example :

-----------------
Keyword arguments
-----------------
infile -- name of input SD dataset
splitant -- split output file by antenna. this parameter is only
            effective for MS input.
        options: (bool) True, False
        default: False
    >>>splitant expandable parameter
        antenna -- select an antenna name or ID. this parameter is
                   effective only for MS input.
                default: 0
                example: antenna=0 specifies antenna by id
                         antenna='PM03' specifies antenna by name
getpt -- fill DIRECTION column properly (True), or reuse POINTING
         table in original MS (False). this parameter is only
         effective for MS input.
    options: (bool) True, False
    default: True
field -- select data by field IDs and names
        default: '' (use all fields)
        example: field='3C2*' (all names starting with 3C2)
                 field='0,4,5~7' (field IDs 0,4,5,6,7)
                 field='0,3C273' (field ID 0 or field named 3C273)
        this selection is in addition to the other selections to data
spw -- select data by IF IDs (spectral windows)/channels
        default: '' (use all IFs and channels)
        example: spw='3,5,7' (IF IDs 3,5,7; all channels)
                 spw='<2' (IF IDs less than 2, i.e., 0,1; all channels)
                 spw='30~45GHz' (IF IDs with the center frequencies in range 30-45GHz; all channels)
                 spw='0:5~61' (IF ID 0; channels 5 to 61; all channels)
                 spw='3:10~20;50~60' (select multiple channel ranges within IF ID 3)
                 spw='3:10~20,4:0~30' (select different channel ranges for IF IDs 3 and 4)
                 spw='1~4;6:15~48' (for channels 15 through 48 for IF IDs 1,2,3,4 and 6)
        this selection is in addition to the other selections to data
timerange -- select data by time range
        default: '' (use all)
        example: timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
                 Note: YYYY/MM/DD can be dropped as needed:
                 timerange='09:14:00~09:54:00' # this time range
                 timerange='09:44:00' # data within one integration of time
                 timerange='>10:24:00' # data after this time
                 timerange='09:44:00+00:13:00' #data 13 minutes after time
        this selection is in addition to the other selections to data
scan -- select data by scan numbers
        default: '' (use all scans)
        example: scan='21~23' (scan IDs 21,22,23)
        this selection is in addition to the other selections to data
pol -- select data by polarization IDs
        default: '' (use all polarizations)
        example: pol='0,1' (polarization IDs 0,1)
        this selection is in addition to the other selections to data
beam -- select data by beam IDs
        default: '' (use all beams)
        example: beam='0,1' (beam IDs 0,1)
        this selection is in addition to the other selections to data
restfreq -- the rest frequency
            available type includes float, int, string, list of float, 
            list of int, list of string, and list of dictionary. the 
            default unit of restfreq in case of float, int, or string 
            without unit is Hz. string input can be a value only 
            (treated as Hz) or a value followed by unit for which 'GHz',
            'MHz','kHz',and 'Hz' are available. 
            a list can be used to set different rest frequencies for 
            each IF. the length of list input must be number of IFs. 
            dictionary input should be a pair of line name and 
            frequency with keys of 'name' and 'value', respectively. 
            values in the dictionary input follows the same manner as 
            as for single float or string input. 
        example: 345.796
                 '1420MHz'
                 [345.8, 347.0, 356.7]
                 ['345.8MHz', '347.0MHz', '356.7MHz']
                 [{'name':'CO','value':345}]
outfile -- name of output file
        default: '' ((<infile>_saved)
        NOTE actual output file name(s) will be modified if splitant 
        is True as antenna names are to be included. If outfile has a 
        suffix '.asap' or '.ASAP', antenna name will be inserted before 
        the suffix like 'out.antName.asap', otherwise, antenna name 
        will be simply appended to outfile like 'out.sdfits.antName'. 
outform -- output file format
        options: 'ASAP','MS2', 'ASCII','SDFITS'
        default: 'ASAP'
        NOTE the ASAP format is easiest for further sd
        processing; use MS2 for CASA imaging.
        If ASCII, then will append some stuff to
        the outfile name
fillweight -- fill WEIGHT and SIGMA column for output MS
        default: True
        options: True, False

-----------
DESCRIPTION
-----------
Task sdsave writes the single dish data to a disk file in 
specified format (ASAP, MS2, SDFITS, ASCII). It is possible to 
save the subset of the data by selecting field names, spw ids,
time ranges, scan numbers, and polarization ids. The ASAP
(scantable) format is recommended for further analysis using Sd
tool or tasks except imaging. For further imaging using imager
or sdimaging/sdtpimaging, save the data to the Measurement Set
(MS2).

Note that setting getpt=False needs a lot of attention.
If you set getpt=False, the task retrieves pointing direction from 
MS's FIELD table, which might not be correct for single dish 
observation, instead to check MS's POINTING table, which is the 
default behavior of the task (getpt=True). To compensate this, 
absolute path to MS's POINTING table is stored, and it will be used 
for POINTING table when the data is converted back to MS format. 
In general, getpt=False is faster especially for large data. However, 
MS created from Scantable cannot have correct POINTING table if 
original MS's POINTING table doesn't exist. Such situation will 
happen when original MS is removed or renamed, or imported Scantable 
is moved to other computer alone.

-------
WARNING
-------
For the GBT raw SDFITS format data as input:
SDtasks are able to handle GBT raw SDFITS format data since the data
filler is available. However, the functionality is not well tested yet,
so that there may be unknown bugs. 
  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdsave'
        self.__globals__['taskname'] = 'sdsave'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['splitant'] = splitant = self.parameters['splitant']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['getpt'] = getpt = self.parameters['getpt']
            myparams['field'] = field = self.parameters['field']
            myparams['spw'] = spw = self.parameters['spw']
            myparams['timerange'] = timerange = self.parameters['timerange']
            myparams['scan'] = scan = self.parameters['scan']
            myparams['pol'] = pol = self.parameters['pol']
            myparams['beam'] = beam = self.parameters['beam']
            myparams['restfreq'] = restfreq = self.parameters['restfreq']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['outform'] = outform = self.parameters['outform']
            myparams['fillweight'] = fillweight = self.parameters['fillweight']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']


	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['splitant'] = splitant
        mytmp['antenna'] = antenna
        mytmp['getpt'] = getpt
        mytmp['field'] = field
        mytmp['spw'] = spw
        mytmp['timerange'] = timerange
        mytmp['scan'] = scan
        mytmp['pol'] = pol
        mytmp['beam'] = beam
        mytmp['restfreq'] = restfreq
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['fillweight'] = fillweight
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdsave.xml')

        casalog.origin('sdsave')
	try :
          #if not trec.has_key('sdsave') or not casac.casac.utils().verify(mytmp, trec['sdsave']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdsave'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdsave', 'sdsave.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdsave'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdsave(infile, splitant, antenna, getpt, field, spw, timerange, scan, pol, beam, restfreq, outfile, outform, fillweight, overwrite)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdsave'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdsave', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['splitant']  = False
        a['getpt']  = True
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['beam']  = ''
        a['restfreq']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        a['splitant'] = {
                    0:odict([{'value':False}, {'antenna':0}]), 
                    1:{'value':True}}
        a['outform'] = {
                    0:{'value':'ASAP'}, 
                    1:odict([{'value':'MS2'}, {'fillweight':False}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdsave', subkey=None):
        desc={'sdsave': 'Save the sd spectra in various format',
               'infile': 'name of input SD dataset',
               'splitant': 'split output file by antenna [True, False]',
               'antenna': 'select an antenna name or ID, e.g. \'PM03\' (only effective for MS input)',
               'getpt': 'fill DIRECTION column properly [True, False]',
               'field': 'select data by field IDs and names, e.g. \'3C2*\' (\'\'=all)',
               'spw': 'select data by IF IDs (spectral windows), e.g. \'3,5,7\' (\'\'=all)',
               'timerange': 'select data by time range, e.g. \'09:14:0~09:54:0\' (\'\'=all) (see examples in help)',
               'scan': 'select data by scan numbers, e.g. \'21~23\' (\'\'=all)',
               'pol': 'select data by polarization IDs, e.g. \'0,1\' (\'\'=all)',
               'beam': 'select data by beam IDs, e.g. \'0,1\' (\'\'=all)',
               'restfreq': 'the rest frequency, e.g. \'1.41GHz\' (default unit: Hz) (see examples in help)',
               'outfile': 'name of output file (See a WARNING in help)',
               'outform': 'output file format [\'ASAP\', \'MS2\', \'ASCII\', or \'SDFITS\']',
               'fillweight': 'fill the WEIGHT and SIGMA columns for output MS [True, False]',
               'overwrite': 'overwrite the output file if already exists [True, False] ',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['splitant']  = False
        a['antenna']  = 0
        a['getpt']  = True
        a['field']  = ''
        a['spw']  = ''
        a['timerange']  = ''
        a['scan']  = ''
        a['pol']  = ''
        a['beam']  = ''
        a['restfreq']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['fillweight']  = False
        a['overwrite']  = False

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['splitant']  == False:
            a['antenna'] = 0

        if self.parameters['outform']  == 'MS2':
            a['fillweight'] = False

        if a.has_key(paramname) :
	      return a[paramname]
sdsave_cli = sdsave_cli_()
