#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdsaveold import sdsaveold
from task_sdsaveold import casalog

class sdsaveold_pg_:
    __name__ = "sdsaveold"

    def __init__(self) :
       self.__bases__ = (sdsaveold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,splitant=None,antenna=None,getpt=None,rowlist=None,scanlist=None,field=None,iflist=None,pollist=None,scanaverage=None,timeaverage=None,tweight=None,polaverage=None,pweight=None,restfreq=None,outfile=None,outform=None,overwrite=None,):

        """ASAP SD task: save the sd spectra in various format
  Keyword arguments:
        infile -- name of input SD dataset
        splitant -- split output file by antenna (True,False) 
                (only effective for MS input).
                default: False
                >>>splitant expandable parameter
                 antenna -- antenna name or id (only effective for MS input). 
                         default: 0
        getpt -- fill DIRECTION column properly, or reuse POINTING table 
                in original MS (only effective for MS input)
                default: True
                options: True (fill DIRECTION column properly)
                         False (reuse POINTING table in original MS)
        rowlist -- list of row numbers to process
                default: [] (use all rows)
                example: [0,2,4,6]
                For expert users only!
                this selection is applied first, and then followed by
                the selection with scans, fields, ifs, and polarizations. 
        scanlist -- list of to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist, and pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
        scanaverage --  average integrations within scans
                options: (bool) True,False
                default: False
                example: if True, average integrations before it is saved
        timeaverage -- average times for multiple scan cycles
                options: (bool) True,False
                default: False
                >>>timeaverage expandable parameter
                 tweight -- weighting for time average
                         options: 'none'
                                  'var'   (1/var(spec) weighted)
                                  'tsys'  (1/Tsys**2 weighted)
                                  'tint'  (integration time weighted)
                                  'tintsys'  (Tint/Tsys**2)
                                  'median'  ( median averaging)
                         default: 'none'

        polaverage -- average polarizations
                options: (bool) True,False
                default: False
                >>>polaverage expandable parameter
                 pweight -- weighting for polarization average
                         options: 'none'
                                  'var'  (1/var(spec) weighted)
                                  'tsys' (1/Tsys**2 weighted)
        restfreq -- rest frequencies of output data
                Available types are a number, string, a list of numbers or
                strings (see examples below), and list of dictionaries. 
                The default unit of restfreq is Hz, if not specified.
                A list can be used to set different rest frequencies to
                each IF. the length of list input must be nIF. Dictionary 
                input should be a pair of molecule name and frequency with 
                keys of 'name' and 'value', respectively. The 'value's in the 
                dictionary input follows the same manner as for single 
                float or string input. 
                default: '' (use current setting)
                example: 4.6e10 (float value in Hz),
                         '46GHz' (string with unit),
                         ['345.8GHz', '347.0GHz', 356.7e9] (for each IF)
                         [{'name':'CO','value':345e9}] (a value with name)
        outfile -- name of output dataset
                default: '' 
                NOTE actual output file name(s) will be modified if splitant 
                is True as antenna names are to be included. If outfile has a 
                suffix '.asap' or '.ASAP', antenna name will be inserted before 
                the suffix like 'out.antName.asap', otherwise, antenna name 
                will be simply appended to outfile like 'out.sdfits.antName'. 
        outform -- output data format
                default: 'ASAP'
                Options: 'ASAP', 'MS2', 'SDFITS', 'ASCII'
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored


        DESCRIPTION:

        Task sdsaveold writes the single dish data to a disk file in 
        specified format (ASAP, MS2, SDFITS, ASCII). It is possible to 
        save the subset of the data by selecting row numbers, scan numbers, 
        IF ids, polarization ids and field names. The ASAP (scantable) format 
        is recommended for further analysis using Sd tool. For further 
        imaging using imager, save the data to the Measurement Set (MS2).

        Note that setting getpt=False needs a lot of attention.
        If you set getpt=False, the task retrieves pointing direction from 
        MS's FIELD table, which might not be correct for single dish 
        observation, instead to check MS's POINTING table, which is the 
        default behavior of the task (getpt=True). To compensate this, 
        absolute path to MS's POINTING table is stored, and it will be used 
        for POINTING table when the data is converted back to MS format. 
        In general, getpt=False is faster especially for large data. However, 
        MS created from Scantable cannot have correct POINTING table if 
        original MS's POINTING table doesn't exist. Such situation will 
        happen when original MS is removed or renamed, or imported Scantable 
        is moved to other computer alone.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdsaveold'
        myf['taskname'] = 'sdsaveold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['splitant'] = splitant = myf['splitant']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['getpt'] = getpt = myf['getpt']
            myparams['rowlist'] = rowlist = myf['rowlist']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['scanaverage'] = scanaverage = myf['scanaverage']
            myparams['timeaverage'] = timeaverage = myf['timeaverage']
            myparams['tweight'] = tweight = myf['tweight']
            myparams['polaverage'] = polaverage = myf['polaverage']
            myparams['pweight'] = pweight = myf['pweight']
            myparams['restfreq'] = restfreq = myf['restfreq']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']

        if type(rowlist)==int: rowlist=[rowlist]
        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['splitant'] = splitant
        mytmp['antenna'] = antenna
        mytmp['getpt'] = getpt
        mytmp['rowlist'] = rowlist
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['scanaverage'] = scanaverage
        mytmp['timeaverage'] = timeaverage
        mytmp['tweight'] = tweight
        mytmp['polaverage'] = polaverage
        mytmp['pweight'] = pweight
        mytmp['restfreq'] = restfreq
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdsaveold.xml')

        casalog.origin('sdsaveold')
        if not trec.has_key('sdsaveold') or not casac.utils().verify(mytmp, trec['sdsaveold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdsaveold           #####')
          casalog.post('')
          result = sdsaveold(infile, splitant, antenna, getpt, rowlist, scanlist, field, iflist, pollist, scanaverage, timeaverage, tweight, polaverage, pweight, restfreq, outfile, outform, overwrite)
          casalog.post('')
          casalog.post('##### End Task: sdsaveold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdsaveold', 'sdsaveold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdsaveold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdsaveold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['splitant']  = False
        a['getpt']  = True
        a['rowlist']  = []
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['scanaverage']  = False
        a['timeaverage']  = False
        a['polaverage']  = False
        a['restfreq']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        a['splitant'] = {
                    0:odict([{'value':False}, {'antenna':0}]), 
                    1:{'value':True}}
        a['timeaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'tweight':'none'}])}
        a['polaverage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'pweight':'none'}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdsaveold', subkey=None):
        desc={'sdsaveold': 'ASAP SD task: save the sd spectra in various format',
               'infile': 'name of input SD dataset',
               'splitant': 'split output file by antenna (True,False) (only effective for MS input)',
               'antenna': 'antenna name or id (only effective for MS input)',
               'getpt': 'fill DIRECTION column properly, or reuse POINTING table in original MS (only effective for MS input)',
               'rowlist': 'list of row numbers to process (e.g. [0,2,4,6])',
               'scanlist': 'list of scan to process (e.g. [20,21,22,23])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'scanaverage': 'average integs within scans (True,False) ',
               'timeaverage': 'average scans over time (True,False)',
               'tweight': 'weighting for time averaging',
               'polaverage': 'average over polarizations (True,False)',
               'pweight': '',
               'restfreq': 'rest frequency (default unit: Hz)',
               'outfile': 'output file name',
               'outform': 'output file format (ASCII,MS2,SDFITS,ASAP)',
               'overwrite': 'overwrite the output file if already exists',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['splitant']  = False
        a['antenna']  = 0
        a['getpt']  = True
        a['rowlist']  = []
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['scanaverage']  = False
        a['timeaverage']  = False
        a['tweight']  = 'none'
        a['polaverage']  = False
        a['pweight']  = 'none'
        a['restfreq']  = ''
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False

        if a.has_key(paramname) :
	      return a[paramname]
sdsaveold_pg = sdsaveold_pg_()
