#
# This file was generated using xslt from its XML file
#
# Copyright 2009, Associated Universities Inc., Washington DC
#
import sys
import os
from  casac import *
import string
from taskinit import casalog
#from taskmanager import tm
import task_sdsmoothold
def sdsmoothold(infile='', antenna=0, scanaverage=False, scanlist=[], field='', iflist=[], pollist=[], kernel='hanning', kwidth=5, chanwidth='5', verify=False, outfile='', outform='ASAP', overwrite=False, plotlevel=0):

        """ASAP SD task: do smoothing of spectra

	Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        scanaverage -- average integrations within scans
                options: (bool) True,False
                default: False
                example: if True, this happens in read-in
                         For GBT, set False!
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist,pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
	kernel -- type of spectral smoothing
	        options: 'hanning','gaussian','boxcar','regrid'
	        default: 'hanning'

            >>>kernel expandable parameter
                 kwidth -- width of spectral smoothing kernel
	                 options: (int) in channels 
	                 default: 5
		         example: 5 or 10 seem to be popular for boxcar
		                  ignored for hanning (fixed at 5 chans)
                                  (0 will turn off gaussian or boxcar)
                 chanwidth -- channel width of regridded spectra
	                 default: '5' (in channels)
		         example: '500MHz', '0.2km/s'

        verify -- interactively verify the results of smoothing for each
                  spectrum.
                  When verify = True, for each input spectrum, spectra
                  before and after the smoothing are displayed in a plot
                  window. At the prompt there are four choices of action:
                  'Y' (accept the smoothing and continue to the next input
                  spectrum), 'N' (reject the smoothing and continue to the
                  next input spectrum), 'A' (accept the current smoothing
                  and continue non-interactively), and 'R' (reject the
                  current smoothing and exit from smoothing).
                  Note that when the smoothing is rejected by 'N' or 'R',
                  no smoothing is done to the spectrum/spectra. 
                options: (bool) True,False
                default: False
                Note: verification is not yet available for kernel='regrid'
	outfile -- Name of output ASAP format(scantable) file
	        default: '' (<infile>_sm)
	outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                         If ASCII, then will append some stuff to
                         the outfile name
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored
        plotlevel -- control for plotting of results
	        options: (int) 0=none, 1=some, 2=more, <0=hardcopy
		default: 0 (no plotting)
		example: plotlevel<0 as abs(plotlevel), e.g.
		         -1 => hardcopy of final plot (will be named
		         <outfile>_smspec.eps)


        DESCRIPTION:

	Task sdsmoothold performs smoothing of the single-dish spectra.
        Set plotlevel >= 1 to plot spectrum before and after smoothing.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['scanaverage'] = scanaverage
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['verify'] = verify
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdsmoothold.xml')

        casalog.origin('sdsmoothold')
        if trec.has_key('sdsmoothold') and casac.utils().verify(mytmp, trec['sdsmoothold']) :
	    result = task_sdsmoothold.sdsmoothold(infile, antenna, scanaverage, scanlist, field, iflist, pollist, kernel, kwidth, chanwidth, verify, outfile, outform, overwrite, plotlevel)

	else :
	  result = False
        return result
