#
# This file was generated using xslt from its XML file
#
# Copyright 2014, Associated Universities Inc., Washington DC
#
import sys
import os
#from casac import *
import casac
import string
import time
import inspect
import gc
import numpy
from odict import odict
from types import * 
from task_sdsmoothold import sdsmoothold
class sdsmoothold_cli_:
    __name__ = "sdsmoothold"
    rkey = None
    i_am_a_casapy_task = None
    # The existence of the i_am_a_casapy_task attribute allows help()
    # (and other) to treat casapy tasks as a special case.

    def __init__(self) :
       self.__bases__ = (sdsmoothold_cli_,)
       self.__doc__ = self.__call__.__doc__

       self.parameters={'infile':None, 'antenna':None, 'scanaverage':None, 'scanlist':None, 'field':None, 'iflist':None, 'pollist':None, 'kernel':None, 'kwidth':None, 'chanwidth':None, 'verify':None, 'outfile':None, 'outform':None, 'overwrite':None, 'plotlevel':None, }


    def result(self, key=None):
	    #### and add any that have completed...
	    return None


    def __call__(self, infile=None, antenna=None, scanaverage=None, scanlist=None, field=None, iflist=None, pollist=None, kernel=None, kwidth=None, chanwidth=None, verify=None, outfile=None, outform=None, overwrite=None, plotlevel=None, ):

        """ASAP SD task: do smoothing of spectra

	Detailed Description: 
ASAP SD calibration task (based on the ALMATST5 recommendations):
        Do smoothing of spectra

        Task Version 2008-01-27 TT
        split and modified from Task Version 2007-03-04 STM

	Arguments :
		infile:	name of input SD dataset
		   Default Value: 

		antenna:	antenna name or id (only effective for MS input)
		   Default Value: 0

		scanaverage:	average integs within scans (True,False) 
		   Default Value: False

		scanlist:	list of scans to use (e.g. [1,2,3,4])
		   Default Value: 

		field:	string for selection by source name
		   Default Value: 

		iflist:	list of IF ids to select (e.g. [0,1])
		   Default Value: 

		pollist:	list of polarization ids to select (e.g. [0,1])
		   Default Value: 

		kernel:	type of spectral smoothing
		   Default Value: hanning
		   Allowed Values:
				hanning
				gaussian
				boxcar
				regrid

		kwidth:	width of spectral kernel in channels
		   Default Value: 5

		chanwidth:	width of regridded channels
		   Default Value: 5

		verify:	interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]
		   Default Value: False

		outfile:	output file name
		   Default Value: 

		outform:	output file format (ASCII,MS,SDFITS,ASAP)
		   Default Value: ASAP
		   Allowed Values:
				ASCII
				ascii
				MS
				ms
				MS2
				ms2
				SDFITS
				sdfits
				ASAP
				asap

		overwrite:	overwrite the output file if already exists
		   Default Value: False

		plotlevel:	plot results (0=none,1+=some,&lt;0=hardcopy)
		   Default Value: 0

	Returns: void

	Example :


	Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        scanaverage -- average integrations within scans
                options: (bool) True,False
                default: False
                example: if True, this happens in read-in
                         For GBT, set False!
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist,pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
	kernel -- type of spectral smoothing
	        options: 'hanning','gaussian','boxcar','regrid'
	        default: 'hanning'

            >>>kernel expandable parameter
                 kwidth -- width of spectral smoothing kernel
	                 options: (int) in channels 
	                 default: 5
		         example: 5 or 10 seem to be popular for boxcar
		                  ignored for hanning (fixed at 5 chans)
                                  (0 will turn off gaussian or boxcar)
                 chanwidth -- channel width of regridded spectra
	                 default: '5' (in channels)
		         example: '500MHz', '0.2km/s'

        verify -- interactively verify the results of smoothing for each
                  spectrum.
                  When verify = True, for each input spectrum, spectra
                  before and after the smoothing are displayed in a plot
                  window. At the prompt there are four choices of action:
                  'Y' (accept the smoothing and continue to the next input
                  spectrum), 'N' (reject the smoothing and continue to the
                  next input spectrum), 'A' (accept the current smoothing
                  and continue non-interactively), and 'R' (reject the
                  current smoothing and exit from smoothing).
                  Note that when the smoothing is rejected by 'N' or 'R',
                  no smoothing is done to the spectrum/spectra. 
                options: (bool) True,False
                default: False
                Note: verification is not yet available for kernel='regrid'
	outfile -- Name of output ASAP format(scantable) file
	        default: '' (<infile>_sm)
	outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                         If ASCII, then will append some stuff to
                         the outfile name
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored
        plotlevel -- control for plotting of results
	        options: (int) 0=none, 1=some, 2=more, <0=hardcopy
		default: 0 (no plotting)
		example: plotlevel<0 as abs(plotlevel), e.g.
		         -1 => hardcopy of final plot (will be named
		         <outfile>_smspec.eps)


        DESCRIPTION:

	Task sdsmoothold performs smoothing of the single-dish spectra.
        Set plotlevel >= 1 to plot spectrum before and after smoothing.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
	#casac = self.__globals__['casac']
	casalog = self.__globals__['casalog']
	#casalog = casac.casac.logsink()
        self.__globals__['__last_task'] = 'sdsmoothold'
        self.__globals__['taskname'] = 'sdsmoothold'
        ###
        self.__globals__['update_params'](func=self.__globals__['taskname'],printtext=False,ipython_globals=self.__globals__)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
        if type(self.__call__.func_defaults) is NoneType:
            function_signature_defaults={}
	else:
	    function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames[1:],self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       self.parameters[key] = keyVal
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
                      if len(keyVal) > 0 :
		         exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')
		      else :
		         exec('myparams[key] = ' + key + ' = {}')
	 
        else :
            print ''

            myparams['infile'] = infile = self.parameters['infile']
            myparams['antenna'] = antenna = self.parameters['antenna']
            myparams['scanaverage'] = scanaverage = self.parameters['scanaverage']
            myparams['scanlist'] = scanlist = self.parameters['scanlist']
            myparams['field'] = field = self.parameters['field']
            myparams['iflist'] = iflist = self.parameters['iflist']
            myparams['pollist'] = pollist = self.parameters['pollist']
            myparams['kernel'] = kernel = self.parameters['kernel']
            myparams['kwidth'] = kwidth = self.parameters['kwidth']
            myparams['chanwidth'] = chanwidth = self.parameters['chanwidth']
            myparams['verify'] = verify = self.parameters['verify']
            myparams['outfile'] = outfile = self.parameters['outfile']
            myparams['outform'] = outform = self.parameters['outform']
            myparams['overwrite'] = overwrite = self.parameters['overwrite']
            myparams['plotlevel'] = plotlevel = self.parameters['plotlevel']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['scanaverage'] = scanaverage
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['verify'] = verify
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.casac.utils().torecord(pathname+'sdsmoothold.xml')

        casalog.origin('sdsmoothold')
	try :
          #if not trec.has_key('sdsmoothold') or not casac.casac.utils().verify(mytmp, trec['sdsmoothold']) :
	    #return False

          casac.casac.utils().verify(mytmp, trec['sdsmoothold'], True)
          scriptstr=['']
          saveinputs = self.__globals__['saveinputs']
          if type(self.__call__.func_defaults) is NoneType:
              saveinputs=''
          else:
              saveinputs('sdsmoothold', 'sdsmoothold.last', myparams, self.__globals__,scriptstr=scriptstr)
          tname = 'sdsmoothold'
          spaces = ' '*(18-len(tname))
          casalog.post('\n##########################################'+
                       '\n##### Begin Task: ' + tname + spaces + ' #####')
          if type(self.__call__.func_defaults) is NoneType:
              casalog.post(scriptstr[0]+'\n', 'INFO')
          else :
              casalog.post(scriptstr[1][1:]+'\n', 'INFO')
          result = sdsmoothold(infile, antenna, scanaverage, scanlist, field, iflist, pollist, kernel, kwidth, chanwidth, verify, outfile, outform, overwrite, plotlevel)
          casalog.post('##### End Task: ' + tname + '  ' + spaces + ' #####'+
                       '\n##########################################')

	except Exception, instance:
          if(self.__globals__.has_key('__rethrow_casa_exceptions') and self.__globals__['__rethrow_casa_exceptions']) :
             raise
          else :
             #print '**** Error **** ',instance
	     tname = 'sdsmoothold'
             casalog.post('An error occurred running task '+tname+'.', 'ERROR')
             pass

        gc.collect()
        return result
#
#
#
    def paramgui(self, useGlobals=True, ipython_globals=None):
        """
        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
        """
        import paramgui
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals

        if useGlobals:
	    if ipython_globals == None:
                myf=self.__globals__
            else:
                myf=ipython_globals

            paramgui.setGlobals(myf)
        else:
            paramgui.setGlobals({})

        paramgui.runTask('sdsmoothold', myf['_ip'])
        paramgui.setGlobals({})

#
#
#
    def defaults(self, param=None, ipython_globals=None, paramvalue=None, subparam=None):
	if not hasattr(self, "__globals__") or self.__globals__ == None :
           self.__globals__=sys._getframe(len(inspect.stack())-1).f_globals
        if ipython_globals == None:
            myf=self.__globals__
        else:
            myf=ipython_globals

        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['scanaverage']  = False
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['kernel']  = 'hanning'
        a['verify']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        a['kernel'] = {
                    0:{'value':'hanning'}, 
                    1:odict([{'value':'gaussian'}, {'kwidth':5}]), 
                    2:odict([{'value':'boxcar'}, {'kwidth':5}]), 
                    3:odict([{'value':'regrid'}, {'chanwidth':'5'}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
            if(paramvalue==None and subparam==None):
               if(a.has_key(param)):
                  return a[param]
               else:
                  return self.itsdefault(param)
            else:
               retval=a[param]
               if(type(a[param])==dict):
                  for k in range(len(a[param])):
                     valornotval='value'
                     if(a[param][k].has_key('notvalue')):
                        valornotval='notvalue'
                     if((a[param][k][valornotval])==paramvalue):
                        retval=a[param][k].copy()
                        retval.pop(valornotval)
                        if(subparam != None):
                           if(retval.has_key(subparam)):
                              retval=retval[subparam]
                           else:
                              retval=self.itsdefault(subparam)
		     else:
                        retval=self.itsdefault(subparam)
               return retval


#
#
    def check_params(self, param=None, value=None, ipython_globals=None):
      if ipython_globals == None:
          myf=self.__globals__
      else:
          myf=ipython_globals
#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value
#
#
    def description(self, key='sdsmoothold', subkey=None):
        desc={'sdsmoothold': 'ASAP SD task: do smoothing of spectra',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'scanaverage': 'average integs within scans (True,False) ',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'kernel': 'type of spectral smoothing',
               'kwidth': 'width of spectral kernel in channels',
               'chanwidth': 'width of regridded channels',
               'verify': 'interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]',
               'outfile': 'output file name',
               'outform': 'output file format (ASCII,MS,SDFITS,ASAP)',
               'overwrite': 'overwrite the output file if already exists',
               'plotlevel': 'plot results (0=none,1+=some,<0=hardcopy)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['scanaverage']  = False
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['kernel']  = 'hanning'
        a['kwidth']  = 5
        a['chanwidth']  = '5'
        a['verify']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        #a = sys._getframe(len(inspect.stack())-1).f_globals

        if self.parameters['kernel']  == 'gaussian':
            a['kwidth'] = 5

        if self.parameters['kernel']  == 'boxcar':
            a['kwidth'] = 5

        if self.parameters['kernel']  == 'regrid':
            a['chanwidth'] = '5'

        if a.has_key(paramname) :
	      return a[paramname]
sdsmoothold_cli = sdsmoothold_cli_()
