#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdsmoothold import sdsmoothold
from task_sdsmoothold import casalog

class sdsmoothold_pg_:
    __name__ = "sdsmoothold"

    def __init__(self) :
       self.__bases__ = (sdsmoothold_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,antenna=None,scanaverage=None,scanlist=None,field=None,iflist=None,pollist=None,kernel=None,kwidth=None,chanwidth=None,verify=None,outfile=None,outform=None,overwrite=None,plotlevel=None,):

        """ASAP SD task: do smoothing of spectra

	Keyword arguments:
        infile -- name of input SD dataset
        antenna -- antenna name or id (only effective for MS input). 
        scanaverage -- average integrations within scans
                options: (bool) True,False
                default: False
                example: if True, this happens in read-in
                         For GBT, set False!
        scanlist -- list of scan numbers to process
                default: [] (use all scans)
                example: [21,22,23,24]
                this selection is in addition to field, iflist, and pollist
        field -- selection string for selecting scans by name
                default: '' (no name selection)
                example: 'FLS3a*'
                this selection is in addition to scanlist, iflist,pollist
        iflist -- list of IF id numbers to select
                default: [] (use all IFs)
                example: [15]
                this selection is in addition to scanlist, field, and pollist
        pollist -- list of polarization id numbers to select
                default: [] (use all polarizations)
                example: [1]
                this selection is in addition to scanlist, field, and iflist
	kernel -- type of spectral smoothing
	        options: 'hanning','gaussian','boxcar','regrid'
	        default: 'hanning'

            >>>kernel expandable parameter
                 kwidth -- width of spectral smoothing kernel
	                 options: (int) in channels 
	                 default: 5
		         example: 5 or 10 seem to be popular for boxcar
		                  ignored for hanning (fixed at 5 chans)
                                  (0 will turn off gaussian or boxcar)
                 chanwidth -- channel width of regridded spectra
	                 default: '5' (in channels)
		         example: '500MHz', '0.2km/s'

        verify -- interactively verify the results of smoothing for each
                  spectrum.
                  When verify = True, for each input spectrum, spectra
                  before and after the smoothing are displayed in a plot
                  window. At the prompt there are four choices of action:
                  'Y' (accept the smoothing and continue to the next input
                  spectrum), 'N' (reject the smoothing and continue to the
                  next input spectrum), 'A' (accept the current smoothing
                  and continue non-interactively), and 'R' (reject the
                  current smoothing and exit from smoothing).
                  Note that when the smoothing is rejected by 'N' or 'R',
                  no smoothing is done to the spectrum/spectra. 
                options: (bool) True,False
                default: False
                Note: verification is not yet available for kernel='regrid'
	outfile -- Name of output ASAP format(scantable) file
	        default: '' (<infile>_sm)
	outform -- format of output file
                options: 'ASCII','SDFITS','MS','ASAP'
                default: 'ASAP'
                example: the ASAP format is easiest for further sd
                         processing; use MS for CASA imaging.
                         If ASCII, then will append some stuff to
                         the outfile name
        overwrite -- overwrite the output file if already exists
                options: (bool) True,False
                default: False
                WARNING: if outform='ASCII', this parameter is ignored
        plotlevel -- control for plotting of results
	        options: (int) 0=none, 1=some, 2=more, <0=hardcopy
		default: 0 (no plotting)
		example: plotlevel<0 as abs(plotlevel), e.g.
		         -1 => hardcopy of final plot (will be named
		         <outfile>_smspec.eps)


        DESCRIPTION:

	Task sdsmoothold performs smoothing of the single-dish spectra.
        Set plotlevel >= 1 to plot spectrum before and after smoothing.

        WARNING for the GBT raw SDFITS format data as input:
        SDtasks are able to handle GBT raw SDFITS format data since the 
        data filler is available. However, the functionality is not well 
        tested yet, so that there may be unknown bugs.  

  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdsmoothold'
        myf['taskname'] = 'sdsmoothold'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['scanaverage'] = scanaverage = myf['scanaverage']
            myparams['scanlist'] = scanlist = myf['scanlist']
            myparams['field'] = field = myf['field']
            myparams['iflist'] = iflist = myf['iflist']
            myparams['pollist'] = pollist = myf['pollist']
            myparams['kernel'] = kernel = myf['kernel']
            myparams['kwidth'] = kwidth = myf['kwidth']
            myparams['chanwidth'] = chanwidth = myf['chanwidth']
            myparams['verify'] = verify = myf['verify']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['outform'] = outform = myf['outform']
            myparams['overwrite'] = overwrite = myf['overwrite']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']

        if type(scanlist)==int: scanlist=[scanlist]
        if type(iflist)==int: iflist=[iflist]
        if type(pollist)==int: pollist=[pollist]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['antenna'] = antenna
        mytmp['scanaverage'] = scanaverage
        mytmp['scanlist'] = scanlist
        mytmp['field'] = field
        mytmp['iflist'] = iflist
        mytmp['pollist'] = pollist
        mytmp['kernel'] = kernel
        mytmp['kwidth'] = kwidth
        mytmp['chanwidth'] = chanwidth
        mytmp['verify'] = verify
        mytmp['outfile'] = outfile
        mytmp['outform'] = outform
        mytmp['overwrite'] = overwrite
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdsmoothold.xml')

        casalog.origin('sdsmoothold')
        if not trec.has_key('sdsmoothold') or not casac.utils().verify(mytmp, trec['sdsmoothold']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdsmoothold           #####')
          casalog.post('')
          result = sdsmoothold(infile, antenna, scanaverage, scanlist, field, iflist, pollist, kernel, kwidth, chanwidth, verify, outfile, outform, overwrite, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdsmoothold           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdsmoothold', 'sdsmoothold.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdsmoothold'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdsmoothold', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['antenna']  = 0
        a['scanaverage']  = False
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['kernel']  = 'hanning'
        a['verify']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        a['kernel'] = {
                    0:{'value':'hanning'}, 
                    1:odict([{'value':'gaussian'}, {'kwidth':5}]), 
                    2:odict([{'value':'boxcar'}, {'kwidth':5}]), 
                    3:odict([{'value':'regrid'}, {'chanwidth':'5'}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdsmoothold', subkey=None):
        desc={'sdsmoothold': 'ASAP SD task: do smoothing of spectra',
               'infile': 'name of input SD dataset',
               'antenna': 'antenna name or id (only effective for MS input)',
               'scanaverage': 'average integs within scans (True,False) ',
               'scanlist': 'list of scans to use (e.g. [1,2,3,4])',
               'field': 'string for selection by source name',
               'iflist': 'list of IF ids to select (e.g. [0,1])',
               'pollist': 'list of polarization ids to select (e.g. [0,1])',
               'kernel': 'type of spectral smoothing',
               'kwidth': 'width of spectral kernel in channels',
               'chanwidth': 'width of regridded channels',
               'verify': 'interactively verify the results of smoothing for each spectrum. [not available for kernel="regrid"]',
               'outfile': 'output file name',
               'outform': 'output file format (ASCII,MS,SDFITS,ASAP)',
               'overwrite': 'overwrite the output file if already exists',
               'plotlevel': 'plot results (0=none,1+=some,<0=hardcopy)',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['antenna']  = 0
        a['scanaverage']  = False
        a['scanlist']  = []
        a['field']  = ''
        a['iflist']  = []
        a['pollist']  = []
        a['kernel']  = 'hanning'
        a['kwidth']  = 5
        a['chanwidth']  = '5'
        a['verify']  = False
        a['outfile']  = ''
        a['outform']  = 'ASAP'
        a['overwrite']  = False
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdsmoothold_pg = sdsmoothold_pg_()
