#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_sdtpimaging import sdtpimaging
from task_sdtpimaging import casalog

class sdtpimaging_pg_:
    __name__ = "sdtpimaging"

    def __init__(self) :
       self.__bases__ = (sdtpimaging_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, infile=None,calmode=None,masklist=None,blpoly=None,backup=None,flaglist=None,antenna=None,spw=None,stokes=None,createimage=None,outfile=None,imsize=None,cell=None,phasecenter=None,ephemsrcname=None,pointingcolumn=None,gridfunction=None,plotlevel=None,):

        """SD task: do a simple calibration (baseline subtraction) and imaging for total power data 
Keyword arguments:
infile -- name of an input SD Measurementset
        example: 'm100.tp.ms'
calmode -- SD calibration mode (currently only baseline subtraction)
        options: 'baseline','none'
        default: 'none'
        example: choose mode 'none' if you have
                 already calibrated and want to do
                 plotting nd/or imaging 
    >>> calmode='baseline' expandable parameters
        masklist -- numbers of integrations from each edge of each scan
                    to be included for baseline fitting
                default: [] (no edge. should define positive number)
                example: [30,30] or [30] 
                         used first 30 rows and last 30 rows of each scan
                         for the baseline 
        blpoly -- polynomial order of the baseline fit
                default: (int) 1
                example: any number >=0
        backup -- set True to create backup of input data
                options: (bool) True, False
                default: True
flaglist -- list of scan numbers to flag (ranges can be accepted)  
        default: [] (use all scans)
        example: [[0,3],80]
                 flag the scan range [0,3] = [0,1,2,3] and scan 80 
antenna -- select data based on antenna names or IDs
        default: '' (use all antennas)
        example: antenna='0,1' (antenna ID 0 and 1)
                 antenna='DV01'
        WARNING: currently baseline subtraction properly 
                 only one of the antennas.
spw -- spectral window ID for imaging (should have only one channel)
        default: 0
        example: spw=11 (SPW ID 11)
stokes -- stokes parameters or polarization types to image
        default: '' (Stokes I)
        example: stokes='XX' (image plane of linear polarization, XX)
                 stokes='XXYY' (image cube with XX and YY image in each plane)
                 stokes='I' (Stokes I image = total intensity)
createimage -- do imaging? 
        options: (bool) True, False
        default: False 
    >>> createimage=True expandable parameters
        outfile -- name of output image
                default: ''
                example: 'mySDimage.im'
        imsize -- x and y image size in pixels, symmetric for single value
                default: [256,256]
                example: imsize=200 (equivalent to [200,200])
        cell -- x and y cell size. default unit arcmin
                default: '1.0arcmin'
                example: cell=['0.2arcmin, 0.2arcmin']
                         cell='0.2arcmin' (equivalent to example above)
        phasecenter -- image phase center: direction measure or field ID
                default: 0
                example: 'J2000 13h44m00 -17d02m00', 'AZEL -123d48m29 15d41m41'
        ephemsrcname -- ephemeris source name of moving source to use to
                        correct movements of source direction during
                        observation.
                default: ''
                         if the source name in the data matches one of the 
                         known solar objects by the system, this task 
                         automatically set the source name. 
                example: 'mars'
        pointingcolumn -- pointing data column to use
                options: 'direction', 'target', 'pointing_offset', 
                        'source_offset', 'encoder' 
                default: 'direction'
        gridfunction -- gridding function for imaging
                options: 'BOX' (Box-car), 'SF' (Spheroidal), 
                         'PB' (Primary-beam), 'GAUSS' (Gaussian),
                         'GJINC' (Gaussian*Jinc)
                default: 'BOX'
                example: 'SF'
plotlevel -- control for plotting of results
        options: (int) 0=none, 1=some, 2=more, <0=hardcopy
        default: 0 (no plotting)
        example: plotlevel<0 as abs(plotlevel), e.g.
                 -1: hardcopy plot 
                     (will be named <infile>_scans.eps)
                  1: plot raw data, calibrated data 
                     (for calmode='baseline)
                     plot raw or if exist calibrated data 
                     (for calmode='none')
                  2: plot raw data, progressively display baseline 
                     fitting for each scan, and final calibrated data 
                     (for calmode='baseline')  


-----------------
Gridding Kernel
-----------------
The parameter gridfunction sets gridding function (convolution kernel)
for imaging. Currently, the task supports 'BOX' (Box-car), 'SF' (Prolate
Spheroidal Wave Function), 'GAUSS' (Gaussian), 'GJINC' (Gaussian*Jinc),
where Jinc(x) = J_1(pi*x/c)/(pi*x/c) with a first order Bessel function
J_1, and 'PB' (Primary Beam). For 'PB', correct antenna informations
should be included in input file. 
Sub-parameters for convolution functions cannot be specified in this
task. To costomize your convolution function, please do imaging using
sdimaging task or imager tool.
  
        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'sdtpimaging'
        myf['taskname'] = 'sdtpimaging'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['infile'] = infile = myf['infile']
            myparams['calmode'] = calmode = myf['calmode']
            myparams['masklist'] = masklist = myf['masklist']
            myparams['blpoly'] = blpoly = myf['blpoly']
            myparams['backup'] = backup = myf['backup']
            myparams['flaglist'] = flaglist = myf['flaglist']
            myparams['antenna'] = antenna = myf['antenna']
            myparams['spw'] = spw = myf['spw']
            myparams['stokes'] = stokes = myf['stokes']
            myparams['createimage'] = createimage = myf['createimage']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['imsize'] = imsize = myf['imsize']
            myparams['cell'] = cell = myf['cell']
            myparams['phasecenter'] = phasecenter = myf['phasecenter']
            myparams['ephemsrcname'] = ephemsrcname = myf['ephemsrcname']
            myparams['pointingcolumn'] = pointingcolumn = myf['pointingcolumn']
            myparams['gridfunction'] = gridfunction = myf['gridfunction']
            myparams['plotlevel'] = plotlevel = myf['plotlevel']

        if type(masklist)==int: masklist=[masklist]
        if type(flaglist)==int: flaglist=[flaglist]
        if type(imsize)==int: imsize=[imsize]
        if type(cell)==float: cell=[cell]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['infile'] = infile
        mytmp['calmode'] = calmode
        mytmp['masklist'] = masklist
        mytmp['blpoly'] = blpoly
        mytmp['backup'] = backup
        mytmp['flaglist'] = flaglist
        mytmp['antenna'] = antenna
        mytmp['spw'] = spw
        mytmp['stokes'] = stokes
        mytmp['createimage'] = createimage
        mytmp['outfile'] = outfile
        mytmp['imsize'] = imsize
        if type(cell) == str :
           mytmp['cell'] = casac.quanta().quantity(cell)
        else :
           mytmp['cell'] = cell
        mytmp['phasecenter'] = phasecenter
        mytmp['ephemsrcname'] = ephemsrcname
        mytmp['pointingcolumn'] = pointingcolumn
        mytmp['gridfunction'] = gridfunction
        mytmp['plotlevel'] = plotlevel
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'sdtpimaging.xml')

        casalog.origin('sdtpimaging')
        if not trec.has_key('sdtpimaging') or not casac.utils().verify(mytmp, trec['sdtpimaging']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: sdtpimaging           #####')
          casalog.post('')
          result = sdtpimaging(infile, calmode, masklist, blpoly, backup, flaglist, antenna, spw, stokes, createimage, outfile, imsize, cell, phasecenter, ephemsrcname, pointingcolumn, gridfunction, plotlevel)
          casalog.post('')
          casalog.post('##### End Task: sdtpimaging           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('sdtpimaging', 'sdtpimaging.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'sdtpimaging'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('sdtpimaging', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['infile']  = ''
        a['calmode']  = 'none'
        a['flaglist']  = []
        a['antenna']  = ''
        a['spw']  = 0
        a['stokes']  = ''
        a['createimage']  = False
        a['plotlevel']  = 0

        a['calmode'] = {
                    0:{'value':'none'}, 
                    1:odict([{'value':'baseline'}, {'masklist':[]}, {'blpoly':1}, {'backup':True}])}
        a['createimage'] = {
                    0:{'value':False}, 
                    1:odict([{'value':True}, {'outfile':''}, {'imsize':[256, 256]}, {'cell':[1, 1]}, {'phasecenter':''}, {'ephemsrcname':''}, {'pointingcolumn':'direction'}, {'gridfunction':'BOX'}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='sdtpimaging', subkey=None):
        desc={'sdtpimaging': 'SD task: do a simple calibration (baseline subtraction) and imaging for total power data ',
               'infile': 'name of an input SD Measurementset (only MS is allowed for this task)',
               'calmode': 'SD calibration mode',
               'masklist': 'numbers of integrations from each edge of each scan to be included for baseline fitting, e.g. [30,30]',
               'blpoly': 'polynomial order of the baseline fit, e.g. 1',
               'backup': 'set True to create backup of input data [True, False]',
               'flaglist': 'list of scan numbers to flag, e.g. [[1,3], 80]',
               'antenna': 'select data by antenna names or IDs, e.g. \'PM03\'',
               'spw': 'spectral window ID for imaging, e.g. 11 (should have only one channel)',
               'stokes': 'stokes parameters or polarization types to image, e.g. \'XX\' (\'\'=Stokes I)',
               'createimage': 'do imaging? [True, False]',
               'outfile': 'name of output image',
               'imsize': 'x and y image size in pixels, e.g., [64,64]. Single value: same for both spatial axes',
               'cell': 'x and y cell size, (e.g., [\'8arcsec\',\'8arcsec\']. default unit arcmin.',
               'phasecenter': 'image center direction: position or field index, e.g., \'J2000 17:30:15.0 -25.30.00.0\'',
               'ephemsrcname': 'ephemeris source name, e.g. \'mars\'',
               'pointingcolumn': 'pointing data column to use',
               'gridfunction': 'gridding function for imaging (see description in help)',
               'plotlevel': 'plot results (0=none) see description in help',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['infile']  = ''
        a['calmode']  = 'none'
        a['masklist']  = []
        a['blpoly']  = 1
        a['backup']  = True
        a['flaglist']  = []
        a['antenna']  = ''
        a['spw']  = 0
        a['stokes']  = ''
        a['createimage']  = False
        a['outfile']  = ''
        a['imsize']  = [256, 256]
        a['cell']  = ['1.0arcmin', '1.0arcmin']
        a['phasecenter']  = ''
        a['ephemsrcname']  = ''
        a['pointingcolumn']  = 'direction'
        a['gridfunction']  = 'BOX'
        a['plotlevel']  = 0

        if a.has_key(paramname) :
	      return a[paramname]
sdtpimaging_pg = sdtpimaging_pg_()
