#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_slsearch import slsearch
from task_slsearch import casalog

class slsearch_pg_:
    __name__ = "slsearch"

    def __init__(self) :
       self.__bases__ = (slsearch_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, tablename=None,outfile=None,freqrange=None,species=None,reconly=None,chemnames=None,qns=None,intensity=None,smu2=None,loga=None,el=None,eu=None,rrlinclude=None,rrlonly=None,verbose=None,logfile=None,append=None,):

        """Search a spectral line table.

PARAMETER SUMMARY

tablename      Input spectral line table name to search. If not specified, use the default table in the system.
outfile        Results table name. Blank means do not write the table to disk.
freqrange      Frequency range in GHz.
species        Species to search for.
reconly        List only NRAO recommended frequencies.
chemnames      Chemical names to search for.
qns            Resolved quantum numbers to search for.
intensity      CDMS/JPL intensity range. -1 -> do not use an intensity range.
smu2           S*mu*mu range in Debye**2. -1 -> do not use an S*mu*mu range.
loga           log(A) (Einstein coefficient) range. -1 -> do not use a loga range.
el             Lower energy state range in Kelvin. -1 -> do not use an el range.
eu             Upper energy state range in Kelvin. -1 -> do not use an eu range.
rrlinclude     Include RRLs in the result set?
rrlonly        Include only RRLs in the result set?
verbose        List result set to logger (and optionally logfile)?
logfile        List result set to this logfile (only used if verbose=True).
append         If true, append to logfile if it already exists, if false overwrite logfile it it exists. Only used if verbose=True and logfile not blank.

    Search the specfied spectral line table. The results table can be written to disk by specifying its name in the outfile parameter.
    If outfile is not specified (ie outfile=""), no table is created. Because Splatalogue does not have values for intensity, smu2,
    loga, eu, and el for radio recombination lines (rrls), one must specify to include RRLs in the specified frequency range in the
    output. In this case, RRLs will be included ignoring any filters on intensity, smu2, loga, eu, and el. One can also specify to
    list only RRLs. One can specify to list the search results to the logger via the verbose parameter. If verbose is False, no
    logger output is listed. If verbose=True, one can also specify that the results be listed to a logfile and if this file already
    exists, one can specify that the results be appended to it or to overwrite it with the results.
    
    # put search results in a table but do not list to the logger
    slsearch("myspectrallines.tbl", verbose=False)


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'slsearch'
        myf['taskname'] = 'slsearch'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['tablename'] = tablename = myf['tablename']
            myparams['outfile'] = outfile = myf['outfile']
            myparams['freqrange'] = freqrange = myf['freqrange']
            myparams['species'] = species = myf['species']
            myparams['reconly'] = reconly = myf['reconly']
            myparams['chemnames'] = chemnames = myf['chemnames']
            myparams['qns'] = qns = myf['qns']
            myparams['intensity'] = intensity = myf['intensity']
            myparams['smu2'] = smu2 = myf['smu2']
            myparams['loga'] = loga = myf['loga']
            myparams['el'] = el = myf['el']
            myparams['eu'] = eu = myf['eu']
            myparams['rrlinclude'] = rrlinclude = myf['rrlinclude']
            myparams['rrlonly'] = rrlonly = myf['rrlonly']
            myparams['verbose'] = verbose = myf['verbose']
            myparams['logfile'] = logfile = myf['logfile']
            myparams['append'] = append = myf['append']

        if type(freqrange)==float: freqrange=[freqrange]
        if type(species)==str: species=[species]
        if type(chemnames)==str: chemnames=[chemnames]
        if type(qns)==str: qns=[qns]
        if type(intensity)==float: intensity=[intensity]
        if type(smu2)==float: smu2=[smu2]
        if type(loga)==float: loga=[loga]
        if type(el)==float: el=[el]
        if type(eu)==float: eu=[eu]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['tablename'] = tablename
        mytmp['outfile'] = outfile
        mytmp['freqrange'] = freqrange
        mytmp['species'] = species
        mytmp['reconly'] = reconly
        mytmp['chemnames'] = chemnames
        mytmp['qns'] = qns
        mytmp['intensity'] = intensity
        mytmp['smu2'] = smu2
        mytmp['loga'] = loga
        mytmp['el'] = el
        mytmp['eu'] = eu
        mytmp['rrlinclude'] = rrlinclude
        mytmp['rrlonly'] = rrlonly
        mytmp['verbose'] = verbose
        mytmp['logfile'] = logfile
        mytmp['append'] = append
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'slsearch.xml')

        casalog.origin('slsearch')
        if not trec.has_key('slsearch') or not casac.utils().verify(mytmp, trec['slsearch']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: slsearch           #####')
          casalog.post('')
          result = slsearch(tablename, outfile, freqrange, species, reconly, chemnames, qns, intensity, smu2, loga, el, eu, rrlinclude, rrlonly, verbose, logfile, append)
          casalog.post('')
          casalog.post('##### End Task: slsearch           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('slsearch', 'slsearch.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'slsearch'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('slsearch', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['tablename']  = ''
        a['outfile']  = ''
        a['freqrange']  = [84,90]
        a['species']  = ['']
        a['reconly']  = False
        a['chemnames']  = ['']
        a['qns']  = ['']
        a['rrlinclude']  = True
        a['rrlonly']  = False
        a['verbose']  = False

        a['verbose'] = {
                    0:odict([{'value':True}, {'logfile':""}, {'append':True}])}
        a['rrlonly'] = {
                    0:odict([{'value':False}, {'intensity':-1}, {'smu2':-1}, {'loga':-1}, {'eu':-1}, {'el':-1}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='slsearch', subkey=None):
        desc={'slsearch': 'Search a spectral line table.',
               'tablename': 'Input spectral line table name to search. If not specified, use the default table in the system.',
               'outfile': 'Results table name. Blank means do not write the table to disk.',
               'freqrange': 'Frequency range in GHz.',
               'species': 'Species to search for.',
               'reconly': 'List only NRAO recommended frequencies.',
               'chemnames': 'Chemical names to search for.',
               'qns': 'Resolved quantum numbers to search for.',
               'intensity': 'CDMS/JPL intensity range. -1 -> do not use an intensity range.',
               'smu2': 'S*mu*mu range in Debye**2. -1 -> do not use an S*mu*mu range.',
               'loga': 'log(A) (Einstein coefficient) range. -1 -> do not use a loga range.',
               'el': 'Lower energy state range in Kelvin. -1 -> do not use an el range.',
               'eu': 'Upper energy state range in Kelvin. -1 -> do not use an eu range.',
               'rrlinclude': 'Include RRLs in the result set?',
               'rrlonly': 'Include only RRLs in the result set?',
               'verbose': 'List result set to logger (and optionally logfile)?',
               'logfile': 'List result set to this logfile (only used if verbose=True).',
               'append': 'If true, append to logfile if it already exists, if false overwrite logfile it it exists. Only used if verbose=True and logfile not blank.',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['tablename']  = ''
        a['outfile']  = ''
        a['freqrange']  = [84,90]
        a['species']  = ['']
        a['reconly']  = False
        a['chemnames']  = ['']
        a['qns']  = ['']
        a['intensity']  = [-1]
        a['smu2']  = [-1]
        a['loga']  = [-1]
        a['el']  = [-1]
        a['eu']  = [-1]
        a['rrlinclude']  = True
        a['rrlonly']  = False
        a['verbose']  = False
        a['logfile']  = '""'
        a['append']  = False

        if a.has_key(paramname) :
	      return a[paramname]
slsearch_pg = slsearch_pg_()
