#
# This file was generated using xslt from its XML file
#
# Copyright 2008, Associated Universities Inc., Washington DC
#
import sys
import os
from casac import *
import string
import time
import inspect
import gc
import numpy
from odict import odict
from task_vishead import vishead
from task_vishead import casalog

class vishead_pg_:
    __name__ = "vishead"

    def __init__(self) :
       self.__bases__ = (vishead_pg_,)
       self.__doc__ = self.__call__.__doc__


    def __call__(self, vis=None,mode=None,listitems=None,hdkey=None,hdindex=None,hdvalue=None,):

        """List, summary, get, and put metadata in a measurement set

        This task allows the user to manipulate some meta-data parameters in a
        measurement set.  The mode='list' shows those keywords that are
        presently implemented, with their values.  The contents associated
        with the keywords can be obtained (get) and changed (put).

        The modes that are available are:

           list    --- List all keywords that are recognized, and list the
                       value(s) for each.  Only these keywords can be obtained
                       (get) or changed (put) 
           summary --- Equivalent to running taskname='listobs'; verbose=F
           get     --- Get the specified keyword value(s) from the ms
           put     --- Put the specified keyword value(s) into the ms

        Parameters currently implemented are (June 1, 2009):

           cal_grp              
           field                 Field names
           fld_code              Field Observing codes
           freq_group_name       
           log                   
           observer              Observer name
           project               Project name
           ptcs                  Phase tracking centers for each field
           release_date          
           schedule
           schedule_type
           spw_name              Spectral parameters?
           source_name           Source Names (=Field Names?)
           telescope             Telescope Name

        Keyword arguments:

        vis  --- Name of input visibility file
                  default: none, example: vis='my.ms'

        mode --- Mode of operation for vishead
                  default = 'list'; example: mode='get'

        hdkey--- keyword to get or put from the ms (used in get/put mode only)
                  ex: hdkey='telescope'

        hdindex--- index (counting from 0) if keyword is an array (used in get/put
                  mode only)
                  ex: hdindex='2'; hdindex=''->put/get full array; 

        hdvalue   --- value to be put in the MS (used in put mode only)
                  ex: hdvalue=array(['MyTelescope'])




        Examples:

           To transfer the parameters to useful python items requires some care.

           taskname = 'vishead'
           default() 
           vis = '3C84C.ms'
           mode = 'get'
                      
           to get a field name (string),
              hdkey = 'field'; hdindex = '2'; hdvalue=vishead(); 
                       print hdvalue[0] = the name for field='2'                    
                
           to get an phase center (number)
              hdkey = 'ptcs'; hdindex = '1'; hdvalue = vishead();
                       hdvalue[0][0] gives the ra, hdvalue[0][1] gives the dec in field '1'


           taskname = 'vishead'
           default() 
           vis = '3C84C.ms'
           mode = 'put'

          To change a string,

              hdkey = 'field'; hdindex = '2'; hdvalue = 'junk'; vishead()
                   field='2' is renamed 'junk'

          To change a number, (egs. ra of field=1 to 0.5 radian)
              is too complicated to figure out!
             


        """
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        myf['__last_task'] = 'vishead'
        myf['taskname'] = 'vishead'
        ###
        myf['update_params'](func=myf['taskname'],printtext=False)
        ###
        ###
        #Handle globals or user over-ride of arguments
        #
	function_signature_defaults=dict(zip(self.__call__.func_code.co_varnames,self.__call__.func_defaults))
	useLocalDefaults = False

        for item in function_signature_defaults.iteritems():
                key,val = item
                keyVal = eval(key)
                if (keyVal == None):
                        #user hasn't set it - use global/default
                        pass
                else:
                        #user has set it - use over-ride
			if (key != 'self') :
			   useLocalDefaults = True
                        #myf[key]=keyVal

	myparams = {}
	if useLocalDefaults :
	   for item in function_signature_defaults.iteritems():
	       key,val = item
	       keyVal = eval(key)
	       exec('myparams[key] = keyVal')
	       if (keyVal == None):
	           exec('myparams[key] = '+ key + ' = self.itsdefault(key)')
		   keyVal = eval(key)
		   if(type(keyVal) == dict) :
		      exec('myparams[key] = ' + key + ' = keyVal[len(keyVal)-1][\'value\']')

	else :
            uselessvariable = None 
            myparams['vis'] = vis = myf['vis']
            myparams['mode'] = mode = myf['mode']
            myparams['listitems'] = listitems = myf['listitems']
            myparams['hdkey'] = hdkey = myf['hdkey']
            myparams['hdindex'] = hdindex = myf['hdindex']
            myparams['hdvalue'] = hdvalue = myf['hdvalue']

        if type(listitems)==str: listitems=[listitems]

	result = None

#
#    The following is work around to avoid a bug with current python translation
#
        mytmp = {}

        mytmp['vis'] = vis
        mytmp['mode'] = mode
        mytmp['listitems'] = listitems
        mytmp['hdkey'] = hdkey
        mytmp['hdindex'] = hdindex
        mytmp['hdvalue'] = hdvalue
	pathname='file:///'+os.environ.get('CASAPATH').split()[0]+'/xml/'
        trec = casac.utils().torecord(pathname+'vishead.xml')

        casalog.origin('vishead')
        if not trec.has_key('vishead') or not casac.utils().verify(mytmp, trec['vishead']) :
	    return False


	try :
          casalog.post('')
          casalog.post('##########################################')
          casalog.post('##### Begin Task: vishead           #####')
          casalog.post('')
          result = vishead(vis, mode, listitems, hdkey, hdindex, hdvalue)
          casalog.post('')
          casalog.post('##### End Task: vishead           #####')
          casalog.post('##########################################')


# saveinputs for individule engine has no use
# saveinputs should alos be removed from casa_in_py.py
#
#
#          saveinputs = myf['saveinputs']
#          saveinputs('vishead', 'vishead.last', myparams)
#
#
	except Exception, instance:
	  if(myf.has_key('__rethrow_casa_exceptions') and myf['__rethrow_casa_exceptions']) :
	     raise
	  else:
	     tname = 'vishead'
	     casalog.post('An error occurred running task '+tname+'.', 'ERROR')
	     pass

        gc.collect()
        return result
#
#
##
#    def paramgui(self, useGlobals=True):
#        """
#        Opens a parameter GUI for this task.  If useGlobals is true, then any relevant global parameter settings are used.
#        """
#        import paramgui
#
#        a=inspect.stack()
#        stacklevel=0
#        for k in range(len(a)):
#          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
#            stacklevel=k
#            break
#        myf = sys._getframe(stacklevel).f_globals
#
#        if useGlobals:
#            paramgui.setGlobals(myf)
#        else:
#            paramgui.setGlobals({})
#
#        paramgui.runTask('vishead', myf['_ip'])
#        paramgui.setGlobals({})
#
#
#
#
    def defaults(self, param=None):
        a=inspect.stack()
        stacklevel=0
        for k in range(len(a)):
          if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
                stacklevel=k
                break
        myf=sys._getframe(stacklevel).f_globals
        a = odict()
        a['vis']  = ''
        a['mode']  = 'summary'

        a['mode'] = {
                    0:odict([{'value':'list'}, {'listitems':['telescope', 'observer', 'project', 'field', 'freq_group_name', 'spw_name', 'schedule', 'schedule_type', 'release_date']}]), 
                    1:{'value':'summary'}, 
                    2:odict([{'value':'get'}, {'hdkey':''}, {'hdindex':''}]), 
                    3:odict([{'value':'put'}, {'hdkey':''}, {'hdindex':''}, {'hdvalue':''}])}

### This function sets the default values but also will return the list of
### parameters or the default value of a given parameter
        if(param == None):
                myf['__set_default_parameters'](a)
        elif(param == 'paramkeys'):
                return a.keys()
        else:
	        if(a.has_key(param)):
		   #if(type(a[param]) == dict) :
		   #   return a[param][len(a[param])-1]['value']
	   	   #else :
		      return a[param]


#
#
    def check_params(self, param=None, value=None):
      a=inspect.stack() 
      stacklevel=0
      for k in range(len(a)):
        if (string.find(a[k][1], 'ipython console') > 0) or (string.find(a[k][1], '<string>') >= 0):
	    stacklevel=k
	    break
      myf=sys._getframe(stacklevel).f_globals

#      print 'param:', param, 'value:', value
      try :
         if str(type(value)) != "<type 'instance'>" :
            value0 = value
            value = myf['cu'].expandparam(param, value)
            matchtype = False
            if(type(value) == numpy.ndarray):
               if(type(value) == type(value0)):
                  myf[param] = value.tolist()
               else:
                  #print 'value:', value, 'value0:', value0
                  #print 'type(value):', type(value), 'type(value0):', type(value0)
                  myf[param] = value0
                  if type(value0) != list :
                     matchtype = True
            else :
               myf[param] = value
            value = myf['cu'].verifyparam({param:value})
            if matchtype:
               value = False
      except Exception, instance:
         #ignore the exception and just return it unchecked
         myf[param] = value
      return value

#
#
    def description(self, key='vishead', subkey=None):
        desc={'vishead': 'List, summary, get, and put metadata in a measurement set',
               'vis': 'Name of input visibility file',
               'mode': 'options: list, summary, get, put',
               'listitems': 'items to list ([] for all)',
               'hdkey': 'keyword to get/put',
               'hdindex': 'keyword index to get/put, counting from zero. ''==>all',
               'hdvalue': 'value of hdkey',

              }

#
# Set subfields defaults if needed
#

        if(desc.has_key(key)) :
           return desc[key]

    def itsdefault(self, paramname) :
        a = {}
        a['vis']  = ''
        a['mode']  = 'summary'
        a['listitems']  = ['telescope', 'observer', 'project', 'field', 'freq_group_name', 'spw_name', 'schedule', 'schedule_type', 'release_date']
        a['hdkey']  = ''
        a['hdindex']  = ''
        a['hdvalue']  = ''

        if a.has_key(paramname) :
	      return a[paramname]
vishead_pg = vishead_pg_()
